/********************************************************************************
** Form generated from reading UI file 'EditTableDialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EDITTABLEDIALOG_H
#define UI_EDITTABLEDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "sqltextedit.h"

QT_BEGIN_NAMESPACE

class Ui_EditTableDialog
{
public:
    QAction *actionAddPrimaryKey;
    QAction *actionAddUniqueConstraint;
    QVBoxLayout *verticalLayout_2;
    QGroupBox *groupTable;
    QVBoxLayout *verticalLayout_3;
    QLineEdit *editTableName;
    QToolButton *buttonMore;
    QWidget *widgetExtension;
    QFormLayout *formLayout;
    QLabel *label;
    QComboBox *comboSchema;
    QLabel *label_1;
    QCheckBox *checkWithoutRowid;
    QLabel *label_2;
    QComboBox *comboOnConflict;
    QLabel *label_3;
    QCheckBox *checkStrict;
    QSplitter *splitter;
    QTabWidget *groupDefinition;
    QWidget *groupFieldsPage1;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QToolButton *addFieldButton;
    QToolButton *removeFieldButton;
    QToolButton *buttonMoveTop;
    QToolButton *buttonMoveUp;
    QToolButton *buttonMoveDown;
    QToolButton *buttonMoveBottom;
    QSpacerItem *horizontalSpacer;
    QTreeWidget *treeWidget;
    QWidget *tab;
    QVBoxLayout *verticalLayout_4;
    QHBoxLayout *horizontalLayout_2;
    QToolButton *buttonAddIndexConstraint;
    QToolButton *buttonRemoveIndexConstraint;
    QSpacerItem *horizontalSpacer_2;
    QTableWidget *tableIndexConstraints;
    QWidget *tab_2;
    QVBoxLayout *verticalLayout_5;
    QHBoxLayout *horizontalLayout_3;
    QToolButton *buttonAddForeignKey;
    QToolButton *buttonRemoveForeignKey;
    QSpacerItem *horizontalSpacer_3;
    QTableWidget *tableForeignKeys;
    QWidget *tab_3;
    QVBoxLayout *verticalLayout_6;
    QHBoxLayout *horizontalLayout_4;
    QToolButton *buttonAddCheckConstraint;
    QToolButton *buttonRemoveCheckConstraint;
    QSpacerItem *horizontalSpacer_4;
    QTableWidget *tableCheckConstraints;
    SqlTextEdit *sqlTextEdit;
    QLabel *labelEditWarning;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *EditTableDialog)
    {
        if (EditTableDialog->objectName().isEmpty())
            EditTableDialog->setObjectName(QString::fromUtf8("EditTableDialog"));
        EditTableDialog->resize(800, 600);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/icons/table"), QSize(), QIcon::Normal, QIcon::Off);
        EditTableDialog->setWindowIcon(icon);
        EditTableDialog->setSizeGripEnabled(true);
        actionAddPrimaryKey = new QAction(EditTableDialog);
        actionAddPrimaryKey->setObjectName(QString::fromUtf8("actionAddPrimaryKey"));
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/icons/field_key"), QSize(), QIcon::Normal, QIcon::Off);
        actionAddPrimaryKey->setIcon(icon1);
        actionAddUniqueConstraint = new QAction(EditTableDialog);
        actionAddUniqueConstraint->setObjectName(QString::fromUtf8("actionAddUniqueConstraint"));
        verticalLayout_2 = new QVBoxLayout(EditTableDialog);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        groupTable = new QGroupBox(EditTableDialog);
        groupTable->setObjectName(QString::fromUtf8("groupTable"));
        verticalLayout_3 = new QVBoxLayout(groupTable);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        editTableName = new QLineEdit(groupTable);
        editTableName->setObjectName(QString::fromUtf8("editTableName"));
        QFont font;
        font.setBold(true);
        font.setWeight(75);
        editTableName->setFont(font);

        verticalLayout_3->addWidget(editTableName);

        buttonMore = new QToolButton(groupTable);
        buttonMore->setObjectName(QString::fromUtf8("buttonMore"));
        buttonMore->setCheckable(true);
        buttonMore->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
        buttonMore->setArrowType(Qt::DownArrow);

        verticalLayout_3->addWidget(buttonMore);

        widgetExtension = new QWidget(groupTable);
        widgetExtension->setObjectName(QString::fromUtf8("widgetExtension"));
        formLayout = new QFormLayout(widgetExtension);
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        label = new QLabel(widgetExtension);
        label->setObjectName(QString::fromUtf8("label"));

        formLayout->setWidget(0, QFormLayout::LabelRole, label);

        comboSchema = new QComboBox(widgetExtension);
        comboSchema->setObjectName(QString::fromUtf8("comboSchema"));

        formLayout->setWidget(0, QFormLayout::FieldRole, comboSchema);

        label_1 = new QLabel(widgetExtension);
        label_1->setObjectName(QString::fromUtf8("label_1"));

        formLayout->setWidget(1, QFormLayout::LabelRole, label_1);

        checkWithoutRowid = new QCheckBox(widgetExtension);
        checkWithoutRowid->setObjectName(QString::fromUtf8("checkWithoutRowid"));

        formLayout->setWidget(1, QFormLayout::FieldRole, checkWithoutRowid);

        label_2 = new QLabel(widgetExtension);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        formLayout->setWidget(3, QFormLayout::LabelRole, label_2);

        comboOnConflict = new QComboBox(widgetExtension);
        comboOnConflict->addItem(QString::fromUtf8(""));
        comboOnConflict->addItem(QString::fromUtf8("ROLLBACK"));
        comboOnConflict->addItem(QString::fromUtf8("ABORT"));
        comboOnConflict->addItem(QString::fromUtf8("FAIL"));
        comboOnConflict->addItem(QString::fromUtf8("IGNORE"));
        comboOnConflict->addItem(QString::fromUtf8("REPLACE"));
        comboOnConflict->setObjectName(QString::fromUtf8("comboOnConflict"));

        formLayout->setWidget(3, QFormLayout::FieldRole, comboOnConflict);

        label_3 = new QLabel(widgetExtension);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        formLayout->setWidget(2, QFormLayout::LabelRole, label_3);

        checkStrict = new QCheckBox(widgetExtension);
        checkStrict->setObjectName(QString::fromUtf8("checkStrict"));

        formLayout->setWidget(2, QFormLayout::FieldRole, checkStrict);


        verticalLayout_3->addWidget(widgetExtension);


        verticalLayout_2->addWidget(groupTable);

        splitter = new QSplitter(EditTableDialog);
        splitter->setObjectName(QString::fromUtf8("splitter"));
        splitter->setOrientation(Qt::Vertical);
        groupDefinition = new QTabWidget(splitter);
        groupDefinition->setObjectName(QString::fromUtf8("groupDefinition"));
        groupFieldsPage1 = new QWidget();
        groupFieldsPage1->setObjectName(QString::fromUtf8("groupFieldsPage1"));
        verticalLayout = new QVBoxLayout(groupFieldsPage1);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        addFieldButton = new QToolButton(groupFieldsPage1);
        addFieldButton->setObjectName(QString::fromUtf8("addFieldButton"));
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/icons/field_add"), QSize(), QIcon::Normal, QIcon::Off);
        addFieldButton->setIcon(icon2);
        addFieldButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
        addFieldButton->setAutoRaise(true);

        horizontalLayout->addWidget(addFieldButton);

        removeFieldButton = new QToolButton(groupFieldsPage1);
        removeFieldButton->setObjectName(QString::fromUtf8("removeFieldButton"));
        removeFieldButton->setEnabled(false);
        QIcon icon3;
        icon3.addFile(QString::fromUtf8(":/icons/field_delete"), QSize(), QIcon::Normal, QIcon::Off);
        removeFieldButton->setIcon(icon3);
        removeFieldButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
        removeFieldButton->setAutoRaise(true);

        horizontalLayout->addWidget(removeFieldButton);

        buttonMoveTop = new QToolButton(groupFieldsPage1);
        buttonMoveTop->setObjectName(QString::fromUtf8("buttonMoveTop"));
        buttonMoveTop->setEnabled(false);
        QIcon icon4;
        icon4.addFile(QString::fromUtf8(":/icons/arrow_top"), QSize(), QIcon::Normal, QIcon::Off);
        buttonMoveTop->setIcon(icon4);
        buttonMoveTop->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
        buttonMoveTop->setAutoRaise(true);
        buttonMoveTop->setArrowType(Qt::NoArrow);

        horizontalLayout->addWidget(buttonMoveTop);

        buttonMoveUp = new QToolButton(groupFieldsPage1);
        buttonMoveUp->setObjectName(QString::fromUtf8("buttonMoveUp"));
        buttonMoveUp->setEnabled(false);
        QIcon icon5;
        icon5.addFile(QString::fromUtf8(":/icons/up"), QSize(), QIcon::Normal, QIcon::Off);
        buttonMoveUp->setIcon(icon5);
        buttonMoveUp->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
        buttonMoveUp->setAutoRaise(true);

        horizontalLayout->addWidget(buttonMoveUp);

        buttonMoveDown = new QToolButton(groupFieldsPage1);
        buttonMoveDown->setObjectName(QString::fromUtf8("buttonMoveDown"));
        buttonMoveDown->setEnabled(false);
        QIcon icon6;
        icon6.addFile(QString::fromUtf8(":/icons/down"), QSize(), QIcon::Normal, QIcon::Off);
        buttonMoveDown->setIcon(icon6);
        buttonMoveDown->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
        buttonMoveDown->setAutoRaise(true);

        horizontalLayout->addWidget(buttonMoveDown);

        buttonMoveBottom = new QToolButton(groupFieldsPage1);
        buttonMoveBottom->setObjectName(QString::fromUtf8("buttonMoveBottom"));
        buttonMoveBottom->setEnabled(false);
        QIcon icon7;
        icon7.addFile(QString::fromUtf8(":/icons/arrow_bottom"), QSize(), QIcon::Normal, QIcon::Off);
        buttonMoveBottom->setIcon(icon7);
        buttonMoveBottom->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
        buttonMoveBottom->setAutoRaise(true);

        horizontalLayout->addWidget(buttonMoveBottom);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout);

        treeWidget = new QTreeWidget(groupFieldsPage1);
        treeWidget->setObjectName(QString::fromUtf8("treeWidget"));
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(treeWidget->sizePolicy().hasHeightForWidth());
        treeWidget->setSizePolicy(sizePolicy);
        treeWidget->setMinimumSize(QSize(0, 140));
        treeWidget->setEditTriggers(QAbstractItemView::AnyKeyPressed|QAbstractItemView::DoubleClicked|QAbstractItemView::EditKeyPressed|QAbstractItemView::SelectedClicked);
        treeWidget->setAlternatingRowColors(true);
        treeWidget->setSelectionBehavior(QAbstractItemView::SelectItems);
        treeWidget->setRootIsDecorated(false);

        verticalLayout->addWidget(treeWidget);

        groupDefinition->addTab(groupFieldsPage1, QString());
        tab = new QWidget();
        tab->setObjectName(QString::fromUtf8("tab"));
        verticalLayout_4 = new QVBoxLayout(tab);
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        buttonAddIndexConstraint = new QToolButton(tab);
        buttonAddIndexConstraint->setObjectName(QString::fromUtf8("buttonAddIndexConstraint"));
        buttonAddIndexConstraint->setIcon(icon2);
        buttonAddIndexConstraint->setPopupMode(QToolButton::InstantPopup);
        buttonAddIndexConstraint->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
        buttonAddIndexConstraint->setAutoRaise(true);

        horizontalLayout_2->addWidget(buttonAddIndexConstraint);

        buttonRemoveIndexConstraint = new QToolButton(tab);
        buttonRemoveIndexConstraint->setObjectName(QString::fromUtf8("buttonRemoveIndexConstraint"));
        buttonRemoveIndexConstraint->setEnabled(false);
        buttonRemoveIndexConstraint->setIcon(icon3);
        buttonRemoveIndexConstraint->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
        buttonRemoveIndexConstraint->setAutoRaise(true);

        horizontalLayout_2->addWidget(buttonRemoveIndexConstraint);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);


        verticalLayout_4->addLayout(horizontalLayout_2);

        tableIndexConstraints = new QTableWidget(tab);
        if (tableIndexConstraints->columnCount() < 4)
            tableIndexConstraints->setColumnCount(4);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        tableIndexConstraints->setHorizontalHeaderItem(0, __qtablewidgetitem);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        tableIndexConstraints->setHorizontalHeaderItem(1, __qtablewidgetitem1);
        QTableWidgetItem *__qtablewidgetitem2 = new QTableWidgetItem();
        tableIndexConstraints->setHorizontalHeaderItem(2, __qtablewidgetitem2);
        QTableWidgetItem *__qtablewidgetitem3 = new QTableWidgetItem();
        tableIndexConstraints->setHorizontalHeaderItem(3, __qtablewidgetitem3);
        tableIndexConstraints->setObjectName(QString::fromUtf8("tableIndexConstraints"));
        tableIndexConstraints->setAlternatingRowColors(true);
        tableIndexConstraints->setHorizontalScrollMode(QAbstractItemView::ScrollPerPixel);

        verticalLayout_4->addWidget(tableIndexConstraints);

        groupDefinition->addTab(tab, QString());
        tab_2 = new QWidget();
        tab_2->setObjectName(QString::fromUtf8("tab_2"));
        verticalLayout_5 = new QVBoxLayout(tab_2);
        verticalLayout_5->setObjectName(QString::fromUtf8("verticalLayout_5"));
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        buttonAddForeignKey = new QToolButton(tab_2);
        buttonAddForeignKey->setObjectName(QString::fromUtf8("buttonAddForeignKey"));
        QIcon icon8;
        icon8.addFile(QString::fromUtf8(":/icons/index_create"), QSize(), QIcon::Normal, QIcon::Off);
        buttonAddForeignKey->setIcon(icon8);
        buttonAddForeignKey->setPopupMode(QToolButton::InstantPopup);
        buttonAddForeignKey->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
        buttonAddForeignKey->setAutoRaise(true);

        horizontalLayout_3->addWidget(buttonAddForeignKey);

        buttonRemoveForeignKey = new QToolButton(tab_2);
        buttonRemoveForeignKey->setObjectName(QString::fromUtf8("buttonRemoveForeignKey"));
        buttonRemoveForeignKey->setEnabled(false);
        QIcon icon9;
        icon9.addFile(QString::fromUtf8(":/icons/index_delete"), QSize(), QIcon::Normal, QIcon::Off);
        buttonRemoveForeignKey->setIcon(icon9);
        buttonRemoveForeignKey->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
        buttonRemoveForeignKey->setAutoRaise(true);

        horizontalLayout_3->addWidget(buttonRemoveForeignKey);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_3);


        verticalLayout_5->addLayout(horizontalLayout_3);

        tableForeignKeys = new QTableWidget(tab_2);
        if (tableForeignKeys->columnCount() < 4)
            tableForeignKeys->setColumnCount(4);
        QTableWidgetItem *__qtablewidgetitem4 = new QTableWidgetItem();
        tableForeignKeys->setHorizontalHeaderItem(0, __qtablewidgetitem4);
        QTableWidgetItem *__qtablewidgetitem5 = new QTableWidgetItem();
        tableForeignKeys->setHorizontalHeaderItem(1, __qtablewidgetitem5);
        QTableWidgetItem *__qtablewidgetitem6 = new QTableWidgetItem();
        tableForeignKeys->setHorizontalHeaderItem(2, __qtablewidgetitem6);
        QTableWidgetItem *__qtablewidgetitem7 = new QTableWidgetItem();
        tableForeignKeys->setHorizontalHeaderItem(3, __qtablewidgetitem7);
        tableForeignKeys->setObjectName(QString::fromUtf8("tableForeignKeys"));
        tableForeignKeys->setAlternatingRowColors(true);
        tableForeignKeys->setHorizontalScrollMode(QAbstractItemView::ScrollPerPixel);

        verticalLayout_5->addWidget(tableForeignKeys);

        groupDefinition->addTab(tab_2, QString());
        tab_3 = new QWidget();
        tab_3->setObjectName(QString::fromUtf8("tab_3"));
        verticalLayout_6 = new QVBoxLayout(tab_3);
        verticalLayout_6->setObjectName(QString::fromUtf8("verticalLayout_6"));
        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName(QString::fromUtf8("horizontalLayout_4"));
        buttonAddCheckConstraint = new QToolButton(tab_3);
        buttonAddCheckConstraint->setObjectName(QString::fromUtf8("buttonAddCheckConstraint"));
        buttonAddCheckConstraint->setPopupMode(QToolButton::InstantPopup);
        buttonAddCheckConstraint->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
        buttonAddCheckConstraint->setAutoRaise(true);

        horizontalLayout_4->addWidget(buttonAddCheckConstraint);

        buttonRemoveCheckConstraint = new QToolButton(tab_3);
        buttonRemoveCheckConstraint->setObjectName(QString::fromUtf8("buttonRemoveCheckConstraint"));
        buttonRemoveCheckConstraint->setEnabled(false);
        buttonRemoveCheckConstraint->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
        buttonRemoveCheckConstraint->setAutoRaise(true);

        horizontalLayout_4->addWidget(buttonRemoveCheckConstraint);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_4);


        verticalLayout_6->addLayout(horizontalLayout_4);

        tableCheckConstraints = new QTableWidget(tab_3);
        if (tableCheckConstraints->columnCount() < 3)
            tableCheckConstraints->setColumnCount(3);
        QTableWidgetItem *__qtablewidgetitem8 = new QTableWidgetItem();
        tableCheckConstraints->setHorizontalHeaderItem(0, __qtablewidgetitem8);
        QTableWidgetItem *__qtablewidgetitem9 = new QTableWidgetItem();
        tableCheckConstraints->setHorizontalHeaderItem(1, __qtablewidgetitem9);
        QTableWidgetItem *__qtablewidgetitem10 = new QTableWidgetItem();
        tableCheckConstraints->setHorizontalHeaderItem(2, __qtablewidgetitem10);
        tableCheckConstraints->setObjectName(QString::fromUtf8("tableCheckConstraints"));
        tableCheckConstraints->setAlternatingRowColors(true);
        tableCheckConstraints->setHorizontalScrollMode(QAbstractItemView::ScrollPerPixel);

        verticalLayout_6->addWidget(tableCheckConstraints);

        groupDefinition->addTab(tab_3, QString());
        splitter->addWidget(groupDefinition);
        sqlTextEdit = new SqlTextEdit(splitter);
        sqlTextEdit->setObjectName(QString::fromUtf8("sqlTextEdit"));
        sqlTextEdit->setProperty("readOnly", QVariant(true));
        splitter->addWidget(sqlTextEdit);

        verticalLayout_2->addWidget(splitter);

        labelEditWarning = new QLabel(EditTableDialog);
        labelEditWarning->setObjectName(QString::fromUtf8("labelEditWarning"));
        labelEditWarning->setWordWrap(true);

        verticalLayout_2->addWidget(labelEditWarning);

        buttonBox = new QDialogButtonBox(EditTableDialog);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout_2->addWidget(buttonBox);

#if QT_CONFIG(shortcut)
        label->setBuddy(comboSchema);
        label_1->setBuddy(checkWithoutRowid);
        label_3->setBuddy(checkStrict);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(editTableName, buttonMore);
        QWidget::setTabOrder(buttonMore, comboSchema);
        QWidget::setTabOrder(comboSchema, checkWithoutRowid);
        QWidget::setTabOrder(checkWithoutRowid, checkStrict);
        QWidget::setTabOrder(checkStrict, comboOnConflict);
        QWidget::setTabOrder(comboOnConflict, groupDefinition);
        QWidget::setTabOrder(groupDefinition, addFieldButton);
        QWidget::setTabOrder(addFieldButton, removeFieldButton);
        QWidget::setTabOrder(removeFieldButton, buttonMoveTop);
        QWidget::setTabOrder(buttonMoveTop, buttonMoveUp);
        QWidget::setTabOrder(buttonMoveUp, buttonMoveDown);
        QWidget::setTabOrder(buttonMoveDown, buttonMoveBottom);
        QWidget::setTabOrder(buttonMoveBottom, treeWidget);
        QWidget::setTabOrder(treeWidget, buttonAddIndexConstraint);
        QWidget::setTabOrder(buttonAddIndexConstraint, buttonRemoveIndexConstraint);
        QWidget::setTabOrder(buttonRemoveIndexConstraint, tableIndexConstraints);

        retranslateUi(EditTableDialog);
        QObject::connect(buttonBox, SIGNAL(accepted()), EditTableDialog, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), EditTableDialog, SLOT(reject()));
        QObject::connect(treeWidget, SIGNAL(itemSelectionChanged()), EditTableDialog, SLOT(fieldSelectionChanged()));
        QObject::connect(addFieldButton, SIGNAL(clicked()), EditTableDialog, SLOT(addField()));
        QObject::connect(removeFieldButton, SIGNAL(clicked()), EditTableDialog, SLOT(removeField()));
        QObject::connect(editTableName, SIGNAL(textChanged(QString)), EditTableDialog, SLOT(checkInput()));
        QObject::connect(buttonMoveUp, SIGNAL(clicked()), EditTableDialog, SLOT(moveUp()));
        QObject::connect(buttonMoveDown, SIGNAL(clicked()), EditTableDialog, SLOT(moveDown()));
        QObject::connect(buttonMore, SIGNAL(toggled(bool)), widgetExtension, SLOT(setVisible(bool)));
        QObject::connect(checkWithoutRowid, SIGNAL(toggled(bool)), EditTableDialog, SLOT(setWithoutRowid(bool)));
        QObject::connect(treeWidget, SIGNAL(currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)), EditTableDialog, SLOT(fieldSelectionChanged()));
        QObject::connect(comboSchema, SIGNAL(currentIndexChanged(QString)), EditTableDialog, SLOT(changeSchema(QString)));
        QObject::connect(buttonRemoveIndexConstraint, SIGNAL(clicked()), EditTableDialog, SLOT(removeIndexConstraint()));
        QObject::connect(buttonMoveTop, SIGNAL(clicked()), EditTableDialog, SLOT(moveTop()));
        QObject::connect(buttonMoveBottom, SIGNAL(clicked()), EditTableDialog, SLOT(moveBottom()));
        QObject::connect(comboOnConflict, SIGNAL(currentIndexChanged(QString)), EditTableDialog, SLOT(setOnConflict(QString)));
        QObject::connect(checkStrict, SIGNAL(toggled(bool)), EditTableDialog, SLOT(setStrict(bool)));
        QObject::connect(buttonRemoveForeignKey, SIGNAL(clicked()), EditTableDialog, SLOT(removeForeignKey()));
        QObject::connect(buttonAddForeignKey, SIGNAL(clicked()), EditTableDialog, SLOT(addForeignKey()));
        QObject::connect(buttonAddCheckConstraint, SIGNAL(clicked()), EditTableDialog, SLOT(addCheckConstraint()));
        QObject::connect(buttonRemoveCheckConstraint, SIGNAL(clicked()), EditTableDialog, SLOT(removeCheckConstraint()));

        groupDefinition->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(EditTableDialog);
    } // setupUi

    void retranslateUi(QDialog *EditTableDialog)
    {
        EditTableDialog->setWindowTitle(QCoreApplication::translate("EditTableDialog", "Edit table definition", nullptr));
        actionAddPrimaryKey->setText(QCoreApplication::translate("EditTableDialog", "Primary Key", nullptr));
#if QT_CONFIG(tooltip)
        actionAddPrimaryKey->setToolTip(QCoreApplication::translate("EditTableDialog", "Add a primary key constraint", nullptr));
#endif // QT_CONFIG(tooltip)
        actionAddUniqueConstraint->setText(QCoreApplication::translate("EditTableDialog", "Unique", nullptr));
#if QT_CONFIG(tooltip)
        actionAddUniqueConstraint->setToolTip(QCoreApplication::translate("EditTableDialog", "Add a unique constraint", nullptr));
#endif // QT_CONFIG(tooltip)
        groupTable->setTitle(QCoreApplication::translate("EditTableDialog", "Table", nullptr));
        buttonMore->setText(QCoreApplication::translate("EditTableDialog", "Advanced", nullptr));
        label->setText(QCoreApplication::translate("EditTableDialog", "Database sche&ma", nullptr));
        label_1->setText(QCoreApplication::translate("EditTableDialog", "Without Rowid", nullptr));
#if QT_CONFIG(tooltip)
        checkWithoutRowid->setToolTip(QCoreApplication::translate("EditTableDialog", "Make this a 'WITHOUT ROWID' table. Setting this flag requires specifying a PRIMARY KEY (which can be of any type, and can be composite), and forbids the AUTOINCREMENT flag.", nullptr));
#endif // QT_CONFIG(tooltip)
        label_2->setText(QCoreApplication::translate("EditTableDialog", "On Conflict", nullptr));

        label_3->setText(QCoreApplication::translate("EditTableDialog", "Strict", nullptr));
#if QT_CONFIG(tooltip)
        checkStrict->setToolTip(QCoreApplication::translate("EditTableDialog", "When the strict option is enabled SQLite enforces the data types of each column when updating or inserting data.", nullptr));
#endif // QT_CONFIG(tooltip)
        addFieldButton->setText(QCoreApplication::translate("EditTableDialog", "Add", nullptr));
        removeFieldButton->setText(QCoreApplication::translate("EditTableDialog", "Remove", nullptr));
        buttonMoveTop->setText(QCoreApplication::translate("EditTableDialog", "Move to top", nullptr));
        buttonMoveUp->setText(QCoreApplication::translate("EditTableDialog", "Move up", nullptr));
        buttonMoveDown->setText(QCoreApplication::translate("EditTableDialog", "Move down", nullptr));
        buttonMoveBottom->setText(QCoreApplication::translate("EditTableDialog", "Move to bottom", nullptr));
        QTreeWidgetItem *___qtreewidgetitem = treeWidget->headerItem();
        ___qtreewidgetitem->setText(9, QCoreApplication::translate("EditTableDialog", "Foreign Key", nullptr));
        ___qtreewidgetitem->setText(8, QCoreApplication::translate("EditTableDialog", "Collation", nullptr));
        ___qtreewidgetitem->setText(7, QCoreApplication::translate("EditTableDialog", "Check", nullptr));
        ___qtreewidgetitem->setText(6, QCoreApplication::translate("EditTableDialog", "Default", nullptr));
        ___qtreewidgetitem->setText(5, QCoreApplication::translate("EditTableDialog", "U", nullptr));
        ___qtreewidgetitem->setText(4, QCoreApplication::translate("EditTableDialog", "AI", nullptr));
        ___qtreewidgetitem->setText(3, QCoreApplication::translate("EditTableDialog", "PK", nullptr));
        ___qtreewidgetitem->setText(2, QCoreApplication::translate("EditTableDialog", "NN", nullptr));
        ___qtreewidgetitem->setText(1, QCoreApplication::translate("EditTableDialog", "Type", nullptr));
        ___qtreewidgetitem->setText(0, QCoreApplication::translate("EditTableDialog", "Name", nullptr));
#if QT_CONFIG(tooltip)
        ___qtreewidgetitem->setToolTip(9, QCoreApplication::translate("EditTableDialog", "<html><head/><body><p><img src=\":/icons/field_fk\"/> Foreign Key</p></body></html>", nullptr));
        ___qtreewidgetitem->setToolTip(7, QCoreApplication::translate("EditTableDialog", "Check constraint", nullptr));
        ___qtreewidgetitem->setToolTip(6, QCoreApplication::translate("EditTableDialog", "Default value", nullptr));
        ___qtreewidgetitem->setToolTip(5, QCoreApplication::translate("EditTableDialog", "Unique", nullptr));
        ___qtreewidgetitem->setToolTip(4, QCoreApplication::translate("EditTableDialog", "Autoincrement", nullptr));
        ___qtreewidgetitem->setToolTip(3, QCoreApplication::translate("EditTableDialog", "<html><head/><body><p><img src=\":/icons/field_key\"/> Primary key</p></body></html>", nullptr));
        ___qtreewidgetitem->setToolTip(2, QCoreApplication::translate("EditTableDialog", "Not null", nullptr));
#endif // QT_CONFIG(tooltip)
        groupDefinition->setTabText(groupDefinition->indexOf(groupFieldsPage1), QCoreApplication::translate("EditTableDialog", "Fields", nullptr));
        buttonAddIndexConstraint->setText(QCoreApplication::translate("EditTableDialog", "Add constraint", nullptr));
        buttonRemoveIndexConstraint->setText(QCoreApplication::translate("EditTableDialog", "Remove constraint", nullptr));
        QTableWidgetItem *___qtablewidgetitem = tableIndexConstraints->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(QCoreApplication::translate("EditTableDialog", "Columns", nullptr));
        QTableWidgetItem *___qtablewidgetitem1 = tableIndexConstraints->horizontalHeaderItem(1);
        ___qtablewidgetitem1->setText(QCoreApplication::translate("EditTableDialog", "Type", nullptr));
        QTableWidgetItem *___qtablewidgetitem2 = tableIndexConstraints->horizontalHeaderItem(2);
        ___qtablewidgetitem2->setText(QCoreApplication::translate("EditTableDialog", "Name", nullptr));
        QTableWidgetItem *___qtablewidgetitem3 = tableIndexConstraints->horizontalHeaderItem(3);
        ___qtablewidgetitem3->setText(QCoreApplication::translate("EditTableDialog", "SQL", nullptr));
        groupDefinition->setTabText(groupDefinition->indexOf(tab), QCoreApplication::translate("EditTableDialog", "Index Constraints", nullptr));
        buttonAddForeignKey->setText(QCoreApplication::translate("EditTableDialog", "Add", nullptr));
        buttonRemoveForeignKey->setText(QCoreApplication::translate("EditTableDialog", "Remove", nullptr));
        QTableWidgetItem *___qtablewidgetitem4 = tableForeignKeys->horizontalHeaderItem(0);
        ___qtablewidgetitem4->setText(QCoreApplication::translate("EditTableDialog", "Columns", nullptr));
        QTableWidgetItem *___qtablewidgetitem5 = tableForeignKeys->horizontalHeaderItem(1);
        ___qtablewidgetitem5->setText(QCoreApplication::translate("EditTableDialog", "Name", nullptr));
        QTableWidgetItem *___qtablewidgetitem6 = tableForeignKeys->horizontalHeaderItem(2);
        ___qtablewidgetitem6->setText(QCoreApplication::translate("EditTableDialog", "References", nullptr));
        QTableWidgetItem *___qtablewidgetitem7 = tableForeignKeys->horizontalHeaderItem(3);
        ___qtablewidgetitem7->setText(QCoreApplication::translate("EditTableDialog", "SQL", nullptr));
        groupDefinition->setTabText(groupDefinition->indexOf(tab_2), QCoreApplication::translate("EditTableDialog", "Foreign Keys", nullptr));
        buttonAddCheckConstraint->setText(QCoreApplication::translate("EditTableDialog", "Add", nullptr));
        buttonRemoveCheckConstraint->setText(QCoreApplication::translate("EditTableDialog", "Remove", nullptr));
        QTableWidgetItem *___qtablewidgetitem8 = tableCheckConstraints->horizontalHeaderItem(0);
        ___qtablewidgetitem8->setText(QCoreApplication::translate("EditTableDialog", "Check", nullptr));
        QTableWidgetItem *___qtablewidgetitem9 = tableCheckConstraints->horizontalHeaderItem(1);
        ___qtablewidgetitem9->setText(QCoreApplication::translate("EditTableDialog", "Name", nullptr));
        QTableWidgetItem *___qtablewidgetitem10 = tableCheckConstraints->horizontalHeaderItem(2);
        ___qtablewidgetitem10->setText(QCoreApplication::translate("EditTableDialog", "SQL", nullptr));
        groupDefinition->setTabText(groupDefinition->indexOf(tab_3), QCoreApplication::translate("EditTableDialog", "Check Constraints", nullptr));
        labelEditWarning->setText(QCoreApplication::translate("EditTableDialog", "<html><head/><body><p><span style=\" font-weight:600; color:#ff0000;\">Warning: </span>There is something with this table definition that our parser doesn't fully understand. Modifying and saving this table might result in problems.</p></body></html>", nullptr));
    } // retranslateUi

};

namespace Ui {
    class EditTableDialog: public Ui_EditTableDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EDITTABLEDIALOG_H
