/****************************************************************************
** Meta object code from reading C++ file 'RunSql.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../src/RunSql.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'RunSql.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_RunSql_t {
    QByteArrayData data[12];
    char stringdata0[164];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_RunSql_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_RunSql_t qt_meta_stringdata_RunSql = {
    {
QT_MOC_LITERAL(0, 0, 6), // "RunSql"
QT_MOC_LITERAL(1, 7, 16), // "statementErrored"
QT_MOC_LITERAL(2, 24, 0), // ""
QT_MOC_LITERAL(3, 25, 7), // "message"
QT_MOC_LITERAL(4, 33, 13), // "from_position"
QT_MOC_LITERAL(5, 47, 11), // "to_position"
QT_MOC_LITERAL(6, 59, 17), // "statementExecuted"
QT_MOC_LITERAL(7, 77, 20), // "statementReturnsRows"
QT_MOC_LITERAL(8, 98, 5), // "query"
QT_MOC_LITERAL(9, 104, 10), // "time_in_ms"
QT_MOC_LITERAL(10, 115, 16), // "structureUpdated"
QT_MOC_LITERAL(11, 132, 31) // "confirmSaveBeforePragmaOrVacuum"

    },
    "RunSql\0statementErrored\0\0message\0"
    "from_position\0to_position\0statementExecuted\0"
    "statementReturnsRows\0query\0time_in_ms\0"
    "structureUpdated\0confirmSaveBeforePragmaOrVacuum"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_RunSql[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    3,   39,    2, 0x06 /* Public */,
       6,    3,   46,    2, 0x06 /* Public */,
       7,    4,   53,    2, 0x06 /* Public */,
      10,    0,   62,    2, 0x06 /* Public */,
      11,    0,   63,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::Int, QMetaType::Int,    3,    4,    5,
    QMetaType::Void, QMetaType::QString, QMetaType::Int, QMetaType::Int,    3,    4,    5,
    QMetaType::Void, QMetaType::QString, QMetaType::Int, QMetaType::Int, QMetaType::LongLong,    8,    4,    5,    9,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void RunSql::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<RunSql *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->statementErrored((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 1: _t->statementExecuted((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 2: _t->statementReturnsRows((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< qint64(*)>(_a[4]))); break;
        case 3: _t->structureUpdated(); break;
        case 4: _t->confirmSaveBeforePragmaOrVacuum(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (RunSql::*)(QString , int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RunSql::statementErrored)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (RunSql::*)(QString , int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RunSql::statementExecuted)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (RunSql::*)(QString , int , int , qint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RunSql::statementReturnsRows)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (RunSql::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RunSql::structureUpdated)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (RunSql::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RunSql::confirmSaveBeforePragmaOrVacuum)) {
                *result = 4;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject RunSql::staticMetaObject = { {
    QMetaObject::SuperData::link<QThread::staticMetaObject>(),
    qt_meta_stringdata_RunSql.data,
    qt_meta_data_RunSql,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *RunSql::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *RunSql::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_RunSql.stringdata0))
        return static_cast<void*>(this);
    return QThread::qt_metacast(_clname);
}

int RunSql::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QThread::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void RunSql::statementErrored(QString _t1, int _t2, int _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void RunSql::statementExecuted(QString _t1, int _t2, int _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void RunSql::statementReturnsRows(QString _t1, int _t2, int _t3, qint64 _t4)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void RunSql::structureUpdated()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void RunSql::confirmSaveBeforePragmaOrVacuum()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
