/****************************************************************************
** Meta object code from reading C++ file 'EditDialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../src/EditDialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'EditDialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_EditDialog_t {
    QByteArrayData data[37];
    char stringdata0[449];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_EditDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_EditDialog_t qt_meta_stringdata_EditDialog = {
    {
QT_MOC_LITERAL(0, 0, 10), // "EditDialog"
QT_MOC_LITERAL(1, 11, 17), // "recordTextUpdated"
QT_MOC_LITERAL(2, 29, 0), // ""
QT_MOC_LITERAL(3, 30, 21), // "QPersistentModelIndex"
QT_MOC_LITERAL(4, 52, 3), // "idx"
QT_MOC_LITERAL(5, 56, 8), // "bArrdata"
QT_MOC_LITERAL(6, 65, 6), // "isBlob"
QT_MOC_LITERAL(7, 72, 12), // "evaluateText"
QT_MOC_LITERAL(8, 85, 11), // "std::string"
QT_MOC_LITERAL(9, 97, 4), // "text"
QT_MOC_LITERAL(10, 102, 20), // "requestUrlOrFileOpen"
QT_MOC_LITERAL(11, 123, 9), // "urlString"
QT_MOC_LITERAL(12, 133, 8), // "setFocus"
QT_MOC_LITERAL(13, 142, 6), // "reject"
QT_MOC_LITERAL(14, 149, 11), // "setReadOnly"
QT_MOC_LITERAL(15, 161, 2), // "ro"
QT_MOC_LITERAL(16, 164, 14), // "reloadSettings"
QT_MOC_LITERAL(17, 179, 10), // "importData"
QT_MOC_LITERAL(18, 190, 6), // "asLink"
QT_MOC_LITERAL(19, 197, 10), // "exportData"
QT_MOC_LITERAL(20, 208, 7), // "setNull"
QT_MOC_LITERAL(21, 216, 17), // "updateApplyButton"
QT_MOC_LITERAL(22, 234, 6), // "accept"
QT_MOC_LITERAL(23, 241, 8), // "loadData"
QT_MOC_LITERAL(24, 250, 19), // "toggleOverwriteMode"
QT_MOC_LITERAL(25, 270, 15), // "editModeChanged"
QT_MOC_LITERAL(26, 286, 7), // "newMode"
QT_MOC_LITERAL(27, 294, 15), // "editTextChanged"
QT_MOC_LITERAL(28, 310, 16), // "switchEditorMode"
QT_MOC_LITERAL(29, 327, 17), // "autoSwitchForType"
QT_MOC_LITERAL(30, 345, 21), // "updateCellInfoAndMode"
QT_MOC_LITERAL(31, 367, 23), // "setMustIndentAndCompact"
QT_MOC_LITERAL(32, 391, 6), // "enable"
QT_MOC_LITERAL(33, 398, 15), // "openPrintDialog"
QT_MOC_LITERAL(34, 414, 12), // "copyHexAscii"
QT_MOC_LITERAL(35, 427, 15), // "setWordWrapping"
QT_MOC_LITERAL(36, 443, 5) // "value"

    },
    "EditDialog\0recordTextUpdated\0\0"
    "QPersistentModelIndex\0idx\0bArrdata\0"
    "isBlob\0evaluateText\0std::string\0text\0"
    "requestUrlOrFileOpen\0urlString\0setFocus\0"
    "reject\0setReadOnly\0ro\0reloadSettings\0"
    "importData\0asLink\0exportData\0setNull\0"
    "updateApplyButton\0accept\0loadData\0"
    "toggleOverwriteMode\0editModeChanged\0"
    "newMode\0editTextChanged\0switchEditorMode\0"
    "autoSwitchForType\0updateCellInfoAndMode\0"
    "setMustIndentAndCompact\0enable\0"
    "openPrintDialog\0copyHexAscii\0"
    "setWordWrapping\0value"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_EditDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      23,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    3,  129,    2, 0x06 /* Public */,
       7,    2,  136,    2, 0x06 /* Public */,
      10,    1,  141,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      12,    0,  144,    2, 0x0a /* Public */,
      13,    0,  145,    2, 0x0a /* Public */,
      14,    1,  146,    2, 0x0a /* Public */,
      16,    0,  149,    2, 0x0a /* Public */,
      17,    1,  150,    2, 0x08 /* Private */,
      17,    0,  153,    2, 0x28 /* Private | MethodCloned */,
      19,    0,  154,    2, 0x08 /* Private */,
      20,    0,  155,    2, 0x08 /* Private */,
      21,    0,  156,    2, 0x08 /* Private */,
      22,    0,  157,    2, 0x08 /* Private */,
      23,    1,  158,    2, 0x08 /* Private */,
      24,    0,  161,    2, 0x08 /* Private */,
      25,    1,  162,    2, 0x08 /* Private */,
      27,    0,  165,    2, 0x08 /* Private */,
      28,    1,  166,    2, 0x08 /* Private */,
      30,    1,  169,    2, 0x08 /* Private */,
      31,    1,  172,    2, 0x08 /* Private */,
      33,    0,  175,    2, 0x08 /* Private */,
      34,    0,  176,    2, 0x08 /* Private */,
      35,    1,  177,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::QByteArray, QMetaType::Bool,    4,    5,    6,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 8,    4,    9,
    QMetaType::Void, QMetaType::QString,   11,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   15,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QByteArray,    5,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   26,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   29,
    QMetaType::Void, QMetaType::QByteArray,    5,
    QMetaType::Void, QMetaType::Bool,   32,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   36,

       0        // eod
};

void EditDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<EditDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->recordTextUpdated((*reinterpret_cast< const QPersistentModelIndex(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 1: _t->evaluateText((*reinterpret_cast< const QPersistentModelIndex(*)>(_a[1])),(*reinterpret_cast< const std::string(*)>(_a[2]))); break;
        case 2: _t->requestUrlOrFileOpen((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->setFocus(); break;
        case 4: _t->reject(); break;
        case 5: _t->setReadOnly((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->reloadSettings(); break;
        case 7: _t->importData((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: _t->importData(); break;
        case 9: _t->exportData(); break;
        case 10: _t->setNull(); break;
        case 11: _t->updateApplyButton(); break;
        case 12: _t->accept(); break;
        case 13: _t->loadData((*reinterpret_cast< const QByteArray(*)>(_a[1]))); break;
        case 14: _t->toggleOverwriteMode(); break;
        case 15: _t->editModeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 16: _t->editTextChanged(); break;
        case 17: _t->switchEditorMode((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 18: _t->updateCellInfoAndMode((*reinterpret_cast< const QByteArray(*)>(_a[1]))); break;
        case 19: _t->setMustIndentAndCompact((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 20: _t->openPrintDialog(); break;
        case 21: _t->copyHexAscii(); break;
        case 22: _t->setWordWrapping((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (EditDialog::*)(const QPersistentModelIndex & , const QByteArray & , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&EditDialog::recordTextUpdated)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (EditDialog::*)(const QPersistentModelIndex & , const std::string & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&EditDialog::evaluateText)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (EditDialog::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&EditDialog::requestUrlOrFileOpen)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject EditDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_EditDialog.data,
    qt_meta_data_EditDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *EditDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *EditDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_EditDialog.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int EditDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 23;
    }
    return _id;
}

// SIGNAL 0
void EditDialog::recordTextUpdated(const QPersistentModelIndex & _t1, const QByteArray & _t2, bool _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void EditDialog::evaluateText(const QPersistentModelIndex & _t1, const std::string & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void EditDialog::requestUrlOrFileOpen(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
