/********************************************************************************
** Form generated from reading UI file 'RemoteDock.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_REMOTEDOCK_H
#define UI_REMOTEDOCK_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QCommandLinkButton>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolBar>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_RemoteDock
{
public:
    QAction *actionDeleteDatabase;
    QAction *actionDatabaseOpenBrowser;
    QAction *actionCloneDatabaseLink;
    QAction *actionRefresh;
    QAction *actionCloneDatabaseDoubleClick;
    QAction *actionOpenLocalDatabase;
    QAction *actionFetchCommit;
    QAction *actionFetchLatestCommit;
    QAction *actionDownloadCommit;
    QAction *actionPushLocalDatabase;
    QHBoxLayout *horizontalLayout_2;
    QStackedWidget *stack;
    QWidget *pageMain;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QLabel *label_2;
    QComboBox *comboUser;
    QSpacerItem *horizontalSpacer;
    QToolButton *buttonPushDatabase;
    QTabWidget *tabs;
    QWidget *tabRemote;
    QVBoxLayout *verticalLayout_4;
    QToolBar *toolBar_2;
    QTreeView *treeRemote;
    QWidget *tabLocal;
    QVBoxLayout *verticalLayout_5;
    QTreeView *treeLocal;
    QWidget *tabDatabase;
    QVBoxLayout *verticalLayout_7;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QVBoxLayout *verticalLayout_8;
    QToolBar *toolBar;
    QGroupBox *groupBox;
    QFormLayout *formLayout;
    QLabel *label;
    QLineEdit *editDatabaseUser;
    QLabel *label_4;
    QLineEdit *editDatabaseFile;
    QLabel *label_6;
    QLineEdit *editDatabaseBranch;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout_6;
    QHBoxLayout *horizontalLayout_4;
    QLabel *label_3;
    QComboBox *comboDatabaseBranch;
    QTreeView *treeDatabaseCommits;
    QWidget *pageNoCerts;
    QVBoxLayout *verticalLayout_3;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *horizontalLayout_3;
    QSpacerItem *horizontalSpacer_2;
    QVBoxLayout *verticalLayout_2;
    QLabel *labelNoCert;
    QCommandLinkButton *buttonNoCertBack;
    QSpacerItem *horizontalSpacer_3;
    QSpacerItem *verticalSpacer_2;

    void setupUi(QDialog *RemoteDock)
    {
        if (RemoteDock->objectName().isEmpty())
            RemoteDock->setObjectName(QString::fromUtf8("RemoteDock"));
        RemoteDock->resize(534, 387);
        actionDeleteDatabase = new QAction(RemoteDock);
        actionDeleteDatabase->setObjectName(QString::fromUtf8("actionDeleteDatabase"));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/icons/close"), QSize(), QIcon::Normal, QIcon::Off);
        actionDeleteDatabase->setIcon(icon);
        actionDatabaseOpenBrowser = new QAction(RemoteDock);
        actionDatabaseOpenBrowser->setObjectName(QString::fromUtf8("actionDatabaseOpenBrowser"));
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/icons/browser_open"), QSize(), QIcon::Normal, QIcon::Off);
        actionDatabaseOpenBrowser->setIcon(icon1);
        actionCloneDatabaseLink = new QAction(RemoteDock);
        actionCloneDatabaseLink->setObjectName(QString::fromUtf8("actionCloneDatabaseLink"));
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/icons/clone_database"), QSize(), QIcon::Normal, QIcon::Off);
        actionCloneDatabaseLink->setIcon(icon2);
        actionRefresh = new QAction(RemoteDock);
        actionRefresh->setObjectName(QString::fromUtf8("actionRefresh"));
        QIcon icon3;
        icon3.addFile(QString::fromUtf8(":/icons/refresh"), QSize(), QIcon::Normal, QIcon::Off);
        actionRefresh->setIcon(icon3);
        actionCloneDatabaseDoubleClick = new QAction(RemoteDock);
        actionCloneDatabaseDoubleClick->setObjectName(QString::fromUtf8("actionCloneDatabaseDoubleClick"));
        actionCloneDatabaseDoubleClick->setIcon(icon2);
        QFont font;
        font.setBold(true);
        font.setWeight(75);
        actionCloneDatabaseDoubleClick->setFont(font);
        actionOpenLocalDatabase = new QAction(RemoteDock);
        actionOpenLocalDatabase->setObjectName(QString::fromUtf8("actionOpenLocalDatabase"));
        QIcon icon4;
        icon4.addFile(QString::fromUtf8(":/icons/db_open"), QSize(), QIcon::Normal, QIcon::Off);
        actionOpenLocalDatabase->setIcon(icon4);
        actionOpenLocalDatabase->setFont(font);
        actionFetchCommit = new QAction(RemoteDock);
        actionFetchCommit->setObjectName(QString::fromUtf8("actionFetchCommit"));
        actionFetchCommit->setFont(font);
        actionFetchLatestCommit = new QAction(RemoteDock);
        actionFetchLatestCommit->setObjectName(QString::fromUtf8("actionFetchLatestCommit"));
        QIcon icon5;
        icon5.addFile(QString::fromUtf8(":/icons/db_revert"), QSize(), QIcon::Normal, QIcon::Off);
        actionFetchLatestCommit->setIcon(icon5);
        actionDownloadCommit = new QAction(RemoteDock);
        actionDownloadCommit->setObjectName(QString::fromUtf8("actionDownloadCommit"));
        actionPushLocalDatabase = new QAction(RemoteDock);
        actionPushLocalDatabase->setObjectName(QString::fromUtf8("actionPushLocalDatabase"));
        QIcon icon6;
        icon6.addFile(QString::fromUtf8(":/icons/push_database"), QSize(), QIcon::Normal, QIcon::Off);
        actionPushLocalDatabase->setIcon(icon6);
        horizontalLayout_2 = new QHBoxLayout(RemoteDock);
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        horizontalLayout_2->setContentsMargins(2, 0, 2, 0);
        stack = new QStackedWidget(RemoteDock);
        stack->setObjectName(QString::fromUtf8("stack"));
        pageMain = new QWidget();
        pageMain->setObjectName(QString::fromUtf8("pageMain"));
        verticalLayout = new QVBoxLayout(pageMain);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        label_2 = new QLabel(pageMain);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        horizontalLayout->addWidget(label_2);

        comboUser = new QComboBox(pageMain);
        comboUser->setObjectName(QString::fromUtf8("comboUser"));
        comboUser->setSizeAdjustPolicy(QComboBox::AdjustToContents);

        horizontalLayout->addWidget(comboUser);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        buttonPushDatabase = new QToolButton(pageMain);
        buttonPushDatabase->setObjectName(QString::fromUtf8("buttonPushDatabase"));
        buttonPushDatabase->setIcon(icon6);
        buttonPushDatabase->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);

        horizontalLayout->addWidget(buttonPushDatabase);


        verticalLayout->addLayout(horizontalLayout);

        tabs = new QTabWidget(pageMain);
        tabs->setObjectName(QString::fromUtf8("tabs"));
        tabRemote = new QWidget();
        tabRemote->setObjectName(QString::fromUtf8("tabRemote"));
        verticalLayout_4 = new QVBoxLayout(tabRemote);
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        verticalLayout_4->setContentsMargins(2, 0, 2, 0);
        toolBar_2 = new QToolBar(tabRemote);
        toolBar_2->setObjectName(QString::fromUtf8("toolBar_2"));
        toolBar_2->setMovable(false);
        toolBar_2->setFloatable(false);

        verticalLayout_4->addWidget(toolBar_2);

        treeRemote = new QTreeView(tabRemote);
        treeRemote->setObjectName(QString::fromUtf8("treeRemote"));
        treeRemote->setContextMenuPolicy(Qt::ActionsContextMenu);

        verticalLayout_4->addWidget(treeRemote);

        tabs->addTab(tabRemote, QString());
        tabLocal = new QWidget();
        tabLocal->setObjectName(QString::fromUtf8("tabLocal"));
        verticalLayout_5 = new QVBoxLayout(tabLocal);
        verticalLayout_5->setObjectName(QString::fromUtf8("verticalLayout_5"));
        verticalLayout_5->setContentsMargins(2, 0, 2, 0);
        treeLocal = new QTreeView(tabLocal);
        treeLocal->setObjectName(QString::fromUtf8("treeLocal"));
        treeLocal->setContextMenuPolicy(Qt::ActionsContextMenu);

        verticalLayout_5->addWidget(treeLocal);

        tabs->addTab(tabLocal, QString());
        tabDatabase = new QWidget();
        tabDatabase->setObjectName(QString::fromUtf8("tabDatabase"));
        verticalLayout_7 = new QVBoxLayout(tabDatabase);
        verticalLayout_7->setObjectName(QString::fromUtf8("verticalLayout_7"));
        verticalLayout_7->setContentsMargins(2, 0, 2, 0);
        scrollArea = new QScrollArea(tabDatabase);
        scrollArea->setObjectName(QString::fromUtf8("scrollArea"));
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName(QString::fromUtf8("scrollAreaWidgetContents"));
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 520, 319));
        verticalLayout_8 = new QVBoxLayout(scrollAreaWidgetContents);
        verticalLayout_8->setObjectName(QString::fromUtf8("verticalLayout_8"));
        verticalLayout_8->setContentsMargins(0, 0, 0, 0);
        toolBar = new QToolBar(scrollAreaWidgetContents);
        toolBar->setObjectName(QString::fromUtf8("toolBar"));
        toolBar->setMovable(false);
        toolBar->setFloatable(false);

        verticalLayout_8->addWidget(toolBar);

        groupBox = new QGroupBox(scrollAreaWidgetContents);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        formLayout = new QFormLayout(groupBox);
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        formLayout->setVerticalSpacing(0);
        label = new QLabel(groupBox);
        label->setObjectName(QString::fromUtf8("label"));

        formLayout->setWidget(0, QFormLayout::LabelRole, label);

        editDatabaseUser = new QLineEdit(groupBox);
        editDatabaseUser->setObjectName(QString::fromUtf8("editDatabaseUser"));
        editDatabaseUser->setReadOnly(true);

        formLayout->setWidget(0, QFormLayout::FieldRole, editDatabaseUser);

        label_4 = new QLabel(groupBox);
        label_4->setObjectName(QString::fromUtf8("label_4"));

        formLayout->setWidget(1, QFormLayout::LabelRole, label_4);

        editDatabaseFile = new QLineEdit(groupBox);
        editDatabaseFile->setObjectName(QString::fromUtf8("editDatabaseFile"));
        editDatabaseFile->setReadOnly(true);

        formLayout->setWidget(1, QFormLayout::FieldRole, editDatabaseFile);

        label_6 = new QLabel(groupBox);
        label_6->setObjectName(QString::fromUtf8("label_6"));

        formLayout->setWidget(2, QFormLayout::LabelRole, label_6);

        editDatabaseBranch = new QLineEdit(groupBox);
        editDatabaseBranch->setObjectName(QString::fromUtf8("editDatabaseBranch"));
        editDatabaseBranch->setReadOnly(true);

        formLayout->setWidget(2, QFormLayout::FieldRole, editDatabaseBranch);


        verticalLayout_8->addWidget(groupBox);

        groupBox_2 = new QGroupBox(scrollAreaWidgetContents);
        groupBox_2->setObjectName(QString::fromUtf8("groupBox_2"));
        verticalLayout_6 = new QVBoxLayout(groupBox_2);
        verticalLayout_6->setObjectName(QString::fromUtf8("verticalLayout_6"));
        verticalLayout_6->setContentsMargins(2, 0, 2, 0);
        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName(QString::fromUtf8("horizontalLayout_4"));
        label_3 = new QLabel(groupBox_2);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        horizontalLayout_4->addWidget(label_3);

        comboDatabaseBranch = new QComboBox(groupBox_2);
        comboDatabaseBranch->setObjectName(QString::fromUtf8("comboDatabaseBranch"));

        horizontalLayout_4->addWidget(comboDatabaseBranch);


        verticalLayout_6->addLayout(horizontalLayout_4);

        treeDatabaseCommits = new QTreeView(groupBox_2);
        treeDatabaseCommits->setObjectName(QString::fromUtf8("treeDatabaseCommits"));
        treeDatabaseCommits->setContextMenuPolicy(Qt::ActionsContextMenu);

        verticalLayout_6->addWidget(treeDatabaseCommits);


        verticalLayout_8->addWidget(groupBox_2);

        scrollArea->setWidget(scrollAreaWidgetContents);

        verticalLayout_7->addWidget(scrollArea);

        tabs->addTab(tabDatabase, QString());

        verticalLayout->addWidget(tabs);

        stack->addWidget(pageMain);
        pageNoCerts = new QWidget();
        pageNoCerts->setObjectName(QString::fromUtf8("pageNoCerts"));
        verticalLayout_3 = new QVBoxLayout(pageNoCerts);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        verticalSpacer = new QSpacerItem(20, 85, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_3->addItem(verticalSpacer);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_2);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        labelNoCert = new QLabel(pageNoCerts);
        labelNoCert->setObjectName(QString::fromUtf8("labelNoCert"));
        labelNoCert->setWordWrap(true);

        verticalLayout_2->addWidget(labelNoCert);

        buttonNoCertBack = new QCommandLinkButton(pageNoCerts);
        buttonNoCertBack->setObjectName(QString::fromUtf8("buttonNoCertBack"));

        verticalLayout_2->addWidget(buttonNoCertBack);


        horizontalLayout_3->addLayout(verticalLayout_2);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_3);


        verticalLayout_3->addLayout(horizontalLayout_3);

        verticalSpacer_2 = new QSpacerItem(20, 85, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_2);

        stack->addWidget(pageNoCerts);

        horizontalLayout_2->addWidget(stack);

#if QT_CONFIG(shortcut)
        label_2->setBuddy(comboUser);
        label->setBuddy(editDatabaseUser);
        label_4->setBuddy(editDatabaseFile);
        label_6->setBuddy(editDatabaseBranch);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(comboUser, buttonPushDatabase);
        QWidget::setTabOrder(buttonPushDatabase, tabs);
        QWidget::setTabOrder(tabs, treeRemote);
        QWidget::setTabOrder(treeRemote, treeLocal);
        QWidget::setTabOrder(treeLocal, comboDatabaseBranch);
        QWidget::setTabOrder(comboDatabaseBranch, treeDatabaseCommits);
        QWidget::setTabOrder(treeDatabaseCommits, buttonNoCertBack);

        toolBar_2->addAction(actionRefresh);
        toolBar_2->addAction(actionCloneDatabaseLink);
        toolBar->addAction(actionRefresh);
        toolBar->addAction(actionDatabaseOpenBrowser);
        toolBar->addAction(actionFetchLatestCommit);

        retranslateUi(RemoteDock);
        QObject::connect(comboUser, SIGNAL(currentTextChanged(QString)), RemoteDock, SLOT(setNewIdentity(QString)));
        QObject::connect(treeRemote, SIGNAL(doubleClicked(QModelIndex)), RemoteDock, SLOT(fetchDatabase(QModelIndex)));
        QObject::connect(buttonPushDatabase, SIGNAL(clicked()), RemoteDock, SLOT(pushCurrentlyOpenedDatabase()));
        QObject::connect(buttonNoCertBack, SIGNAL(clicked()), RemoteDock, SLOT(switchToMainView()));
        QObject::connect(treeLocal, SIGNAL(doubleClicked(QModelIndex)), RemoteDock, SLOT(openLocalFile(QModelIndex)));
        QObject::connect(actionCloneDatabaseLink, SIGNAL(triggered()), RemoteDock, SLOT(fetchDatabase()));
        QObject::connect(actionDatabaseOpenBrowser, SIGNAL(triggered()), RemoteDock, SLOT(openCurrentDatabaseInBrowser()));
        QObject::connect(actionRefresh, SIGNAL(triggered()), RemoteDock, SLOT(refresh()));
        QObject::connect(treeDatabaseCommits, SIGNAL(doubleClicked(QModelIndex)), RemoteDock, SLOT(fetchCommit(QModelIndex)));
        QObject::connect(actionPushLocalDatabase, SIGNAL(triggered()), RemoteDock, SLOT(pushSelectedLocalDatabase()));

        stack->setCurrentIndex(0);
        tabs->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(RemoteDock);
    } // setupUi

    void retranslateUi(QDialog *RemoteDock)
    {
        RemoteDock->setWindowTitle(QCoreApplication::translate("RemoteDock", "Remote", nullptr));
        actionDeleteDatabase->setText(QCoreApplication::translate("RemoteDock", "Delete Database", nullptr));
#if QT_CONFIG(tooltip)
        actionDeleteDatabase->setToolTip(QCoreApplication::translate("RemoteDock", "Delete the local clone of this database", nullptr));
#endif // QT_CONFIG(tooltip)
        actionDatabaseOpenBrowser->setText(QCoreApplication::translate("RemoteDock", "Open in Web Browser", nullptr));
#if QT_CONFIG(tooltip)
        actionDatabaseOpenBrowser->setToolTip(QCoreApplication::translate("RemoteDock", "Open the web page for the current database in your browser", nullptr));
#endif // QT_CONFIG(tooltip)
        actionCloneDatabaseLink->setText(QCoreApplication::translate("RemoteDock", "Clone from Link", nullptr));
#if QT_CONFIG(tooltip)
        actionCloneDatabaseLink->setToolTip(QCoreApplication::translate("RemoteDock", "Use this to download a remote database for local editing using a URL as provided on the web page of the database.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionRefresh->setText(QCoreApplication::translate("RemoteDock", "Refresh", nullptr));
#if QT_CONFIG(tooltip)
        actionRefresh->setToolTip(QCoreApplication::translate("RemoteDock", "Reload all data and update the views", nullptr));
#endif // QT_CONFIG(tooltip)
        actionCloneDatabaseDoubleClick->setText(QCoreApplication::translate("RemoteDock", "Clone Database", nullptr));
        actionOpenLocalDatabase->setText(QCoreApplication::translate("RemoteDock", "Open Database", nullptr));
#if QT_CONFIG(tooltip)
        actionOpenLocalDatabase->setToolTip(QCoreApplication::translate("RemoteDock", "Open the local copy of this database", nullptr));
#endif // QT_CONFIG(tooltip)
        actionFetchCommit->setText(QCoreApplication::translate("RemoteDock", "Check out Commit", nullptr));
#if QT_CONFIG(tooltip)
        actionFetchCommit->setToolTip(QCoreApplication::translate("RemoteDock", "Download and open this specific commit", nullptr));
#endif // QT_CONFIG(tooltip)
        actionFetchLatestCommit->setText(QCoreApplication::translate("RemoteDock", "Check out Latest Commit", nullptr));
#if QT_CONFIG(tooltip)
        actionFetchLatestCommit->setToolTip(QCoreApplication::translate("RemoteDock", "Check out the latest commit of the current branch", nullptr));
#endif // QT_CONFIG(tooltip)
        actionDownloadCommit->setText(QCoreApplication::translate("RemoteDock", "Save Revision to File", nullptr));
#if QT_CONFIG(tooltip)
        actionDownloadCommit->setToolTip(QCoreApplication::translate("RemoteDock", "Saves the selected revision of the database to another file", nullptr));
#endif // QT_CONFIG(tooltip)
        actionPushLocalDatabase->setText(QCoreApplication::translate("RemoteDock", "Upload Database", nullptr));
#if QT_CONFIG(tooltip)
        actionPushLocalDatabase->setToolTip(QCoreApplication::translate("RemoteDock", "Upload this database as a new commit", nullptr));
#endif // QT_CONFIG(tooltip)
        label_2->setText(QCoreApplication::translate("RemoteDock", "Identity", nullptr));
#if QT_CONFIG(tooltip)
        buttonPushDatabase->setToolTip(QCoreApplication::translate("RemoteDock", "Push currently opened database to server", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonPushDatabase->setText(QCoreApplication::translate("RemoteDock", "Upload", nullptr));
#if QT_CONFIG(whatsthis)
        treeRemote->setWhatsThis(QCoreApplication::translate("RemoteDock", "<html><head/><body><p>In this pane, remote databases from dbhub.io website can be added to DB Browser for SQLite. First you need an identity:</p><ol style=\"margin-top: 0px; margin-bottom: 0px; margin-left: 0px; margin-right: 0px; -qt-list-indent: 1;\"><li style=\" margin-top:12px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Login to the dbhub.io website (use your GitHub credentials or whatever you want)</li><li style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Click the button to &quot;Generate client certificate&quot; (that's your identity). That'll give you a certificate file (save it to your local disk).</li><li style=\" margin-top:0px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Go to the Remote tab in DB Browser for SQLite Preferences. Click the button to add a new certificate to DB Browser for SQLite and choose the just downloaded certific"
                        "ate file.</li></ol><p>Now the Remote panel shows your identity and you can add remote databases.</p></body></html>", nullptr));
#endif // QT_CONFIG(whatsthis)
        tabs->setTabText(tabs->indexOf(tabRemote), QCoreApplication::translate("RemoteDock", "DBHub.io", nullptr));
        tabs->setTabText(tabs->indexOf(tabLocal), QCoreApplication::translate("RemoteDock", "Local", nullptr));
        groupBox->setTitle(QCoreApplication::translate("RemoteDock", "Clone", nullptr));
        label->setText(QCoreApplication::translate("RemoteDock", "&User", nullptr));
        label_4->setText(QCoreApplication::translate("RemoteDock", "&Database", nullptr));
        label_6->setText(QCoreApplication::translate("RemoteDock", "Branch", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("RemoteDock", "Commits", nullptr));
        label_3->setText(QCoreApplication::translate("RemoteDock", "Commits for", nullptr));
        tabs->setTabText(tabs->indexOf(tabDatabase), QCoreApplication::translate("RemoteDock", "Current Database", nullptr));
        labelNoCert->setText(QCoreApplication::translate("RemoteDock", "<html><head/><body><p>You are currently using a built-in, read-only identity. For uploading your database, you need to configure and use your DBHub.io account.</p><p>No DBHub.io account yet? <a href=\"https://dbhub.io/\"><span style=\" text-decoration: underline; color:#007af4;\">Create one now</span></a> and import your certificate <a href=\"#preferences\"><span style=\" text-decoration: underline; color:#007af4;\">here</span></a> to share your databases.</p><p>For online help visit <a href=\"https://dbhub.io/about\"><span style=\" text-decoration: underline; color:#007af4;\">here</span></a>.</p></body></html>", nullptr));
        buttonNoCertBack->setText(QCoreApplication::translate("RemoteDock", "Back", nullptr));
    } // retranslateUi

};

namespace Ui {
    class RemoteDock: public Ui_RemoteDock {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_REMOTEDOCK_H
