/****************************************************************************
** Meta object code from reading C++ file 'FilterTableHeader.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../src/FilterTableHeader.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'FilterTableHeader.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_FilterTableHeader_t {
    QByteArrayData data[22];
    char stringdata0[296];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FilterTableHeader_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FilterTableHeader_t qt_meta_stringdata_FilterTableHeader = {
    {
QT_MOC_LITERAL(0, 0, 17), // "FilterTableHeader"
QT_MOC_LITERAL(1, 18, 13), // "filterChanged"
QT_MOC_LITERAL(2, 32, 0), // ""
QT_MOC_LITERAL(3, 33, 6), // "size_t"
QT_MOC_LITERAL(4, 40, 6), // "column"
QT_MOC_LITERAL(5, 47, 5), // "value"
QT_MOC_LITERAL(6, 53, 13), // "filterFocused"
QT_MOC_LITERAL(7, 67, 13), // "addCondFormat"
QT_MOC_LITERAL(8, 81, 6), // "filter"
QT_MOC_LITERAL(9, 88, 21), // "allCondFormatsCleared"
QT_MOC_LITERAL(10, 110, 17), // "condFormatsEdited"
QT_MOC_LITERAL(11, 128, 15), // "generateFilters"
QT_MOC_LITERAL(12, 144, 6), // "number"
QT_MOC_LITERAL(13, 151, 24), // "number_of_hidden_filters"
QT_MOC_LITERAL(14, 176, 15), // "adjustPositions"
QT_MOC_LITERAL(15, 192, 12), // "clearFilters"
QT_MOC_LITERAL(16, 205, 9), // "setFilter"
QT_MOC_LITERAL(17, 215, 12), // "inputChanged"
QT_MOC_LITERAL(18, 228, 9), // "new_value"
QT_MOC_LITERAL(19, 238, 21), // "addFilterAsCondFormat"
QT_MOC_LITERAL(20, 260, 19), // "clearAllCondFormats"
QT_MOC_LITERAL(21, 280, 15) // "editCondFormats"

    },
    "FilterTableHeader\0filterChanged\0\0"
    "size_t\0column\0value\0filterFocused\0"
    "addCondFormat\0filter\0allCondFormatsCleared\0"
    "condFormatsEdited\0generateFilters\0"
    "number\0number_of_hidden_filters\0"
    "adjustPositions\0clearFilters\0setFilter\0"
    "inputChanged\0new_value\0addFilterAsCondFormat\0"
    "clearAllCondFormats\0editCondFormats"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FilterTableHeader[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      14,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   84,    2, 0x06 /* Public */,
       6,    0,   89,    2, 0x06 /* Public */,
       7,    2,   90,    2, 0x06 /* Public */,
       9,    1,   95,    2, 0x06 /* Public */,
      10,    1,   98,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      11,    2,  101,    2, 0x0a /* Public */,
      11,    1,  106,    2, 0x2a /* Public | MethodCloned */,
      14,    0,  109,    2, 0x0a /* Public */,
      15,    0,  110,    2, 0x0a /* Public */,
      16,    2,  111,    2, 0x0a /* Public */,
      17,    1,  116,    2, 0x08 /* Private */,
      19,    1,  119,    2, 0x08 /* Private */,
      20,    0,  122,    2, 0x08 /* Private */,
      21,    0,  123,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::QString,    4,    5,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 3, QMetaType::QString,    4,    8,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3,   12,   13,
    QMetaType::Void, 0x80000000 | 3,   12,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 3, QMetaType::QString,    4,    5,
    QMetaType::Void, QMetaType::QString,   18,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void FilterTableHeader::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FilterTableHeader *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->filterChanged((*reinterpret_cast< size_t(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2]))); break;
        case 1: _t->filterFocused(); break;
        case 2: _t->addCondFormat((*reinterpret_cast< size_t(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2]))); break;
        case 3: _t->allCondFormatsCleared((*reinterpret_cast< size_t(*)>(_a[1]))); break;
        case 4: _t->condFormatsEdited((*reinterpret_cast< size_t(*)>(_a[1]))); break;
        case 5: _t->generateFilters((*reinterpret_cast< size_t(*)>(_a[1])),(*reinterpret_cast< size_t(*)>(_a[2]))); break;
        case 6: _t->generateFilters((*reinterpret_cast< size_t(*)>(_a[1]))); break;
        case 7: _t->adjustPositions(); break;
        case 8: _t->clearFilters(); break;
        case 9: _t->setFilter((*reinterpret_cast< size_t(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 10: _t->inputChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 11: _t->addFilterAsCondFormat((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 12: _t->clearAllCondFormats(); break;
        case 13: _t->editCondFormats(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FilterTableHeader::*)(size_t , QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FilterTableHeader::filterChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FilterTableHeader::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FilterTableHeader::filterFocused)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (FilterTableHeader::*)(size_t , QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FilterTableHeader::addCondFormat)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (FilterTableHeader::*)(size_t );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FilterTableHeader::allCondFormatsCleared)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (FilterTableHeader::*)(size_t );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FilterTableHeader::condFormatsEdited)) {
                *result = 4;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject FilterTableHeader::staticMetaObject = { {
    QMetaObject::SuperData::link<QHeaderView::staticMetaObject>(),
    qt_meta_stringdata_FilterTableHeader.data,
    qt_meta_data_FilterTableHeader,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FilterTableHeader::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FilterTableHeader::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FilterTableHeader.stringdata0))
        return static_cast<void*>(this);
    return QHeaderView::qt_metacast(_clname);
}

int FilterTableHeader::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QHeaderView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void FilterTableHeader::filterChanged(size_t _t1, QString _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void FilterTableHeader::filterFocused()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void FilterTableHeader::addCondFormat(size_t _t1, QString _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void FilterTableHeader::allCondFormatsCleared(size_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void FilterTableHeader::condFormatsEdited(size_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
