/********************************************************************************
** Form generated from reading UI file 'EditDialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EDITDIALOG_H
#define UI_EDITDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QToolBar>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_EditDialog
{
public:
    QAction *actionPrint;
    QAction *actionCopyHexAscii;
    QAction *actionIndent;
    QAction *actionExport;
    QAction *actionImport;
    QAction *actionNull;
    QAction *actionWordWrap;
    QAction *actionOpenInApp;
    QAction *actionImportAsLink;
    QAction *actionImportInMenu;
    QAction *actionOpenInExternal;
    QVBoxLayout *verticalLayout_5;
    QHBoxLayout *horizontalLayout;
    QLabel *labelMode;
    QComboBox *comboMode;
    QToolButton *buttonAutoSwitchMode;
    QSpacerItem *horizontalSpacer;
    QToolBar *editCellToolbar;
    QStackedWidget *editorStack;
    QTextEdit *qtEdit;
    QHBoxLayout *horizontalLayout_2;
    QVBoxLayout *verticalLayout;
    QLabel *labelCell;
    QLabel *labelInfo;
    QPushButton *buttonApply;

    void setupUi(QDialog *EditDialog)
    {
        if (EditDialog->objectName().isEmpty())
            EditDialog->setObjectName(QString::fromUtf8("EditDialog"));
        EditDialog->resize(618, 382);
        actionPrint = new QAction(EditDialog);
        actionPrint->setObjectName(QString::fromUtf8("actionPrint"));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/icons/print"), QSize(), QIcon::Normal, QIcon::Off);
        actionPrint->setIcon(icon);
        actionPrint->setShortcutContext(Qt::WidgetShortcut);
        actionCopyHexAscii = new QAction(EditDialog);
        actionCopyHexAscii->setObjectName(QString::fromUtf8("actionCopyHexAscii"));
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/icons/special_copy"), QSize(), QIcon::Normal, QIcon::Off);
        actionCopyHexAscii->setIcon(icon1);
        actionIndent = new QAction(EditDialog);
        actionIndent->setObjectName(QString::fromUtf8("actionIndent"));
        actionIndent->setCheckable(true);
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/icons/text_indent"), QSize(), QIcon::Normal, QIcon::Off);
        actionIndent->setIcon(icon2);
        actionExport = new QAction(EditDialog);
        actionExport->setObjectName(QString::fromUtf8("actionExport"));
        QIcon icon3;
        icon3.addFile(QString::fromUtf8(":/icons/save_sql"), QSize(), QIcon::Normal, QIcon::Off);
        actionExport->setIcon(icon3);
        actionImport = new QAction(EditDialog);
        actionImport->setObjectName(QString::fromUtf8("actionImport"));
        QIcon icon4;
        icon4.addFile(QString::fromUtf8(":/icons/document_open"), QSize(), QIcon::Normal, QIcon::Off);
        actionImport->setIcon(icon4);
        actionNull = new QAction(EditDialog);
        actionNull->setObjectName(QString::fromUtf8("actionNull"));
        QIcon icon5;
        icon5.addFile(QString::fromUtf8(":/icons/set_to_null"), QSize(), QIcon::Normal, QIcon::Off);
        actionNull->setIcon(icon5);
        actionWordWrap = new QAction(EditDialog);
        actionWordWrap->setObjectName(QString::fromUtf8("actionWordWrap"));
        actionWordWrap->setCheckable(true);
        actionWordWrap->setChecked(true);
        QIcon icon6;
        icon6.addFile(QString::fromUtf8(":/icons/word_wrap"), QSize(), QIcon::Normal, QIcon::Off);
        actionWordWrap->setIcon(icon6);
        actionOpenInApp = new QAction(EditDialog);
        actionOpenInApp->setObjectName(QString::fromUtf8("actionOpenInApp"));
        QIcon icon7;
        icon7.addFile(QString::fromUtf8(":/icons/open_in_app"), QSize(), QIcon::Normal, QIcon::Off);
        actionOpenInApp->setIcon(icon7);
        actionImportAsLink = new QAction(EditDialog);
        actionImportAsLink->setObjectName(QString::fromUtf8("actionImportAsLink"));
        QIcon icon8;
        icon8.addFile(QString::fromUtf8(":/icons/document_link"), QSize(), QIcon::Normal, QIcon::Off);
        actionImportAsLink->setIcon(icon8);
        actionImportInMenu = new QAction(EditDialog);
        actionImportInMenu->setObjectName(QString::fromUtf8("actionImportInMenu"));
        actionImportInMenu->setIcon(icon4);
        actionOpenInExternal = new QAction(EditDialog);
        actionOpenInExternal->setObjectName(QString::fromUtf8("actionOpenInExternal"));
        QIcon icon9;
        icon9.addFile(QString::fromUtf8(":/icons/open_data_in_app"), QSize(), QIcon::Normal, QIcon::Off);
        actionOpenInExternal->setIcon(icon9);
        verticalLayout_5 = new QVBoxLayout(EditDialog);
        verticalLayout_5->setSpacing(0);
        verticalLayout_5->setObjectName(QString::fromUtf8("verticalLayout_5"));
        verticalLayout_5->setContentsMargins(2, 0, 2, 0);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        labelMode = new QLabel(EditDialog);
        labelMode->setObjectName(QString::fromUtf8("labelMode"));
        labelMode->setMinimumSize(QSize(0, 0));

        horizontalLayout->addWidget(labelMode);

        comboMode = new QComboBox(EditDialog);
        comboMode->addItem(QString());
        comboMode->addItem(QString());
        comboMode->addItem(QString());
        comboMode->addItem(QString());
        comboMode->addItem(QString());
        comboMode->addItem(QString());
        comboMode->addItem(QString());
        comboMode->setObjectName(QString::fromUtf8("comboMode"));
        comboMode->setMinimumSize(QSize(0, 0));

        horizontalLayout->addWidget(comboMode);

        buttonAutoSwitchMode = new QToolButton(EditDialog);
        buttonAutoSwitchMode->setObjectName(QString::fromUtf8("buttonAutoSwitchMode"));
        QIcon icon10;
        icon10.addFile(QString::fromUtf8(":/icons/keyword"), QSize(), QIcon::Normal, QIcon::Off);
        icon10.addFile(QString::fromUtf8(":/icons/cog_go"), QSize(), QIcon::Normal, QIcon::On);
        buttonAutoSwitchMode->setIcon(icon10);
        buttonAutoSwitchMode->setCheckable(true);
        buttonAutoSwitchMode->setChecked(true);

        horizontalLayout->addWidget(buttonAutoSwitchMode);

        horizontalSpacer = new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        editCellToolbar = new QToolBar(EditDialog);
        editCellToolbar->setObjectName(QString::fromUtf8("editCellToolbar"));
        editCellToolbar->setMovable(false);
        editCellToolbar->setIconSize(QSize(16, 16));

        horizontalLayout->addWidget(editCellToolbar);


        verticalLayout_5->addLayout(horizontalLayout);

        editorStack = new QStackedWidget(EditDialog);
        editorStack->setObjectName(QString::fromUtf8("editorStack"));
        qtEdit = new QTextEdit();
        qtEdit->setObjectName(QString::fromUtf8("qtEdit"));
        QFont font;
        font.setFamily(QString::fromUtf8("Monospace"));
        qtEdit->setFont(font);
        qtEdit->setAcceptRichText(false);
        editorStack->addWidget(qtEdit);

        verticalLayout_5->addWidget(editorStack);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        labelCell = new QLabel(EditDialog);
        labelCell->setObjectName(QString::fromUtf8("labelCell"));
        QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(labelCell->sizePolicy().hasHeightForWidth());
        labelCell->setSizePolicy(sizePolicy);
        labelCell->setMinimumSize(QSize(100, 0));
        labelCell->setWordWrap(true);

        verticalLayout->addWidget(labelCell);

        labelInfo = new QLabel(EditDialog);
        labelInfo->setObjectName(QString::fromUtf8("labelInfo"));
        sizePolicy.setHeightForWidth(labelInfo->sizePolicy().hasHeightForWidth());
        labelInfo->setSizePolicy(sizePolicy);
        labelInfo->setMinimumSize(QSize(100, 0));
        labelInfo->setTextFormat(Qt::PlainText);
        labelInfo->setWordWrap(true);

        verticalLayout->addWidget(labelInfo);


        horizontalLayout_2->addLayout(verticalLayout);

        buttonApply = new QPushButton(EditDialog);
        buttonApply->setObjectName(QString::fromUtf8("buttonApply"));
        QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(buttonApply->sizePolicy().hasHeightForWidth());
        buttonApply->setSizePolicy(sizePolicy1);

        horizontalLayout_2->addWidget(buttonApply);


        verticalLayout_5->addLayout(horizontalLayout_2);

#if QT_CONFIG(shortcut)
        labelMode->setBuddy(comboMode);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(comboMode, buttonAutoSwitchMode);
        QWidget::setTabOrder(buttonAutoSwitchMode, buttonApply);

        editCellToolbar->addAction(actionWordWrap);
        editCellToolbar->addAction(actionIndent);
        editCellToolbar->addAction(actionImport);
        editCellToolbar->addAction(actionExport);
        editCellToolbar->addAction(actionOpenInExternal);
        editCellToolbar->addAction(actionOpenInApp);
        editCellToolbar->addAction(actionNull);
        editCellToolbar->addAction(actionPrint);

        retranslateUi(EditDialog);
        QObject::connect(buttonApply, SIGNAL(clicked()), EditDialog, SLOT(accept()));
        QObject::connect(comboMode, SIGNAL(currentIndexChanged(int)), EditDialog, SLOT(editModeChanged(int)));
        QObject::connect(qtEdit, SIGNAL(textChanged()), EditDialog, SLOT(editTextChanged()));
        QObject::connect(actionIndent, SIGNAL(toggled(bool)), EditDialog, SLOT(setMustIndentAndCompact(bool)));
        QObject::connect(buttonAutoSwitchMode, SIGNAL(toggled(bool)), EditDialog, SLOT(switchEditorMode(bool)));
        QObject::connect(actionPrint, SIGNAL(triggered()), EditDialog, SLOT(openPrintDialog()));
        QObject::connect(actionCopyHexAscii, SIGNAL(triggered()), EditDialog, SLOT(copyHexAscii()));
        QObject::connect(actionNull, SIGNAL(triggered()), EditDialog, SLOT(setNull()));
        QObject::connect(actionImport, SIGNAL(triggered()), EditDialog, SLOT(importData()));
        QObject::connect(actionExport, SIGNAL(triggered()), EditDialog, SLOT(exportData()));
        QObject::connect(actionWordWrap, SIGNAL(toggled(bool)), EditDialog, SLOT(setWordWrapping(bool)));
        QObject::connect(actionImportInMenu, SIGNAL(triggered()), EditDialog, SLOT(importData()));

        editorStack->setCurrentIndex(0);
        buttonApply->setDefault(true);


        QMetaObject::connectSlotsByName(EditDialog);
    } // setupUi

    void retranslateUi(QDialog *EditDialog)
    {
        EditDialog->setWindowTitle(QCoreApplication::translate("EditDialog", "Edit database cell", nullptr));
#if QT_CONFIG(whatsthis)
        EditDialog->setWhatsThis(QCoreApplication::translate("EditDialog", "This area displays information about the data present in this database cell", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionPrint->setText(QCoreApplication::translate("EditDialog", "Print...", nullptr));
#if QT_CONFIG(tooltip)
        actionPrint->setToolTip(QCoreApplication::translate("EditDialog", "Open preview dialog for printing displayed text", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        actionPrint->setWhatsThis(QCoreApplication::translate("EditDialog", "Open preview dialog for printing the data currently stored in the cell", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        actionPrint->setShortcut(QCoreApplication::translate("EditDialog", "Ctrl+P", nullptr));
#endif // QT_CONFIG(shortcut)
        actionCopyHexAscii->setText(QCoreApplication::translate("EditDialog", "Copy Hex and ASCII", nullptr));
#if QT_CONFIG(tooltip)
        actionCopyHexAscii->setToolTip(QCoreApplication::translate("EditDialog", "Copy selected hexadecimal and ASCII columns to the clipboard", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        actionCopyHexAscii->setShortcut(QCoreApplication::translate("EditDialog", "Ctrl+Shift+C", nullptr));
#endif // QT_CONFIG(shortcut)
        actionIndent->setText(QCoreApplication::translate("EditDialog", "Autoformat", nullptr));
#if QT_CONFIG(tooltip)
        actionIndent->setToolTip(QCoreApplication::translate("EditDialog", "Auto-format: pretty print on loading, compact on saving.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        actionIndent->setWhatsThis(QCoreApplication::translate("EditDialog", "When enabled, the auto-format feature formats the data on loading, breaking the text in lines and indenting it for maximum readability. On data saving, the auto-format feature compacts the data removing end of lines, and unnecessary whitespace.", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionExport->setText(QCoreApplication::translate("EditDialog", "&Export...", nullptr));
#if QT_CONFIG(tooltip)
        actionExport->setToolTip(QCoreApplication::translate("EditDialog", "Export to file", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        actionExport->setWhatsThis(QCoreApplication::translate("EditDialog", "Opens a file dialog used to export the contents of this database cell to a file.", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionImport->setText(QCoreApplication::translate("EditDialog", "&Import...", nullptr));
#if QT_CONFIG(tooltip)
        actionImport->setToolTip(QCoreApplication::translate("EditDialog", "Import from file", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        actionImport->setWhatsThis(QCoreApplication::translate("EditDialog", "Opens a file dialog used to import any kind of data to this database cell.", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionNull->setText(QCoreApplication::translate("EditDialog", "Set as &NULL", nullptr));
#if QT_CONFIG(whatsthis)
        actionNull->setWhatsThis(QCoreApplication::translate("EditDialog", "Erases the contents of the cell", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionWordWrap->setText(QCoreApplication::translate("EditDialog", "Word Wrap", nullptr));
#if QT_CONFIG(tooltip)
        actionWordWrap->setToolTip(QCoreApplication::translate("EditDialog", "Wrap lines on word boundaries", nullptr));
#endif // QT_CONFIG(tooltip)
        actionOpenInApp->setText(QCoreApplication::translate("EditDialog", "Open in default application or browser", nullptr));
        actionOpenInApp->setIconText(QCoreApplication::translate("EditDialog", "Open in application", nullptr));
#if QT_CONFIG(tooltip)
        actionOpenInApp->setToolTip(QCoreApplication::translate("EditDialog", "Open in default application or browser", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        actionOpenInApp->setWhatsThis(QCoreApplication::translate("EditDialog", "The value is interpreted as a file or URL and opened in the default application or web browser.", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionImportAsLink->setText(QCoreApplication::translate("EditDialog", "Save file reference...", nullptr));
#if QT_CONFIG(tooltip)
        actionImportAsLink->setToolTip(QCoreApplication::translate("EditDialog", "Save reference to file", nullptr));
#endif // QT_CONFIG(tooltip)
        actionImportInMenu->setText(QCoreApplication::translate("EditDialog", "&Import...", nullptr));
#if QT_CONFIG(tooltip)
        actionImportInMenu->setToolTip(QCoreApplication::translate("EditDialog", "Import from file", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        actionImportInMenu->setWhatsThis(QCoreApplication::translate("EditDialog", "Opens a file dialog used to import any kind of data to this database cell.", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionOpenInExternal->setText(QCoreApplication::translate("EditDialog", "Open in external application", nullptr));
#if QT_CONFIG(tooltip)
        actionOpenInExternal->setToolTip(QCoreApplication::translate("EditDialog", "Open in external application", nullptr));
#endif // QT_CONFIG(tooltip)
        labelMode->setText(QCoreApplication::translate("EditDialog", "Mode:", nullptr));
        comboMode->setItemText(0, QCoreApplication::translate("EditDialog", "Text", nullptr));
        comboMode->setItemText(1, QCoreApplication::translate("EditDialog", "RTL Text", nullptr));
        comboMode->setItemText(2, QCoreApplication::translate("EditDialog", "Binary", nullptr));
        comboMode->setItemText(3, QCoreApplication::translate("EditDialog", "Image", nullptr));
        comboMode->setItemText(4, QCoreApplication::translate("EditDialog", "JSON", nullptr));
        comboMode->setItemText(5, QCoreApplication::translate("EditDialog", "XML", nullptr));
        comboMode->setItemText(6, QCoreApplication::translate("EditDialog", "Evaluation", nullptr));

#if QT_CONFIG(whatsthis)
        comboMode->setWhatsThis(QCoreApplication::translate("EditDialog", "This is the list of supported modes for the cell editor. Choose a mode for viewing or editing the data of the current cell.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(tooltip)
        buttonAutoSwitchMode->setToolTip(QCoreApplication::translate("EditDialog", "Automatically adjust the editor mode to the loaded data type", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        buttonAutoSwitchMode->setStatusTip(QCoreApplication::translate("EditDialog", "Automatically adjust the editor mode to the loaded data type", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        buttonAutoSwitchMode->setWhatsThis(QCoreApplication::translate("EditDialog", "This checkable button enables or disables the automatic switching of the editor mode. When a new cell is selected or new data is imported and the automatic switching is enabled, the mode adjusts to the detected data type. You can then change the editor mode manually. If you want to keep this manually switched mode while moving through the cells, switch the button off.", nullptr));
#endif // QT_CONFIG(whatsthis)
        buttonAutoSwitchMode->setText(QCoreApplication::translate("EditDialog", "Auto-switch", nullptr));
#if QT_CONFIG(whatsthis)
        qtEdit->setWhatsThis(QCoreApplication::translate("EditDialog", "This Qt editor is used for right-to-left scripts, which are not supported by the default Text editor. The presence of right-to-left characters is detected and this editor mode is automatically selected.", nullptr));
#endif // QT_CONFIG(whatsthis)
        labelCell->setText(QCoreApplication::translate("EditDialog", "Identification of the cell currently in the editor", nullptr));
        labelInfo->setText(QCoreApplication::translate("EditDialog", "Type and size of data currently in table", nullptr));
#if QT_CONFIG(tooltip)
        buttonApply->setToolTip(QCoreApplication::translate("EditDialog", "Apply data to cell", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        buttonApply->setWhatsThis(QCoreApplication::translate("EditDialog", "This button saves the changes performed in the cell editor to the database cell.", nullptr));
#endif // QT_CONFIG(whatsthis)
        buttonApply->setText(QCoreApplication::translate("EditDialog", "Apply", nullptr));
    } // retranslateUi

};

namespace Ui {
    class EditDialog: public Ui_EditDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EDITDIALOG_H
