/****************************************************************************
** Meta object code from reading C++ file 'EditTableDialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../src/EditTableDialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'EditTableDialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_EditTableDialog_t {
    QByteArrayData data[46];
    char stringdata0[680];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_EditTableDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_EditTableDialog_t qt_meta_stringdata_EditTableDialog = {
    {
QT_MOC_LITERAL(0, 0, 15), // "EditTableDialog"
QT_MOC_LITERAL(1, 16, 14), // "populateFields"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 24), // "populateIndexConstraints"
QT_MOC_LITERAL(4, 57, 19), // "populateForeignKeys"
QT_MOC_LITERAL(5, 77, 24), // "populateCheckConstraints"
QT_MOC_LITERAL(6, 102, 8), // "addField"
QT_MOC_LITERAL(7, 111, 11), // "removeField"
QT_MOC_LITERAL(8, 123, 21), // "fieldSelectionChanged"
QT_MOC_LITERAL(9, 145, 6), // "accept"
QT_MOC_LITERAL(10, 152, 6), // "reject"
QT_MOC_LITERAL(11, 159, 10), // "checkInput"
QT_MOC_LITERAL(12, 170, 16), // "fieldItemChanged"
QT_MOC_LITERAL(13, 187, 16), // "QTreeWidgetItem*"
QT_MOC_LITERAL(14, 204, 4), // "item"
QT_MOC_LITERAL(15, 209, 6), // "column"
QT_MOC_LITERAL(16, 216, 26), // "indexConstraintItemChanged"
QT_MOC_LITERAL(17, 243, 17), // "QTableWidgetItem*"
QT_MOC_LITERAL(18, 261, 21), // "foreignKeyItemChanged"
QT_MOC_LITERAL(19, 283, 26), // "checkConstraintItemChanged"
QT_MOC_LITERAL(20, 310, 32), // "indexConstraintItemDoubleClicked"
QT_MOC_LITERAL(21, 343, 27), // "foreignKeyItemDoubleClicked"
QT_MOC_LITERAL(22, 371, 22), // "updateTypeAndCollation"
QT_MOC_LITERAL(23, 394, 6), // "object"
QT_MOC_LITERAL(24, 401, 11), // "eventFilter"
QT_MOC_LITERAL(25, 413, 7), // "QEvent*"
QT_MOC_LITERAL(26, 421, 5), // "event"
QT_MOC_LITERAL(27, 427, 6), // "moveUp"
QT_MOC_LITERAL(28, 434, 8), // "moveDown"
QT_MOC_LITERAL(29, 443, 7), // "moveTop"
QT_MOC_LITERAL(30, 451, 10), // "moveBottom"
QT_MOC_LITERAL(31, 462, 15), // "setWithoutRowid"
QT_MOC_LITERAL(32, 478, 13), // "without_rowid"
QT_MOC_LITERAL(33, 492, 9), // "setStrict"
QT_MOC_LITERAL(34, 502, 6), // "strict"
QT_MOC_LITERAL(35, 509, 12), // "changeSchema"
QT_MOC_LITERAL(36, 522, 6), // "schema"
QT_MOC_LITERAL(37, 529, 13), // "setOnConflict"
QT_MOC_LITERAL(38, 543, 11), // "on_conflict"
QT_MOC_LITERAL(39, 555, 18), // "addIndexConstraint"
QT_MOC_LITERAL(40, 574, 11), // "primary_key"
QT_MOC_LITERAL(41, 586, 21), // "removeIndexConstraint"
QT_MOC_LITERAL(42, 608, 13), // "addForeignKey"
QT_MOC_LITERAL(43, 622, 16), // "removeForeignKey"
QT_MOC_LITERAL(44, 639, 18), // "addCheckConstraint"
QT_MOC_LITERAL(45, 658, 21) // "removeCheckConstraint"

    },
    "EditTableDialog\0populateFields\0\0"
    "populateIndexConstraints\0populateForeignKeys\0"
    "populateCheckConstraints\0addField\0"
    "removeField\0fieldSelectionChanged\0"
    "accept\0reject\0checkInput\0fieldItemChanged\0"
    "QTreeWidgetItem*\0item\0column\0"
    "indexConstraintItemChanged\0QTableWidgetItem*\0"
    "foreignKeyItemChanged\0checkConstraintItemChanged\0"
    "indexConstraintItemDoubleClicked\0"
    "foreignKeyItemDoubleClicked\0"
    "updateTypeAndCollation\0object\0eventFilter\0"
    "QEvent*\0event\0moveUp\0moveDown\0moveTop\0"
    "moveBottom\0setWithoutRowid\0without_rowid\0"
    "setStrict\0strict\0changeSchema\0schema\0"
    "setOnConflict\0on_conflict\0addIndexConstraint\0"
    "primary_key\0removeIndexConstraint\0"
    "addForeignKey\0removeForeignKey\0"
    "addCheckConstraint\0removeCheckConstraint"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_EditTableDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      33,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  179,    2, 0x08 /* Private */,
       3,    0,  180,    2, 0x08 /* Private */,
       4,    0,  181,    2, 0x08 /* Private */,
       5,    0,  182,    2, 0x08 /* Private */,
       6,    0,  183,    2, 0x08 /* Private */,
       7,    0,  184,    2, 0x08 /* Private */,
       8,    0,  185,    2, 0x08 /* Private */,
       9,    0,  186,    2, 0x08 /* Private */,
      10,    0,  187,    2, 0x08 /* Private */,
      11,    0,  188,    2, 0x08 /* Private */,
      12,    2,  189,    2, 0x08 /* Private */,
      16,    1,  194,    2, 0x08 /* Private */,
      18,    1,  197,    2, 0x08 /* Private */,
      19,    1,  200,    2, 0x08 /* Private */,
      20,    1,  203,    2, 0x08 /* Private */,
      21,    1,  206,    2, 0x08 /* Private */,
      22,    1,  209,    2, 0x08 /* Private */,
      24,    2,  212,    2, 0x08 /* Private */,
      22,    0,  217,    2, 0x08 /* Private */,
      27,    0,  218,    2, 0x08 /* Private */,
      28,    0,  219,    2, 0x08 /* Private */,
      29,    0,  220,    2, 0x08 /* Private */,
      30,    0,  221,    2, 0x08 /* Private */,
      31,    1,  222,    2, 0x08 /* Private */,
      33,    1,  225,    2, 0x08 /* Private */,
      35,    1,  228,    2, 0x08 /* Private */,
      37,    1,  231,    2, 0x08 /* Private */,
      39,    1,  234,    2, 0x08 /* Private */,
      41,    0,  237,    2, 0x08 /* Private */,
      42,    0,  238,    2, 0x08 /* Private */,
      43,    0,  239,    2, 0x08 /* Private */,
      44,    0,  240,    2, 0x08 /* Private */,
      45,    0,  241,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 13, QMetaType::Int,   14,   15,
    QMetaType::Void, 0x80000000 | 17,   14,
    QMetaType::Void, 0x80000000 | 17,   14,
    QMetaType::Void, 0x80000000 | 17,   14,
    QMetaType::Void, 0x80000000 | 17,   14,
    QMetaType::Void, 0x80000000 | 17,   14,
    QMetaType::Void, QMetaType::QObjectStar,   23,
    QMetaType::Bool, QMetaType::QObjectStar, 0x80000000 | 25,   23,   26,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   32,
    QMetaType::Void, QMetaType::Bool,   34,
    QMetaType::Void, QMetaType::QString,   36,
    QMetaType::Void, QMetaType::QString,   38,
    QMetaType::Void, QMetaType::Bool,   40,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void EditTableDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<EditTableDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->populateFields(); break;
        case 1: _t->populateIndexConstraints(); break;
        case 2: _t->populateForeignKeys(); break;
        case 3: _t->populateCheckConstraints(); break;
        case 4: _t->addField(); break;
        case 5: _t->removeField(); break;
        case 6: _t->fieldSelectionChanged(); break;
        case 7: _t->accept(); break;
        case 8: _t->reject(); break;
        case 9: _t->checkInput(); break;
        case 10: _t->fieldItemChanged((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 11: _t->indexConstraintItemChanged((*reinterpret_cast< QTableWidgetItem*(*)>(_a[1]))); break;
        case 12: _t->foreignKeyItemChanged((*reinterpret_cast< QTableWidgetItem*(*)>(_a[1]))); break;
        case 13: _t->checkConstraintItemChanged((*reinterpret_cast< QTableWidgetItem*(*)>(_a[1]))); break;
        case 14: _t->indexConstraintItemDoubleClicked((*reinterpret_cast< QTableWidgetItem*(*)>(_a[1]))); break;
        case 15: _t->foreignKeyItemDoubleClicked((*reinterpret_cast< QTableWidgetItem*(*)>(_a[1]))); break;
        case 16: _t->updateTypeAndCollation((*reinterpret_cast< QObject*(*)>(_a[1]))); break;
        case 17: { bool _r = _t->eventFilter((*reinterpret_cast< QObject*(*)>(_a[1])),(*reinterpret_cast< QEvent*(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 18: _t->updateTypeAndCollation(); break;
        case 19: _t->moveUp(); break;
        case 20: _t->moveDown(); break;
        case 21: _t->moveTop(); break;
        case 22: _t->moveBottom(); break;
        case 23: _t->setWithoutRowid((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 24: _t->setStrict((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 25: _t->changeSchema((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 26: _t->setOnConflict((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 27: _t->addIndexConstraint((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 28: _t->removeIndexConstraint(); break;
        case 29: _t->addForeignKey(); break;
        case 30: _t->removeForeignKey(); break;
        case 31: _t->addCheckConstraint(); break;
        case 32: _t->removeCheckConstraint(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject EditTableDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_EditTableDialog.data,
    qt_meta_data_EditTableDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *EditTableDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *EditTableDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_EditTableDialog.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int EditTableDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 33)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 33;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 33)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 33;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
