/****************************************************************************
** Meta object code from reading C++ file 'RemoteNetwork.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../src/RemoteNetwork.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'RemoteNetwork.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_RemoteNetwork_t {
    QByteArrayData data[14];
    char stringdata0[141];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_RemoteNetwork_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_RemoteNetwork_t qt_meta_stringdata_RemoteNetwork = {
    {
QT_MOC_LITERAL(0, 0, 13), // "RemoteNetwork"
QT_MOC_LITERAL(1, 14, 13), // "fetchFinished"
QT_MOC_LITERAL(2, 28, 0), // ""
QT_MOC_LITERAL(3, 29, 8), // "filename"
QT_MOC_LITERAL(4, 38, 8), // "identity"
QT_MOC_LITERAL(5, 47, 3), // "url"
QT_MOC_LITERAL(6, 51, 11), // "std::string"
QT_MOC_LITERAL(7, 63, 13), // "new_commit_id"
QT_MOC_LITERAL(8, 77, 6), // "branch"
QT_MOC_LITERAL(9, 84, 13), // "last_modified"
QT_MOC_LITERAL(10, 98, 10), // "QIODevice*"
QT_MOC_LITERAL(11, 109, 6), // "device"
QT_MOC_LITERAL(12, 116, 12), // "pushFinished"
QT_MOC_LITERAL(13, 129, 11) // "source_file"

    },
    "RemoteNetwork\0fetchFinished\0\0filename\0"
    "identity\0url\0std::string\0new_commit_id\0"
    "branch\0last_modified\0QIODevice*\0device\0"
    "pushFinished\0source_file"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_RemoteNetwork[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    7,   24,    2, 0x06 /* Public */,
      12,    6,   39,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::QUrl, 0x80000000 | 6, 0x80000000 | 6, QMetaType::QDateTime, 0x80000000 | 10,    3,    4,    5,    7,    8,    9,   11,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::QUrl, 0x80000000 | 6, 0x80000000 | 6, QMetaType::QString,    3,    4,    5,    7,    8,   13,

       0        // eod
};

void RemoteNetwork::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<RemoteNetwork *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->fetchFinished((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2])),(*reinterpret_cast< const QUrl(*)>(_a[3])),(*reinterpret_cast< std::string(*)>(_a[4])),(*reinterpret_cast< std::string(*)>(_a[5])),(*reinterpret_cast< QDateTime(*)>(_a[6])),(*reinterpret_cast< QIODevice*(*)>(_a[7]))); break;
        case 1: _t->pushFinished((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2])),(*reinterpret_cast< const QUrl(*)>(_a[3])),(*reinterpret_cast< std::string(*)>(_a[4])),(*reinterpret_cast< std::string(*)>(_a[5])),(*reinterpret_cast< QString(*)>(_a[6]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 6:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QIODevice* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (RemoteNetwork::*)(QString , QString , const QUrl & , std::string , std::string , QDateTime , QIODevice * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RemoteNetwork::fetchFinished)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (RemoteNetwork::*)(QString , QString , const QUrl & , std::string , std::string , QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RemoteNetwork::pushFinished)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject RemoteNetwork::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_RemoteNetwork.data,
    qt_meta_data_RemoteNetwork,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *RemoteNetwork::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *RemoteNetwork::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_RemoteNetwork.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int RemoteNetwork::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void RemoteNetwork::fetchFinished(QString _t1, QString _t2, const QUrl & _t3, std::string _t4, std::string _t5, QDateTime _t6, QIODevice * _t7)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t5))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t6))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t7))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void RemoteNetwork::pushFinished(QString _t1, QString _t2, const QUrl & _t3, std::string _t4, std::string _t5, QString _t6)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t5))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t6))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
