/****************************************************************************
** Meta object code from reading C++ file 'PreferencesDialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../src/PreferencesDialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'PreferencesDialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PreferencesDialog_t {
    QByteArrayData data[26];
    char stringdata0[391];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PreferencesDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PreferencesDialog_t qt_meta_stringdata_PreferencesDialog = {
    {
QT_MOC_LITERAL(0, 0, 17), // "PreferencesDialog"
QT_MOC_LITERAL(1, 18, 12), // "loadSettings"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 12), // "saveSettings"
QT_MOC_LITERAL(4, 45, 6), // "accept"
QT_MOC_LITERAL(5, 52, 14), // "chooseLocation"
QT_MOC_LITERAL(6, 67, 16), // "showColourDialog"
QT_MOC_LITERAL(7, 84, 16), // "QTreeWidgetItem*"
QT_MOC_LITERAL(8, 101, 4), // "item"
QT_MOC_LITERAL(9, 106, 6), // "column"
QT_MOC_LITERAL(10, 113, 12), // "addExtension"
QT_MOC_LITERAL(11, 126, 26), // "createBuiltinExtensionList"
QT_MOC_LITERAL(12, 153, 15), // "removeExtension"
QT_MOC_LITERAL(13, 169, 17), // "activateRemoteTab"
QT_MOC_LITERAL(14, 187, 6), // "active"
QT_MOC_LITERAL(15, 194, 20), // "addClientCertificate"
QT_MOC_LITERAL(16, 215, 23), // "removeClientCertificate"
QT_MOC_LITERAL(17, 239, 26), // "chooseRemoteCloneDirectory"
QT_MOC_LITERAL(18, 266, 17), // "updatePreviewFont"
QT_MOC_LITERAL(19, 284, 19), // "adjustColorsToStyle"
QT_MOC_LITERAL(20, 304, 5), // "style"
QT_MOC_LITERAL(21, 310, 14), // "configureProxy"
QT_MOC_LITERAL(22, 325, 24), // "showFileExtensionManager"
QT_MOC_LITERAL(23, 350, 16), // "buttonBoxClicked"
QT_MOC_LITERAL(24, 367, 16), // "QAbstractButton*"
QT_MOC_LITERAL(25, 384, 6) // "button"

    },
    "PreferencesDialog\0loadSettings\0\0"
    "saveSettings\0accept\0chooseLocation\0"
    "showColourDialog\0QTreeWidgetItem*\0"
    "item\0column\0addExtension\0"
    "createBuiltinExtensionList\0removeExtension\0"
    "activateRemoteTab\0active\0addClientCertificate\0"
    "removeClientCertificate\0"
    "chooseRemoteCloneDirectory\0updatePreviewFont\0"
    "adjustColorsToStyle\0style\0configureProxy\0"
    "showFileExtensionManager\0buttonBoxClicked\0"
    "QAbstractButton*\0button"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PreferencesDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      17,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   99,    2, 0x08 /* Private */,
       3,    1,  100,    2, 0x08 /* Private */,
       3,    0,  103,    2, 0x28 /* Private | MethodCloned */,
       5,    0,  104,    2, 0x08 /* Private */,
       6,    2,  105,    2, 0x08 /* Private */,
      10,    0,  110,    2, 0x08 /* Private */,
      11,    0,  111,    2, 0x08 /* Private */,
      12,    0,  112,    2, 0x08 /* Private */,
      13,    1,  113,    2, 0x08 /* Private */,
      15,    0,  116,    2, 0x08 /* Private */,
      16,    0,  117,    2, 0x08 /* Private */,
      17,    0,  118,    2, 0x08 /* Private */,
      18,    0,  119,    2, 0x08 /* Private */,
      19,    1,  120,    2, 0x08 /* Private */,
      21,    0,  123,    2, 0x08 /* Private */,
      22,    0,  124,    2, 0x08 /* Private */,
      23,    1,  125,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    4,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 7, QMetaType::Int,    8,    9,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   14,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   20,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 24,   25,

       0        // eod
};

void PreferencesDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PreferencesDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->loadSettings(); break;
        case 1: _t->saveSettings((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->saveSettings(); break;
        case 3: _t->chooseLocation(); break;
        case 4: _t->showColourDialog((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 5: _t->addExtension(); break;
        case 6: _t->createBuiltinExtensionList(); break;
        case 7: _t->removeExtension(); break;
        case 8: _t->activateRemoteTab((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 9: _t->addClientCertificate(); break;
        case 10: _t->removeClientCertificate(); break;
        case 11: _t->chooseRemoteCloneDirectory(); break;
        case 12: _t->updatePreviewFont(); break;
        case 13: _t->adjustColorsToStyle((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: _t->configureProxy(); break;
        case 15: _t->showFileExtensionManager(); break;
        case 16: _t->buttonBoxClicked((*reinterpret_cast< QAbstractButton*(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PreferencesDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_PreferencesDialog.data,
    qt_meta_data_PreferencesDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PreferencesDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PreferencesDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PreferencesDialog.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int PreferencesDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 17;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
