/****************************************************************************
** Meta object code from reading C++ file 'EditIndexDialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../src/EditIndexDialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'EditIndexDialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_EditIndexDialog_t {
    QByteArrayData data[15];
    char stringdata0[168];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_EditIndexDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_EditIndexDialog_t qt_meta_stringdata_EditIndexDialog = {
    {
QT_MOC_LITERAL(0, 0, 15), // "EditIndexDialog"
QT_MOC_LITERAL(1, 16, 6), // "accept"
QT_MOC_LITERAL(2, 23, 0), // ""
QT_MOC_LITERAL(3, 24, 6), // "reject"
QT_MOC_LITERAL(4, 31, 12), // "tableChanged"
QT_MOC_LITERAL(5, 44, 9), // "new_table"
QT_MOC_LITERAL(6, 54, 11), // "initialLoad"
QT_MOC_LITERAL(7, 66, 10), // "checkInput"
QT_MOC_LITERAL(8, 77, 10), // "addToIndex"
QT_MOC_LITERAL(9, 88, 11), // "QModelIndex"
QT_MOC_LITERAL(10, 100, 3), // "idx"
QT_MOC_LITERAL(11, 104, 15), // "removeFromIndex"
QT_MOC_LITERAL(12, 120, 12), // "moveColumnUp"
QT_MOC_LITERAL(13, 133, 14), // "moveColumnDown"
QT_MOC_LITERAL(14, 148, 19) // "addExpressionColumn"

    },
    "EditIndexDialog\0accept\0\0reject\0"
    "tableChanged\0new_table\0initialLoad\0"
    "checkInput\0addToIndex\0QModelIndex\0idx\0"
    "removeFromIndex\0moveColumnUp\0"
    "moveColumnDown\0addExpressionColumn"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_EditIndexDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   74,    2, 0x08 /* Private */,
       3,    0,   75,    2, 0x08 /* Private */,
       4,    2,   76,    2, 0x08 /* Private */,
       4,    1,   81,    2, 0x28 /* Private | MethodCloned */,
       7,    0,   84,    2, 0x08 /* Private */,
       8,    1,   85,    2, 0x08 /* Private */,
       8,    0,   88,    2, 0x28 /* Private | MethodCloned */,
      11,    1,   89,    2, 0x08 /* Private */,
      11,    0,   92,    2, 0x28 /* Private | MethodCloned */,
      12,    0,   93,    2, 0x08 /* Private */,
      13,    0,   94,    2, 0x08 /* Private */,
      14,    0,   95,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::Bool,    5,    6,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void EditIndexDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<EditIndexDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->accept(); break;
        case 1: _t->reject(); break;
        case 2: _t->tableChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 3: _t->tableChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->checkInput(); break;
        case 5: _t->addToIndex((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 6: _t->addToIndex(); break;
        case 7: _t->removeFromIndex((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 8: _t->removeFromIndex(); break;
        case 9: _t->moveColumnUp(); break;
        case 10: _t->moveColumnDown(); break;
        case 11: _t->addExpressionColumn(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject EditIndexDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_EditIndexDialog.data,
    qt_meta_data_EditIndexDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *EditIndexDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *EditIndexDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_EditIndexDialog.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int EditIndexDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
