/****************************************************************************
** Meta object code from reading C++ file 'PlotDock.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../src/PlotDock.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'PlotDock.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PlotDock_t {
    QByteArrayData data[39];
    char stringdata0[461];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PlotDock_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PlotDock_t qt_meta_stringdata_PlotDock = {
    {
QT_MOC_LITERAL(0, 0, 8), // "PlotDock"
QT_MOC_LITERAL(1, 9, 14), // "pointsSelected"
QT_MOC_LITERAL(2, 24, 0), // ""
QT_MOC_LITERAL(3, 25, 10), // "firstIndex"
QT_MOC_LITERAL(4, 36, 5), // "count"
QT_MOC_LITERAL(5, 42, 10), // "updatePlot"
QT_MOC_LITERAL(6, 53, 17), // "SqliteTableModel*"
QT_MOC_LITERAL(7, 71, 5), // "model"
QT_MOC_LITERAL(8, 77, 24), // "BrowseDataTableSettings*"
QT_MOC_LITERAL(9, 102, 8), // "settings"
QT_MOC_LITERAL(10, 111, 6), // "update"
QT_MOC_LITERAL(11, 118, 20), // "keepOrResetSelection"
QT_MOC_LITERAL(12, 139, 12), // "fetchAllData"
QT_MOC_LITERAL(13, 152, 9), // "resetPlot"
QT_MOC_LITERAL(14, 162, 6), // "reject"
QT_MOC_LITERAL(15, 169, 17), // "columnItemChanged"
QT_MOC_LITERAL(16, 187, 16), // "QTreeWidgetItem*"
QT_MOC_LITERAL(17, 204, 4), // "item"
QT_MOC_LITERAL(18, 209, 6), // "column"
QT_MOC_LITERAL(19, 216, 23), // "columnItemDoubleClicked"
QT_MOC_LITERAL(20, 240, 8), // "savePlot"
QT_MOC_LITERAL(21, 249, 15), // "lineTypeChanged"
QT_MOC_LITERAL(22, 265, 5), // "index"
QT_MOC_LITERAL(23, 271, 17), // "pointShapeChanged"
QT_MOC_LITERAL(24, 289, 16), // "selectionChanged"
QT_MOC_LITERAL(25, 306, 10), // "mousePress"
QT_MOC_LITERAL(26, 317, 10), // "mouseWheel"
QT_MOC_LITERAL(27, 328, 9), // "mouseMove"
QT_MOC_LITERAL(28, 338, 12), // "QMouseEvent*"
QT_MOC_LITERAL(29, 351, 5), // "event"
QT_MOC_LITERAL(30, 357, 4), // "copy"
QT_MOC_LITERAL(31, 362, 19), // "toggleLegendVisible"
QT_MOC_LITERAL(32, 382, 7), // "visible"
QT_MOC_LITERAL(33, 390, 17), // "toggleStackedBars"
QT_MOC_LITERAL(34, 408, 7), // "stacked"
QT_MOC_LITERAL(35, 416, 15), // "openPrintDialog"
QT_MOC_LITERAL(36, 432, 10), // "renderPlot"
QT_MOC_LITERAL(37, 443, 9), // "QPrinter*"
QT_MOC_LITERAL(38, 453, 7) // "printer"

    },
    "PlotDock\0pointsSelected\0\0firstIndex\0"
    "count\0updatePlot\0SqliteTableModel*\0"
    "model\0BrowseDataTableSettings*\0settings\0"
    "update\0keepOrResetSelection\0fetchAllData\0"
    "resetPlot\0reject\0columnItemChanged\0"
    "QTreeWidgetItem*\0item\0column\0"
    "columnItemDoubleClicked\0savePlot\0"
    "lineTypeChanged\0index\0pointShapeChanged\0"
    "selectionChanged\0mousePress\0mouseWheel\0"
    "mouseMove\0QMouseEvent*\0event\0copy\0"
    "toggleLegendVisible\0visible\0"
    "toggleStackedBars\0stacked\0openPrintDialog\0"
    "renderPlot\0QPrinter*\0printer"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PlotDock[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      22,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,  124,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    4,  129,    2, 0x0a /* Public */,
       5,    3,  138,    2, 0x2a /* Public | MethodCloned */,
       5,    2,  145,    2, 0x2a /* Public | MethodCloned */,
       5,    1,  150,    2, 0x2a /* Public | MethodCloned */,
      12,    0,  153,    2, 0x0a /* Public */,
      13,    0,  154,    2, 0x0a /* Public */,
      14,    0,  155,    2, 0x0a /* Public */,
      15,    2,  156,    2, 0x08 /* Private */,
      19,    2,  161,    2, 0x08 /* Private */,
      20,    0,  166,    2, 0x08 /* Private */,
      21,    1,  167,    2, 0x08 /* Private */,
      23,    1,  170,    2, 0x08 /* Private */,
      24,    0,  173,    2, 0x08 /* Private */,
      25,    0,  174,    2, 0x08 /* Private */,
      26,    0,  175,    2, 0x08 /* Private */,
      27,    1,  176,    2, 0x08 /* Private */,
      30,    0,  179,    2, 0x08 /* Private */,
      31,    1,  180,    2, 0x08 /* Private */,
      33,    1,  183,    2, 0x08 /* Private */,
      35,    0,  186,    2, 0x08 /* Private */,
      36,    1,  187,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::Int,    3,    4,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 6, 0x80000000 | 8, QMetaType::Bool, QMetaType::Bool,    7,    9,   10,   11,
    QMetaType::Void, 0x80000000 | 6, 0x80000000 | 8, QMetaType::Bool,    7,    9,   10,
    QMetaType::Void, 0x80000000 | 6, 0x80000000 | 8,    7,    9,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 16, QMetaType::Int,   17,   18,
    QMetaType::Void, 0x80000000 | 16, QMetaType::Int,   17,   18,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   22,
    QMetaType::Void, QMetaType::Int,   22,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 28,   29,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   32,
    QMetaType::Void, QMetaType::Bool,   34,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 37,   38,

       0        // eod
};

void PlotDock::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PlotDock *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->pointsSelected((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 1: _t->updatePlot((*reinterpret_cast< SqliteTableModel*(*)>(_a[1])),(*reinterpret_cast< BrowseDataTableSettings*(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3])),(*reinterpret_cast< bool(*)>(_a[4]))); break;
        case 2: _t->updatePlot((*reinterpret_cast< SqliteTableModel*(*)>(_a[1])),(*reinterpret_cast< BrowseDataTableSettings*(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 3: _t->updatePlot((*reinterpret_cast< SqliteTableModel*(*)>(_a[1])),(*reinterpret_cast< BrowseDataTableSettings*(*)>(_a[2]))); break;
        case 4: _t->updatePlot((*reinterpret_cast< SqliteTableModel*(*)>(_a[1]))); break;
        case 5: _t->fetchAllData(); break;
        case 6: _t->resetPlot(); break;
        case 7: _t->reject(); break;
        case 8: _t->columnItemChanged((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 9: _t->columnItemDoubleClicked((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 10: _t->savePlot(); break;
        case 11: _t->lineTypeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 12: _t->pointShapeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 13: _t->selectionChanged(); break;
        case 14: _t->mousePress(); break;
        case 15: _t->mouseWheel(); break;
        case 16: _t->mouseMove((*reinterpret_cast< QMouseEvent*(*)>(_a[1]))); break;
        case 17: _t->copy(); break;
        case 18: _t->toggleLegendVisible((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 19: _t->toggleStackedBars((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 20: _t->openPrintDialog(); break;
        case 21: _t->renderPlot((*reinterpret_cast< QPrinter*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PlotDock::*)(int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PlotDock::pointsSelected)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PlotDock::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_PlotDock.data,
    qt_meta_data_PlotDock,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PlotDock::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PlotDock::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PlotDock.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int PlotDock::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 22;
    }
    return _id;
}

// SIGNAL 0
void PlotDock::pointsSelected(int _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
