/****************************************************************************
** Meta object code from reading C++ file 'CondFormatManager.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../src/CondFormatManager.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'CondFormatManager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_CondFormatManager_t {
    QByteArrayData data[18];
    char stringdata0[186];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CondFormatManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CondFormatManager_t qt_meta_stringdata_CondFormatManager = {
    {
QT_MOC_LITERAL(0, 0, 17), // "CondFormatManager"
QT_MOC_LITERAL(1, 18, 10), // "addNewItem"
QT_MOC_LITERAL(2, 29, 0), // ""
QT_MOC_LITERAL(3, 30, 7), // "addItem"
QT_MOC_LITERAL(4, 38, 10), // "CondFormat"
QT_MOC_LITERAL(5, 49, 11), // "aCondFormat"
QT_MOC_LITERAL(6, 61, 10), // "removeItem"
QT_MOC_LITERAL(7, 72, 8), // "moveItem"
QT_MOC_LITERAL(8, 81, 6), // "offset"
QT_MOC_LITERAL(9, 88, 6), // "upItem"
QT_MOC_LITERAL(10, 95, 8), // "downItem"
QT_MOC_LITERAL(11, 104, 16), // "buttonBoxClicked"
QT_MOC_LITERAL(12, 121, 16), // "QAbstractButton*"
QT_MOC_LITERAL(13, 138, 6), // "button"
QT_MOC_LITERAL(14, 145, 11), // "itemClicked"
QT_MOC_LITERAL(15, 157, 16), // "QTreeWidgetItem*"
QT_MOC_LITERAL(16, 174, 4), // "item"
QT_MOC_LITERAL(17, 179, 6) // "column"

    },
    "CondFormatManager\0addNewItem\0\0addItem\0"
    "CondFormat\0aCondFormat\0removeItem\0"
    "moveItem\0offset\0upItem\0downItem\0"
    "buttonBoxClicked\0QAbstractButton*\0"
    "button\0itemClicked\0QTreeWidgetItem*\0"
    "item\0column"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CondFormatManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   54,    2, 0x08 /* Private */,
       3,    1,   55,    2, 0x08 /* Private */,
       6,    0,   58,    2, 0x08 /* Private */,
       7,    1,   59,    2, 0x08 /* Private */,
       9,    0,   62,    2, 0x08 /* Private */,
      10,    0,   63,    2, 0x08 /* Private */,
      11,    1,   64,    2, 0x08 /* Private */,
      14,    2,   67,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 15, QMetaType::Int,   16,   17,

       0        // eod
};

void CondFormatManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CondFormatManager *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->addNewItem(); break;
        case 1: _t->addItem((*reinterpret_cast< const CondFormat(*)>(_a[1]))); break;
        case 2: _t->removeItem(); break;
        case 3: _t->moveItem((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->upItem(); break;
        case 5: _t->downItem(); break;
        case 6: _t->buttonBoxClicked((*reinterpret_cast< QAbstractButton*(*)>(_a[1]))); break;
        case 7: _t->itemClicked((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject CondFormatManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_CondFormatManager.data,
    qt_meta_data_CondFormatManager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CondFormatManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CondFormatManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CondFormatManager.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int CondFormatManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
