/********************************************************************************
** Form generated from reading UI file 'ImageViewer.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_IMAGEVIEWER_H
#define UI_IMAGEVIEWER_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ImageViewer
{
public:
    QAction *actionPrintImage;
    QVBoxLayout *verticalLayout;
    QScrollArea *scrollArea;
    QLabel *labelView;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QSlider *sliderScale;
    QToolButton *buttonOriginalSize;
    QToolButton *buttonFitToWindow;

    void setupUi(QWidget *ImageViewer)
    {
        if (ImageViewer->objectName().isEmpty())
            ImageViewer->setObjectName(QString::fromUtf8("ImageViewer"));
        ImageViewer->resize(433, 307);
        actionPrintImage = new QAction(ImageViewer);
        actionPrintImage->setObjectName(QString::fromUtf8("actionPrintImage"));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/icons/print"), QSize(), QIcon::Normal, QIcon::Off);
        actionPrintImage->setIcon(icon);
        verticalLayout = new QVBoxLayout(ImageViewer);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        scrollArea = new QScrollArea(ImageViewer);
        scrollArea->setObjectName(QString::fromUtf8("scrollArea"));
        scrollArea->setAlignment(Qt::AlignCenter);
        labelView = new QLabel();
        labelView->setObjectName(QString::fromUtf8("labelView"));
        labelView->setGeometry(QRect(0, 0, 431, 280));
        QSizePolicy sizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(labelView->sizePolicy().hasHeightForWidth());
        labelView->setSizePolicy(sizePolicy);
        labelView->setContextMenuPolicy(Qt::ActionsContextMenu);
        labelView->setScaledContents(true);
        scrollArea->setWidget(labelView);

        verticalLayout->addWidget(scrollArea);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        sliderScale = new QSlider(ImageViewer);
        sliderScale->setObjectName(QString::fromUtf8("sliderScale"));
        sliderScale->setMinimum(1);
        sliderScale->setMaximum(800);
        sliderScale->setValue(100);
        sliderScale->setOrientation(Qt::Horizontal);

        horizontalLayout->addWidget(sliderScale);

        buttonOriginalSize = new QToolButton(ImageViewer);
        buttonOriginalSize->setObjectName(QString::fromUtf8("buttonOriginalSize"));
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/icons/view"), QSize(), QIcon::Normal, QIcon::Off);
        buttonOriginalSize->setIcon(icon1);

        horizontalLayout->addWidget(buttonOriginalSize);

        buttonFitToWindow = new QToolButton(ImageViewer);
        buttonFitToWindow->setObjectName(QString::fromUtf8("buttonFitToWindow"));
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/icons/monitor_link"), QSize(), QIcon::Normal, QIcon::Off);
        buttonFitToWindow->setIcon(icon2);
        buttonFitToWindow->setCheckable(true);

        horizontalLayout->addWidget(buttonFitToWindow);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(ImageViewer);
        QObject::connect(actionPrintImage, SIGNAL(triggered()), ImageViewer, SLOT(openPrintImageDialog()));
        QObject::connect(sliderScale, SIGNAL(valueChanged(int)), ImageViewer, SLOT(scaleImage(int)));
        QObject::connect(buttonFitToWindow, SIGNAL(toggled(bool)), ImageViewer, SLOT(scaleToFitWindow(bool)));

        QMetaObject::connectSlotsByName(ImageViewer);
    } // setupUi

    void retranslateUi(QWidget *ImageViewer)
    {
        ImageViewer->setWindowTitle(QCoreApplication::translate("ImageViewer", "Image Viewer", nullptr));
        actionPrintImage->setText(QCoreApplication::translate("ImageViewer", "Print...", nullptr));
#if QT_CONFIG(tooltip)
        actionPrintImage->setToolTip(QCoreApplication::translate("ImageViewer", "Open preview dialog for printing displayed image", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        actionPrintImage->setShortcut(QCoreApplication::translate("ImageViewer", "Ctrl+P", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        buttonOriginalSize->setToolTip(QCoreApplication::translate("ImageViewer", "Reset the scaling to match the original size of the image.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        buttonFitToWindow->setToolTip(QCoreApplication::translate("ImageViewer", "Set the scaling to match the size of the viewport.", nullptr));
#endif // QT_CONFIG(tooltip)
    } // retranslateUi

};

namespace Ui {
    class ImageViewer: public Ui_ImageViewer {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_IMAGEVIEWER_H
