/****************************************************************************
** Meta object code from reading C++ file 'RemoteDock.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../src/RemoteDock.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'RemoteDock.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_RemoteDock_t {
    QByteArrayData data[34];
    char stringdata0[428];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_RemoteDock_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_RemoteDock_t qt_meta_stringdata_RemoteDock = {
    {
QT_MOC_LITERAL(0, 0, 10), // "RemoteDock"
QT_MOC_LITERAL(1, 11, 8), // "openFile"
QT_MOC_LITERAL(2, 20, 0), // ""
QT_MOC_LITERAL(3, 21, 4), // "file"
QT_MOC_LITERAL(4, 26, 6), // "reject"
QT_MOC_LITERAL(5, 33, 14), // "setNewIdentity"
QT_MOC_LITERAL(6, 48, 8), // "identity"
QT_MOC_LITERAL(7, 57, 13), // "fetchDatabase"
QT_MOC_LITERAL(8, 71, 11), // "QModelIndex"
QT_MOC_LITERAL(9, 83, 3), // "idx"
QT_MOC_LITERAL(10, 87, 3), // "url"
QT_MOC_LITERAL(11, 91, 26), // "RemoteNetwork::RequestType"
QT_MOC_LITERAL(12, 118, 12), // "request_type"
QT_MOC_LITERAL(13, 131, 11), // "fetchCommit"
QT_MOC_LITERAL(14, 143, 27), // "pushCurrentlyOpenedDatabase"
QT_MOC_LITERAL(15, 171, 25), // "pushSelectedLocalDatabase"
QT_MOC_LITERAL(16, 197, 16), // "newDirectoryNode"
QT_MOC_LITERAL(17, 214, 6), // "parent"
QT_MOC_LITERAL(18, 221, 16), // "switchToMainView"
QT_MOC_LITERAL(19, 238, 13), // "openLocalFile"
QT_MOC_LITERAL(20, 252, 19), // "deleteLocalDatabase"
QT_MOC_LITERAL(21, 272, 5), // "index"
QT_MOC_LITERAL(22, 278, 28), // "openCurrentDatabaseInBrowser"
QT_MOC_LITERAL(23, 307, 7), // "refresh"
QT_MOC_LITERAL(24, 315, 12), // "pushFinished"
QT_MOC_LITERAL(25, 328, 8), // "filename"
QT_MOC_LITERAL(26, 337, 11), // "std::string"
QT_MOC_LITERAL(27, 349, 13), // "new_commit_id"
QT_MOC_LITERAL(28, 363, 6), // "branch"
QT_MOC_LITERAL(29, 370, 11), // "source_file"
QT_MOC_LITERAL(30, 382, 13), // "fetchFinished"
QT_MOC_LITERAL(31, 396, 13), // "last_modified"
QT_MOC_LITERAL(32, 410, 10), // "QIODevice*"
QT_MOC_LITERAL(33, 421, 6) // "device"

    },
    "RemoteDock\0openFile\0\0file\0reject\0"
    "setNewIdentity\0identity\0fetchDatabase\0"
    "QModelIndex\0idx\0url\0RemoteNetwork::RequestType\0"
    "request_type\0fetchCommit\0"
    "pushCurrentlyOpenedDatabase\0"
    "pushSelectedLocalDatabase\0newDirectoryNode\0"
    "parent\0switchToMainView\0openLocalFile\0"
    "deleteLocalDatabase\0index\0"
    "openCurrentDatabaseInBrowser\0refresh\0"
    "pushFinished\0filename\0std::string\0"
    "new_commit_id\0branch\0source_file\0"
    "fetchFinished\0last_modified\0QIODevice*\0"
    "device"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_RemoteDock[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      19,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  109,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,  112,    2, 0x0a /* Public */,
       5,    1,  113,    2, 0x08 /* Private */,
       7,    1,  116,    2, 0x08 /* Private */,
       7,    2,  119,    2, 0x08 /* Private */,
       7,    1,  124,    2, 0x28 /* Private | MethodCloned */,
       7,    0,  127,    2, 0x28 /* Private | MethodCloned */,
      13,    2,  128,    2, 0x08 /* Private */,
      13,    1,  133,    2, 0x28 /* Private | MethodCloned */,
      14,    0,  136,    2, 0x08 /* Private */,
      15,    0,  137,    2, 0x08 /* Private */,
      16,    1,  138,    2, 0x08 /* Private */,
      18,    0,  141,    2, 0x08 /* Private */,
      19,    1,  142,    2, 0x08 /* Private */,
      20,    1,  145,    2, 0x08 /* Private */,
      22,    0,  148,    2, 0x08 /* Private */,
      23,    0,  149,    2, 0x08 /* Private */,
      24,    6,  150,    2, 0x08 /* Private */,
      30,    7,  163,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    6,
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 11,   10,   12,
    QMetaType::Void, QMetaType::QString,   10,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 8, 0x80000000 | 11,    9,   12,
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 8,   17,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void, 0x80000000 | 8,   21,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::QUrl, 0x80000000 | 26, 0x80000000 | 26, QMetaType::QString,   25,    6,   10,   27,   28,   29,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::QUrl, 0x80000000 | 26, 0x80000000 | 26, QMetaType::QDateTime, 0x80000000 | 32,   25,    6,   10,   27,   28,   31,   33,

       0        // eod
};

void RemoteDock::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<RemoteDock *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->openFile((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 1: _t->reject(); break;
        case 2: _t->setNewIdentity((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->fetchDatabase((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 4: _t->fetchDatabase((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< RemoteNetwork::RequestType(*)>(_a[2]))); break;
        case 5: _t->fetchDatabase((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 6: _t->fetchDatabase(); break;
        case 7: _t->fetchCommit((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< RemoteNetwork::RequestType(*)>(_a[2]))); break;
        case 8: _t->fetchCommit((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 9: _t->pushCurrentlyOpenedDatabase(); break;
        case 10: _t->pushSelectedLocalDatabase(); break;
        case 11: _t->newDirectoryNode((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 12: _t->switchToMainView(); break;
        case 13: _t->openLocalFile((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 14: _t->deleteLocalDatabase((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 15: _t->openCurrentDatabaseInBrowser(); break;
        case 16: _t->refresh(); break;
        case 17: _t->pushFinished((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QUrl(*)>(_a[3])),(*reinterpret_cast< const std::string(*)>(_a[4])),(*reinterpret_cast< const std::string(*)>(_a[5])),(*reinterpret_cast< const QString(*)>(_a[6]))); break;
        case 18: _t->fetchFinished((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QUrl(*)>(_a[3])),(*reinterpret_cast< const std::string(*)>(_a[4])),(*reinterpret_cast< const std::string(*)>(_a[5])),(*reinterpret_cast< const QDateTime(*)>(_a[6])),(*reinterpret_cast< QIODevice*(*)>(_a[7]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 6:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QIODevice* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (RemoteDock::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RemoteDock::openFile)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject RemoteDock::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_RemoteDock.data,
    qt_meta_data_RemoteDock,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *RemoteDock::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *RemoteDock::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_RemoteDock.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int RemoteDock::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    return _id;
}

// SIGNAL 0
void RemoteDock::openFile(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
