/********************************************************************************
** Form generated from reading UI file 'TableBrowser.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TABLEBROWSER_H
#define UI_TABLEBROWSER_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFontComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QToolBar>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ExtendedTableWidget.h"
#include "FilterLineEdit.h"

QT_BEGIN_NAMESPACE

class Ui_TableBrowser
{
public:
    QAction *actionShowRowidColumn;
    QAction *actionUnlockViewEditing;
    QAction *actionBrowseTableEditDisplayFormat;
    QAction *actionNewRecord;
    QAction *actionDeleteRecord;
    QAction *newRecordAction;
    QAction *insertValuesAction;
    QAction *actionFilteredTableExportCsv;
    QAction *actionFilteredTableExportJson;
    QAction *actionFilterSaveAsView;
    QAction *actionSaveFilterAsPopup;
    QAction *actionHideColumns;
    QAction *actionShowAllColumns;
    QAction *actionSetTableEncoding;
    QAction *actionSetAllTablesEncoding;
    QAction *actionCopyColumnName;
    QAction *actionClearFilters;
    QAction *actionClearSorting;
    QAction *actionPrintTable;
    QAction *actionAddDock;
    QAction *actionRefresh;
    QAction *actionFind;
    QAction *actionBold;
    QAction *actionItalic;
    QAction *actionUnderline;
    QAction *actionRightAlign;
    QAction *actionLeftAlign;
    QAction *actionCenter;
    QAction *actionJustify;
    QAction *actionEditCondFormats;
    QAction *actionClearFormat;
    QAction *actionFontColor;
    QAction *actionBackgroundColor;
    QAction *actionToggleFormatToolbar;
    QAction *actionSelectColumn;
    QAction *actionReplace;
    QAction *actionFreezeColumns;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QComboBox *comboBrowseTable;
    QToolBar *browseToolbar;
    FilterLineEdit *editGlobalFilter;
    QSpacerItem *horizontalSpacer;
    QFrame *formatFrame;
    QHBoxLayout *horizontalLayoutFormat;
    QFontComboBox *fontComboBox;
    QSpinBox *fontSizeBox;
    QToolBar *formatToolbar;
    ExtendedTableWidget *dataTable;
    QFrame *frameFind;
    QVBoxLayout *vboxLayout;
    QWidget *widgetFind;
    QHBoxLayout *layoutFind;
    QLineEdit *editFindExpression;
    QToolButton *buttonFindPrevious;
    QToolButton *buttonFindNext;
    QCheckBox *checkFindCaseSensitive;
    QCheckBox *checkFindWholeCell;
    QCheckBox *checkFindRegEx;
    QSpacerItem *horizontalSpacer_1;
    QToolButton *buttonFindClose;
    QWidget *widgetReplace;
    QHBoxLayout *layoutReplace;
    QLineEdit *editReplaceExpression;
    QToolButton *buttonReplaceNext;
    QToolButton *buttonReplaceAll;
    QSpacerItem *horizontalSpacer_3;
    QHBoxLayout *horizontalLayout_2;
    QToolButton *buttonBegin;
    QToolButton *buttonPrevious;
    QLabel *labelRecordset;
    QToolButton *buttonNext;
    QToolButton *buttonEnd;
    QSpacerItem *horizontalSpacer_2;
    QPushButton *buttonGoto;
    QLineEdit *editGoto;

    void setupUi(QWidget *TableBrowser)
    {
        if (TableBrowser->objectName().isEmpty())
            TableBrowser->setObjectName(QString::fromUtf8("TableBrowser"));
        TableBrowser->resize(695, 400);
        actionShowRowidColumn = new QAction(TableBrowser);
        actionShowRowidColumn->setObjectName(QString::fromUtf8("actionShowRowidColumn"));
        actionShowRowidColumn->setCheckable(true);
        actionUnlockViewEditing = new QAction(TableBrowser);
        actionUnlockViewEditing->setObjectName(QString::fromUtf8("actionUnlockViewEditing"));
        actionUnlockViewEditing->setCheckable(true);
        actionBrowseTableEditDisplayFormat = new QAction(TableBrowser);
        actionBrowseTableEditDisplayFormat->setObjectName(QString::fromUtf8("actionBrowseTableEditDisplayFormat"));
        actionNewRecord = new QAction(TableBrowser);
        actionNewRecord->setObjectName(QString::fromUtf8("actionNewRecord"));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/icons/add_record"), QSize(), QIcon::Normal, QIcon::Off);
        actionNewRecord->setIcon(icon);
        actionDeleteRecord = new QAction(TableBrowser);
        actionDeleteRecord->setObjectName(QString::fromUtf8("actionDeleteRecord"));
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/icons/delete_record"), QSize(), QIcon::Normal, QIcon::Off);
        actionDeleteRecord->setIcon(icon1);
        newRecordAction = new QAction(TableBrowser);
        newRecordAction->setObjectName(QString::fromUtf8("newRecordAction"));
        newRecordAction->setIcon(icon);
        insertValuesAction = new QAction(TableBrowser);
        insertValuesAction->setObjectName(QString::fromUtf8("insertValuesAction"));
        actionFilteredTableExportCsv = new QAction(TableBrowser);
        actionFilteredTableExportCsv->setObjectName(QString::fromUtf8("actionFilteredTableExportCsv"));
        actionFilteredTableExportJson = new QAction(TableBrowser);
        actionFilteredTableExportJson->setObjectName(QString::fromUtf8("actionFilteredTableExportJson"));
        actionFilterSaveAsView = new QAction(TableBrowser);
        actionFilterSaveAsView->setObjectName(QString::fromUtf8("actionFilterSaveAsView"));
        actionSaveFilterAsPopup = new QAction(TableBrowser);
        actionSaveFilterAsPopup->setObjectName(QString::fromUtf8("actionSaveFilterAsPopup"));
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/icons/save_table"), QSize(), QIcon::Normal, QIcon::Off);
        actionSaveFilterAsPopup->setIcon(icon2);
        actionHideColumns = new QAction(TableBrowser);
        actionHideColumns->setObjectName(QString::fromUtf8("actionHideColumns"));
        actionShowAllColumns = new QAction(TableBrowser);
        actionShowAllColumns->setObjectName(QString::fromUtf8("actionShowAllColumns"));
        actionSetTableEncoding = new QAction(TableBrowser);
        actionSetTableEncoding->setObjectName(QString::fromUtf8("actionSetTableEncoding"));
        actionSetAllTablesEncoding = new QAction(TableBrowser);
        actionSetAllTablesEncoding->setObjectName(QString::fromUtf8("actionSetAllTablesEncoding"));
        actionCopyColumnName = new QAction(TableBrowser);
        actionCopyColumnName->setObjectName(QString::fromUtf8("actionCopyColumnName"));
        actionClearFilters = new QAction(TableBrowser);
        actionClearFilters->setObjectName(QString::fromUtf8("actionClearFilters"));
        QIcon icon3;
        icon3.addFile(QString::fromUtf8(":/icons/clear_filters"), QSize(), QIcon::Normal, QIcon::Off);
        actionClearFilters->setIcon(icon3);
        actionClearSorting = new QAction(TableBrowser);
        actionClearSorting->setObjectName(QString::fromUtf8("actionClearSorting"));
        QIcon icon4;
        icon4.addFile(QString::fromUtf8(":/icons/clear_sorting"), QSize(), QIcon::Normal, QIcon::Off);
        actionClearSorting->setIcon(icon4);
        actionPrintTable = new QAction(TableBrowser);
        actionPrintTable->setObjectName(QString::fromUtf8("actionPrintTable"));
        QIcon icon5;
        icon5.addFile(QString::fromUtf8(":/icons/print"), QSize(), QIcon::Normal, QIcon::Off);
        actionPrintTable->setIcon(icon5);
        actionPrintTable->setShortcutContext(Qt::WidgetShortcut);
        actionAddDock = new QAction(TableBrowser);
        actionAddDock->setObjectName(QString::fromUtf8("actionAddDock"));
        QIcon icon6;
        icon6.addFile(QString::fromUtf8(":/icons/new_tab"), QSize(), QIcon::Normal, QIcon::Off);
        actionAddDock->setIcon(icon6);
        actionRefresh = new QAction(TableBrowser);
        actionRefresh->setObjectName(QString::fromUtf8("actionRefresh"));
        QIcon icon7;
        icon7.addFile(QString::fromUtf8(":/icons/refresh"), QSize(), QIcon::Normal, QIcon::Off);
        actionRefresh->setIcon(icon7);
        actionRefresh->setShortcutContext(Qt::WidgetShortcut);
        actionFind = new QAction(TableBrowser);
        actionFind->setObjectName(QString::fromUtf8("actionFind"));
        actionFind->setCheckable(true);
        QIcon icon8;
        icon8.addFile(QString::fromUtf8(":/icons/find"), QSize(), QIcon::Normal, QIcon::Off);
        actionFind->setIcon(icon8);
        actionBold = new QAction(TableBrowser);
        actionBold->setObjectName(QString::fromUtf8("actionBold"));
        actionBold->setCheckable(true);
        QIcon icon9;
        icon9.addFile(QString::fromUtf8(":/icons/text_bold"), QSize(), QIcon::Normal, QIcon::Off);
        actionBold->setIcon(icon9);
        actionItalic = new QAction(TableBrowser);
        actionItalic->setObjectName(QString::fromUtf8("actionItalic"));
        actionItalic->setCheckable(true);
        QIcon icon10;
        icon10.addFile(QString::fromUtf8(":/icons/text_italic"), QSize(), QIcon::Normal, QIcon::Off);
        actionItalic->setIcon(icon10);
        actionUnderline = new QAction(TableBrowser);
        actionUnderline->setObjectName(QString::fromUtf8("actionUnderline"));
        actionUnderline->setCheckable(true);
        QIcon icon11;
        icon11.addFile(QString::fromUtf8(":/icons/text_underline"), QSize(), QIcon::Normal, QIcon::Off);
        actionUnderline->setIcon(icon11);
        actionRightAlign = new QAction(TableBrowser);
        actionRightAlign->setObjectName(QString::fromUtf8("actionRightAlign"));
        actionRightAlign->setCheckable(true);
        QIcon icon12;
        icon12.addFile(QString::fromUtf8(":/icons/text_align_right"), QSize(), QIcon::Normal, QIcon::Off);
        actionRightAlign->setIcon(icon12);
        actionLeftAlign = new QAction(TableBrowser);
        actionLeftAlign->setObjectName(QString::fromUtf8("actionLeftAlign"));
        actionLeftAlign->setCheckable(true);
        QIcon icon13;
        icon13.addFile(QString::fromUtf8(":/icons/text_align_left"), QSize(), QIcon::Normal, QIcon::Off);
        actionLeftAlign->setIcon(icon13);
        actionCenter = new QAction(TableBrowser);
        actionCenter->setObjectName(QString::fromUtf8("actionCenter"));
        actionCenter->setCheckable(true);
        QIcon icon14;
        icon14.addFile(QString::fromUtf8(":/icons/text_align_center"), QSize(), QIcon::Normal, QIcon::Off);
        actionCenter->setIcon(icon14);
        actionJustify = new QAction(TableBrowser);
        actionJustify->setObjectName(QString::fromUtf8("actionJustify"));
        actionJustify->setCheckable(true);
        QIcon icon15;
        icon15.addFile(QString::fromUtf8(":/icons/text_align_justify"), QSize(), QIcon::Normal, QIcon::Off);
        actionJustify->setIcon(icon15);
        actionEditCondFormats = new QAction(TableBrowser);
        actionEditCondFormats->setObjectName(QString::fromUtf8("actionEditCondFormats"));
        QIcon icon16;
        icon16.addFile(QString::fromUtf8(":/icons/edit_cond_formats"), QSize(), QIcon::Normal, QIcon::Off);
        actionEditCondFormats->setIcon(icon16);
        actionClearFormat = new QAction(TableBrowser);
        actionClearFormat->setObjectName(QString::fromUtf8("actionClearFormat"));
        QIcon icon17;
        icon17.addFile(QString::fromUtf8(":/icons/clear_cond_formats"), QSize(), QIcon::Normal, QIcon::Off);
        actionClearFormat->setIcon(icon17);
        actionFontColor = new QAction(TableBrowser);
        actionFontColor->setObjectName(QString::fromUtf8("actionFontColor"));
        QIcon icon18;
        icon18.addFile(QString::fromUtf8(":/icons/foreground_color"), QSize(), QIcon::Normal, QIcon::Off);
        actionFontColor->setIcon(icon18);
        actionBackgroundColor = new QAction(TableBrowser);
        actionBackgroundColor->setObjectName(QString::fromUtf8("actionBackgroundColor"));
        QIcon icon19;
        icon19.addFile(QString::fromUtf8(":/icons/background_color"), QSize(), QIcon::Normal, QIcon::Off);
        actionBackgroundColor->setIcon(icon19);
        actionToggleFormatToolbar = new QAction(TableBrowser);
        actionToggleFormatToolbar->setObjectName(QString::fromUtf8("actionToggleFormatToolbar"));
        actionToggleFormatToolbar->setCheckable(true);
        QIcon icon20;
        icon20.addFile(QString::fromUtf8(":/icons/cond_formats"), QSize(), QIcon::Normal, QIcon::Off);
        actionToggleFormatToolbar->setIcon(icon20);
        actionSelectColumn = new QAction(TableBrowser);
        actionSelectColumn->setObjectName(QString::fromUtf8("actionSelectColumn"));
        actionReplace = new QAction(TableBrowser);
        actionReplace->setObjectName(QString::fromUtf8("actionReplace"));
        actionReplace->setCheckable(true);
        QIcon icon21;
        icon21.addFile(QString::fromUtf8(":/icons/text_replace"), QSize(), QIcon::Normal, QIcon::Off);
        actionReplace->setIcon(icon21);
        actionFreezeColumns = new QAction(TableBrowser);
        actionFreezeColumns->setObjectName(QString::fromUtf8("actionFreezeColumns"));
        actionFreezeColumns->setCheckable(true);
        verticalLayout_2 = new QVBoxLayout(TableBrowser);
        verticalLayout_2->setSpacing(1);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalLayout->setContentsMargins(2, -1, 2, -1);
        label = new QLabel(TableBrowser);
        label->setObjectName(QString::fromUtf8("label"));

        horizontalLayout->addWidget(label);

        comboBrowseTable = new QComboBox(TableBrowser);
        comboBrowseTable->setObjectName(QString::fromUtf8("comboBrowseTable"));
        comboBrowseTable->setMinimumSize(QSize(150, 0));
        comboBrowseTable->setMaxVisibleItems(30);
        comboBrowseTable->setSizeAdjustPolicy(QComboBox::AdjustToContents);

        horizontalLayout->addWidget(comboBrowseTable);

        browseToolbar = new QToolBar(TableBrowser);
        browseToolbar->setObjectName(QString::fromUtf8("browseToolbar"));
        browseToolbar->setToolButtonStyle(Qt::ToolButtonIconOnly);

        horizontalLayout->addWidget(browseToolbar);

        editGlobalFilter = new FilterLineEdit(TableBrowser);
        editGlobalFilter->setObjectName(QString::fromUtf8("editGlobalFilter"));

        horizontalLayout->addWidget(editGlobalFilter);

        horizontalSpacer = new QSpacerItem(0, 20, QSizePolicy::MinimumExpanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout_2->addLayout(horizontalLayout);

        formatFrame = new QFrame(TableBrowser);
        formatFrame->setObjectName(QString::fromUtf8("formatFrame"));
        horizontalLayoutFormat = new QHBoxLayout(formatFrame);
        horizontalLayoutFormat->setSpacing(5);
        horizontalLayoutFormat->setObjectName(QString::fromUtf8("horizontalLayoutFormat"));
        horizontalLayoutFormat->setContentsMargins(2, 0, 2, 0);
        fontComboBox = new QFontComboBox(formatFrame);
        fontComboBox->setObjectName(QString::fromUtf8("fontComboBox"));

        horizontalLayoutFormat->addWidget(fontComboBox);

        fontSizeBox = new QSpinBox(formatFrame);
        fontSizeBox->setObjectName(QString::fromUtf8("fontSizeBox"));
        fontSizeBox->setMaximumSize(QSize(50, 16777215));
        fontSizeBox->setMinimum(1);

        horizontalLayoutFormat->addWidget(fontSizeBox);

        formatToolbar = new QToolBar(formatFrame);
        formatToolbar->setObjectName(QString::fromUtf8("formatToolbar"));
        formatToolbar->setToolButtonStyle(Qt::ToolButtonIconOnly);

        horizontalLayoutFormat->addWidget(formatToolbar);


        verticalLayout_2->addWidget(formatFrame);

        dataTable = new ExtendedTableWidget(TableBrowser);
        dataTable->setObjectName(QString::fromUtf8("dataTable"));
        dataTable->setAcceptDrops(true);
        dataTable->setEditTriggers(QAbstractItemView::NoEditTriggers);
        dataTable->setDragDropMode(QAbstractItemView::DragDrop);
        dataTable->setDefaultDropAction(Qt::CopyAction);
        dataTable->setSelectionMode(QAbstractItemView::ExtendedSelection);

        verticalLayout_2->addWidget(dataTable);

        frameFind = new QFrame(TableBrowser);
        frameFind->setObjectName(QString::fromUtf8("frameFind"));
        frameFind->setMaximumSize(QSize(16777215, 62));
        frameFind->setFrameShape(QFrame::StyledPanel);
        frameFind->setFrameShadow(QFrame::Raised);
        vboxLayout = new QVBoxLayout(frameFind);
        vboxLayout->setSpacing(1);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        vboxLayout->setContentsMargins(1, 1, 1, 1);
        widgetFind = new QWidget(frameFind);
        widgetFind->setObjectName(QString::fromUtf8("widgetFind"));
        layoutFind = new QHBoxLayout(widgetFind);
        layoutFind->setSpacing(3);
        layoutFind->setObjectName(QString::fromUtf8("layoutFind"));
        layoutFind->setContentsMargins(0, 0, 0, 0);
        editFindExpression = new QLineEdit(widgetFind);
        editFindExpression->setObjectName(QString::fromUtf8("editFindExpression"));
        editFindExpression->setContextMenuPolicy(Qt::DefaultContextMenu);
        editFindExpression->setClearButtonEnabled(true);

        layoutFind->addWidget(editFindExpression);

        buttonFindPrevious = new QToolButton(widgetFind);
        buttonFindPrevious->setObjectName(QString::fromUtf8("buttonFindPrevious"));
        QIcon icon22;
        icon22.addFile(QString::fromUtf8(":/icons/up"), QSize(), QIcon::Normal, QIcon::Off);
        buttonFindPrevious->setIcon(icon22);

        layoutFind->addWidget(buttonFindPrevious);

        buttonFindNext = new QToolButton(widgetFind);
        buttonFindNext->setObjectName(QString::fromUtf8("buttonFindNext"));
        QIcon icon23;
        icon23.addFile(QString::fromUtf8(":/icons/down"), QSize(), QIcon::Normal, QIcon::Off);
        buttonFindNext->setIcon(icon23);

        layoutFind->addWidget(buttonFindNext);

        checkFindCaseSensitive = new QCheckBox(widgetFind);
        checkFindCaseSensitive->setObjectName(QString::fromUtf8("checkFindCaseSensitive"));

        layoutFind->addWidget(checkFindCaseSensitive);

        checkFindWholeCell = new QCheckBox(widgetFind);
        checkFindWholeCell->setObjectName(QString::fromUtf8("checkFindWholeCell"));

        layoutFind->addWidget(checkFindWholeCell);

        checkFindRegEx = new QCheckBox(widgetFind);
        checkFindRegEx->setObjectName(QString::fromUtf8("checkFindRegEx"));

        layoutFind->addWidget(checkFindRegEx);

        horizontalSpacer_1 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        layoutFind->addItem(horizontalSpacer_1);

        buttonFindClose = new QToolButton(widgetFind);
        buttonFindClose->setObjectName(QString::fromUtf8("buttonFindClose"));
        QIcon icon24;
        icon24.addFile(QString::fromUtf8(":/icons/close"), QSize(), QIcon::Normal, QIcon::Off);
        buttonFindClose->setIcon(icon24);
        buttonFindClose->setAutoRaise(true);

        layoutFind->addWidget(buttonFindClose);


        vboxLayout->addWidget(widgetFind);

        widgetReplace = new QWidget(frameFind);
        widgetReplace->setObjectName(QString::fromUtf8("widgetReplace"));
        layoutReplace = new QHBoxLayout(widgetReplace);
        layoutReplace->setSpacing(3);
        layoutReplace->setObjectName(QString::fromUtf8("layoutReplace"));
        layoutReplace->setContentsMargins(0, 0, 0, 0);
        editReplaceExpression = new QLineEdit(widgetReplace);
        editReplaceExpression->setObjectName(QString::fromUtf8("editReplaceExpression"));
        editReplaceExpression->setContextMenuPolicy(Qt::DefaultContextMenu);
        editReplaceExpression->setClearButtonEnabled(true);

        layoutReplace->addWidget(editReplaceExpression);

        buttonReplaceNext = new QToolButton(widgetReplace);
        buttonReplaceNext->setObjectName(QString::fromUtf8("buttonReplaceNext"));

        layoutReplace->addWidget(buttonReplaceNext);

        buttonReplaceAll = new QToolButton(widgetReplace);
        buttonReplaceAll->setObjectName(QString::fromUtf8("buttonReplaceAll"));

        layoutReplace->addWidget(buttonReplaceAll);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        layoutReplace->addItem(horizontalSpacer_3);


        vboxLayout->addWidget(widgetReplace);


        verticalLayout_2->addWidget(frameFind);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        horizontalLayout_2->setContentsMargins(2, 2, 2, 2);
        buttonBegin = new QToolButton(TableBrowser);
        buttonBegin->setObjectName(QString::fromUtf8("buttonBegin"));
        QIcon icon25;
        icon25.addFile(QString::fromUtf8(":/icons/resultset_first"), QSize(), QIcon::Normal, QIcon::Off);
        buttonBegin->setIcon(icon25);

        horizontalLayout_2->addWidget(buttonBegin);

        buttonPrevious = new QToolButton(TableBrowser);
        buttonPrevious->setObjectName(QString::fromUtf8("buttonPrevious"));
        QIcon icon26;
        icon26.addFile(QString::fromUtf8(":/icons/resultset_previous"), QSize(), QIcon::Normal, QIcon::Off);
        buttonPrevious->setIcon(icon26);

        horizontalLayout_2->addWidget(buttonPrevious);

        labelRecordset = new QLabel(TableBrowser);
        labelRecordset->setObjectName(QString::fromUtf8("labelRecordset"));

        horizontalLayout_2->addWidget(labelRecordset);

        buttonNext = new QToolButton(TableBrowser);
        buttonNext->setObjectName(QString::fromUtf8("buttonNext"));
        QIcon icon27;
        icon27.addFile(QString::fromUtf8(":/icons/run"), QSize(), QIcon::Normal, QIcon::Off);
        buttonNext->setIcon(icon27);

        horizontalLayout_2->addWidget(buttonNext);

        buttonEnd = new QToolButton(TableBrowser);
        buttonEnd->setObjectName(QString::fromUtf8("buttonEnd"));
        QIcon icon28;
        icon28.addFile(QString::fromUtf8(":/icons/run_line"), QSize(), QIcon::Normal, QIcon::Off);
        buttonEnd->setIcon(icon28);

        horizontalLayout_2->addWidget(buttonEnd);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);

        buttonGoto = new QPushButton(TableBrowser);
        buttonGoto->setObjectName(QString::fromUtf8("buttonGoto"));

        horizontalLayout_2->addWidget(buttonGoto);

        editGoto = new QLineEdit(TableBrowser);
        editGoto->setObjectName(QString::fromUtf8("editGoto"));

        horizontalLayout_2->addWidget(editGoto);


        verticalLayout_2->addLayout(horizontalLayout_2);

#if QT_CONFIG(shortcut)
        label->setBuddy(comboBrowseTable);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(comboBrowseTable, editGlobalFilter);
        QWidget::setTabOrder(editGlobalFilter, fontComboBox);
        QWidget::setTabOrder(fontComboBox, fontSizeBox);
        QWidget::setTabOrder(fontSizeBox, dataTable);
        QWidget::setTabOrder(dataTable, editFindExpression);
        QWidget::setTabOrder(editFindExpression, editReplaceExpression);
        QWidget::setTabOrder(editReplaceExpression, buttonFindPrevious);
        QWidget::setTabOrder(buttonFindPrevious, buttonFindNext);
        QWidget::setTabOrder(buttonFindNext, checkFindCaseSensitive);
        QWidget::setTabOrder(checkFindCaseSensitive, checkFindWholeCell);
        QWidget::setTabOrder(checkFindWholeCell, checkFindRegEx);
        QWidget::setTabOrder(checkFindRegEx, buttonFindClose);
        QWidget::setTabOrder(buttonFindClose, buttonReplaceNext);
        QWidget::setTabOrder(buttonReplaceNext, buttonReplaceAll);
        QWidget::setTabOrder(buttonReplaceAll, buttonBegin);
        QWidget::setTabOrder(buttonBegin, buttonPrevious);
        QWidget::setTabOrder(buttonPrevious, buttonNext);
        QWidget::setTabOrder(buttonNext, buttonEnd);
        QWidget::setTabOrder(buttonEnd, buttonGoto);
        QWidget::setTabOrder(buttonGoto, editGoto);

        browseToolbar->addAction(actionAddDock);
        browseToolbar->addAction(actionRefresh);
        browseToolbar->addAction(actionClearFilters);
        browseToolbar->addAction(actionClearSorting);
        browseToolbar->addSeparator();
        browseToolbar->addAction(actionSaveFilterAsPopup);
        browseToolbar->addAction(actionPrintTable);
        browseToolbar->addSeparator();
        browseToolbar->addAction(actionNewRecord);
        browseToolbar->addAction(actionDeleteRecord);
        browseToolbar->addSeparator();
        browseToolbar->addAction(actionToggleFormatToolbar);
        browseToolbar->addAction(actionFind);
        browseToolbar->addAction(actionReplace);
        formatToolbar->addAction(actionBold);
        formatToolbar->addAction(actionItalic);
        formatToolbar->addAction(actionUnderline);
        formatToolbar->addSeparator();
        formatToolbar->addAction(actionFontColor);
        formatToolbar->addAction(actionBackgroundColor);
        formatToolbar->addSeparator();
        formatToolbar->addAction(actionLeftAlign);
        formatToolbar->addAction(actionCenter);
        formatToolbar->addAction(actionRightAlign);
        formatToolbar->addAction(actionJustify);
        formatToolbar->addSeparator();
        formatToolbar->addAction(actionEditCondFormats);
        formatToolbar->addAction(actionClearFormat);

        retranslateUi(TableBrowser);
        QObject::connect(comboBrowseTable, SIGNAL(activated(QString)), TableBrowser, SLOT(refresh()));
        QObject::connect(buttonPrevious, SIGNAL(clicked()), TableBrowser, SLOT(navigatePrevious()));
        QObject::connect(buttonNext, SIGNAL(clicked()), TableBrowser, SLOT(navigateNext()));
        QObject::connect(buttonGoto, SIGNAL(clicked()), TableBrowser, SLOT(navigateGoto()));
        QObject::connect(editGoto, SIGNAL(returnPressed()), TableBrowser, SLOT(navigateGoto()));
        QObject::connect(buttonEnd, SIGNAL(clicked()), TableBrowser, SLOT(navigateEnd()));
        QObject::connect(buttonBegin, SIGNAL(clicked()), TableBrowser, SLOT(navigateBegin()));
        QObject::connect(actionShowRowidColumn, SIGNAL(triggered(bool)), TableBrowser, SLOT(showRowidColumn(bool)));
        QObject::connect(actionUnlockViewEditing, SIGNAL(toggled(bool)), TableBrowser, SLOT(unlockViewEditing(bool)));
        QObject::connect(actionBrowseTableEditDisplayFormat, SIGNAL(triggered()), TableBrowser, SLOT(editDisplayFormat()));
        QObject::connect(actionNewRecord, SIGNAL(triggered()), TableBrowser, SLOT(addRecord()));
        QObject::connect(actionDeleteRecord, SIGNAL(triggered()), TableBrowser, SLOT(deleteRecord()));
        QObject::connect(newRecordAction, SIGNAL(triggered()), TableBrowser, SLOT(addRecord()));
        QObject::connect(insertValuesAction, SIGNAL(triggered()), TableBrowser, SLOT(insertValues()));
        QObject::connect(actionFilteredTableExportCsv, SIGNAL(triggered()), TableBrowser, SLOT(exportCsvFilteredTable()));
        QObject::connect(actionFilteredTableExportJson, SIGNAL(triggered()), TableBrowser, SLOT(exportJsonFilteredTable()));
        QObject::connect(actionFilterSaveAsView, SIGNAL(triggered()), TableBrowser, SLOT(saveFilterAsView()));
        QObject::connect(actionSetTableEncoding, SIGNAL(triggered()), TableBrowser, SLOT(setTableEncoding()));
        QObject::connect(actionCopyColumnName, SIGNAL(triggered()), TableBrowser, SLOT(copyColumnName()));
        QObject::connect(actionSetAllTablesEncoding, SIGNAL(triggered()), TableBrowser, SLOT(setDefaultTableEncoding()));
        QObject::connect(actionHideColumns, SIGNAL(triggered()), TableBrowser, SLOT(hideColumns()));
        QObject::connect(actionPrintTable, SIGNAL(triggered()), dataTable, SLOT(openPrintDialog()));
        QObject::connect(actionClearFilters, SIGNAL(triggered()), TableBrowser, SLOT(clearFilters()));
        QObject::connect(actionClearSorting, SIGNAL(triggered()), TableBrowser, SLOT(clearSorting()));
        QObject::connect(actionShowAllColumns, SIGNAL(triggered()), TableBrowser, SLOT(showAllColumns()));

        QMetaObject::connectSlotsByName(TableBrowser);
    } // setupUi

    void retranslateUi(QWidget *TableBrowser)
    {
        TableBrowser->setWindowTitle(QCoreApplication::translate("TableBrowser", "Browse Data", nullptr));
        actionShowRowidColumn->setText(QCoreApplication::translate("TableBrowser", "Show rowid column", nullptr));
#if QT_CONFIG(tooltip)
        actionShowRowidColumn->setToolTip(QCoreApplication::translate("TableBrowser", "Toggle the visibility of the rowid column", nullptr));
#endif // QT_CONFIG(tooltip)
        actionUnlockViewEditing->setText(QCoreApplication::translate("TableBrowser", "Unlock view editing", nullptr));
#if QT_CONFIG(tooltip)
        actionUnlockViewEditing->setToolTip(QCoreApplication::translate("TableBrowser", "This unlocks the current view for editing. However, you will need appropriate triggers for editing.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionBrowseTableEditDisplayFormat->setText(QCoreApplication::translate("TableBrowser", "Edit display format", nullptr));
#if QT_CONFIG(tooltip)
        actionBrowseTableEditDisplayFormat->setToolTip(QCoreApplication::translate("TableBrowser", "Edit the display format of the data in this column", nullptr));
#endif // QT_CONFIG(tooltip)
        actionNewRecord->setText(QCoreApplication::translate("TableBrowser", "New Record", nullptr));
#if QT_CONFIG(tooltip)
        actionNewRecord->setToolTip(QCoreApplication::translate("TableBrowser", "Insert a new record in the current table", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        actionNewRecord->setStatusTip(QCoreApplication::translate("TableBrowser", "Insert a new record in the current table", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        actionNewRecord->setWhatsThis(QCoreApplication::translate("TableBrowser", "<html><head/><body><p>This button creates a new record in the database. Hold the mouse button to open a pop-up menu of different options:</p><ul><li><span style=\" font-weight:600;\">New Record</span>: insert a new record with default values in the database.</li><li><span style=\" font-weight:600;\">Insert Values...</span>: open a dialog for entering values before they are inserted in the database. This allows to enter values accomplishing the different constraints. This dialog is also open if the <span style=\" font-weight:600;\">New Record</span> option fails due to these constraints.</li></ul></body></html>", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionDeleteRecord->setText(QCoreApplication::translate("TableBrowser", "Delete Record", nullptr));
#if QT_CONFIG(tooltip)
        actionDeleteRecord->setToolTip(QCoreApplication::translate("TableBrowser", "Delete the current record", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        actionDeleteRecord->setStatusTip(QCoreApplication::translate("TableBrowser", "This button deletes the record or records currently selected in the table", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        actionDeleteRecord->setWhatsThis(QCoreApplication::translate("TableBrowser", "This button deletes the record or records currently selected in the table", nullptr));
#endif // QT_CONFIG(whatsthis)
        newRecordAction->setText(QCoreApplication::translate("TableBrowser", "New Record", nullptr));
#if QT_CONFIG(tooltip)
        newRecordAction->setToolTip(QCoreApplication::translate("TableBrowser", "Insert new record using default values in browsed table", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        newRecordAction->setStatusTip(QCoreApplication::translate("TableBrowser", "Insert new record using default values in browsed table", nullptr));
#endif // QT_CONFIG(statustip)
        insertValuesAction->setText(QCoreApplication::translate("TableBrowser", "Insert Values...", nullptr));
#if QT_CONFIG(tooltip)
        insertValuesAction->setToolTip(QCoreApplication::translate("TableBrowser", "Open a dialog for inserting values in a new record", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        insertValuesAction->setStatusTip(QCoreApplication::translate("TableBrowser", "Open a dialog for inserting values in a new record", nullptr));
#endif // QT_CONFIG(statustip)
        actionFilteredTableExportCsv->setText(QCoreApplication::translate("TableBrowser", "Export to &CSV", nullptr));
#if QT_CONFIG(tooltip)
        actionFilteredTableExportCsv->setToolTip(QCoreApplication::translate("TableBrowser", "Export the filtered data to CSV", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        actionFilteredTableExportCsv->setStatusTip(QCoreApplication::translate("TableBrowser", "Export the filtered data to CSV", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        actionFilteredTableExportCsv->setWhatsThis(QCoreApplication::translate("TableBrowser", "This button exports the data of the browsed table as currently displayed (after filters, display formats and order column) as a CSV file.", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionFilteredTableExportJson->setText(QCoreApplication::translate("TableBrowser", "Export to &JSON", nullptr));
#if QT_CONFIG(tooltip)
        actionFilteredTableExportJson->setToolTip(QCoreApplication::translate("TableBrowser", "Export the filtered data to JSON", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        actionFilteredTableExportJson->setStatusTip(QCoreApplication::translate("TableBrowser", "Export the filtered data to JSON", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        actionFilteredTableExportJson->setWhatsThis(QCoreApplication::translate("TableBrowser", "This button exports the data of the browsed table as currently displayed (after filters, display formats and order column) as a JSON file.", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionFilterSaveAsView->setText(QCoreApplication::translate("TableBrowser", "Save as &view", nullptr));
#if QT_CONFIG(tooltip)
        actionFilterSaveAsView->setToolTip(QCoreApplication::translate("TableBrowser", "Save the current filter, sort column and display formats as a view", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        actionFilterSaveAsView->setStatusTip(QCoreApplication::translate("TableBrowser", "Save the current filter, sort column and display formats as a view", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        actionFilterSaveAsView->setWhatsThis(QCoreApplication::translate("TableBrowser", "This button saves the current setting of the browsed table (filters, display formats and order column) as an SQL view that you can later browse or use in SQL statements.", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionSaveFilterAsPopup->setText(QCoreApplication::translate("TableBrowser", "Save Table As...", nullptr));
#if QT_CONFIG(tooltip)
        actionSaveFilterAsPopup->setToolTip(QCoreApplication::translate("TableBrowser", "Save the table as currently displayed", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        actionSaveFilterAsPopup->setStatusTip(QCoreApplication::translate("TableBrowser", "Save the table as currently displayed", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        actionSaveFilterAsPopup->setWhatsThis(QCoreApplication::translate("TableBrowser", "<html><head/><body><p>This pop-up menu provides the following options applying to the currently browsed and filtered table:</p><ul style=\"margin-top: 0px; margin-bottom: 0px; margin-left: 0px; margin-right: 0px; -qt-list-indent: 1;\"><li style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Export to CSV: this option exports the data of the browsed table as currently displayed (after filters, display formats and order column) to a CSV file.</li><li style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Save as view: this option saves the current setting of the browsed table (filters, display formats and order column) as an SQL view that you can later browse or use in SQL statements.</li></ul></body></html>", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionHideColumns->setText(QCoreApplication::translate("TableBrowser", "Hide column(s)", nullptr));
#if QT_CONFIG(tooltip)
        actionHideColumns->setToolTip(QCoreApplication::translate("TableBrowser", "Hide selected column(s)", nullptr));
#endif // QT_CONFIG(tooltip)
        actionShowAllColumns->setText(QCoreApplication::translate("TableBrowser", "Show all columns", nullptr));
#if QT_CONFIG(tooltip)
        actionShowAllColumns->setToolTip(QCoreApplication::translate("TableBrowser", "Show all columns that were hidden", nullptr));
#endif // QT_CONFIG(tooltip)
        actionSetTableEncoding->setText(QCoreApplication::translate("TableBrowser", "Set encoding", nullptr));
#if QT_CONFIG(tooltip)
        actionSetTableEncoding->setToolTip(QCoreApplication::translate("TableBrowser", "Change the encoding of the text in the table cells", nullptr));
#endif // QT_CONFIG(tooltip)
        actionSetAllTablesEncoding->setText(QCoreApplication::translate("TableBrowser", "Set encoding for all tables", nullptr));
#if QT_CONFIG(tooltip)
        actionSetAllTablesEncoding->setToolTip(QCoreApplication::translate("TableBrowser", "Change the default encoding assumed for all tables in the database", nullptr));
#endif // QT_CONFIG(tooltip)
        actionCopyColumnName->setText(QCoreApplication::translate("TableBrowser", "Copy column name", nullptr));
#if QT_CONFIG(tooltip)
        actionCopyColumnName->setToolTip(QCoreApplication::translate("TableBrowser", "Copy the database table column name to your clipboard", nullptr));
#endif // QT_CONFIG(tooltip)
        actionClearFilters->setText(QCoreApplication::translate("TableBrowser", "Clear Filters", nullptr));
#if QT_CONFIG(tooltip)
        actionClearFilters->setToolTip(QCoreApplication::translate("TableBrowser", "Clear all filters", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        actionClearFilters->setStatusTip(QCoreApplication::translate("TableBrowser", "This button clears all the filters set in the header input fields for the currently browsed table.", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        actionClearFilters->setWhatsThis(QCoreApplication::translate("TableBrowser", "This button clears all the filters set in the header input fields for the currently browsed table.", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionClearSorting->setText(QCoreApplication::translate("TableBrowser", "Clear Sorting", nullptr));
#if QT_CONFIG(tooltip)
        actionClearSorting->setToolTip(QCoreApplication::translate("TableBrowser", "Reset the order of rows to the default", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        actionClearSorting->setStatusTip(QCoreApplication::translate("TableBrowser", "This button clears the sorting columns specified for the currently browsed table and returns to the default order.", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        actionClearSorting->setWhatsThis(QCoreApplication::translate("TableBrowser", "This button clears the sorting columns specified for the currently browsed table and returns to the default order.", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionPrintTable->setText(QCoreApplication::translate("TableBrowser", "Print", nullptr));
#if QT_CONFIG(tooltip)
        actionPrintTable->setToolTip(QCoreApplication::translate("TableBrowser", "Print currently browsed table data", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        actionPrintTable->setStatusTip(QString());
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        actionPrintTable->setWhatsThis(QCoreApplication::translate("TableBrowser", "Print currently browsed table data. Print selection if more than one cell is selected.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        actionPrintTable->setShortcut(QCoreApplication::translate("TableBrowser", "Ctrl+P", nullptr));
#endif // QT_CONFIG(shortcut)
        actionAddDock->setText(QCoreApplication::translate("TableBrowser", "New Data Browser", nullptr));
#if QT_CONFIG(tooltip)
        actionAddDock->setToolTip(QCoreApplication::translate("TableBrowser", "Add a new docked Data Browser", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        actionAddDock->setStatusTip(QCoreApplication::translate("TableBrowser", "Add a new docked Data Browser", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        actionAddDock->setWhatsThis(QCoreApplication::translate("TableBrowser", "This button adds a new docked Data Browser, which you can detach and arrange in different layouts.", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionRefresh->setText(QCoreApplication::translate("TableBrowser", "Refresh", nullptr));
#if QT_CONFIG(tooltip)
        actionRefresh->setToolTip(QCoreApplication::translate("TableBrowser", "Refresh the data in the selected table", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        actionRefresh->setStatusTip(QCoreApplication::translate("TableBrowser", "This button refreshes the data in the currently selected table.", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(shortcut)
        actionRefresh->setShortcut(QCoreApplication::translate("TableBrowser", "F5", nullptr));
#endif // QT_CONFIG(shortcut)
        actionFind->setText(QCoreApplication::translate("TableBrowser", "Find in cells", nullptr));
#if QT_CONFIG(tooltip)
        actionFind->setToolTip(QCoreApplication::translate("TableBrowser", "Open the find tool bar which allows you to search for values in the table view below.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionBold->setText(QCoreApplication::translate("TableBrowser", "Bold", nullptr));
#if QT_CONFIG(tooltip)
        actionBold->setToolTip(QCoreApplication::translate("TableBrowser", "Bold", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        actionBold->setShortcut(QCoreApplication::translate("TableBrowser", "Ctrl+B", nullptr));
#endif // QT_CONFIG(shortcut)
        actionItalic->setText(QCoreApplication::translate("TableBrowser", "Italic", nullptr));
#if QT_CONFIG(tooltip)
        actionItalic->setToolTip(QCoreApplication::translate("TableBrowser", "Italic", nullptr));
#endif // QT_CONFIG(tooltip)
        actionUnderline->setText(QCoreApplication::translate("TableBrowser", "Underline", nullptr));
#if QT_CONFIG(tooltip)
        actionUnderline->setToolTip(QCoreApplication::translate("TableBrowser", "Underline", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        actionUnderline->setShortcut(QCoreApplication::translate("TableBrowser", "Ctrl+U", nullptr));
#endif // QT_CONFIG(shortcut)
        actionRightAlign->setText(QCoreApplication::translate("TableBrowser", "Align Right", nullptr));
#if QT_CONFIG(tooltip)
        actionRightAlign->setToolTip(QCoreApplication::translate("TableBrowser", "Align Right", nullptr));
#endif // QT_CONFIG(tooltip)
        actionLeftAlign->setText(QCoreApplication::translate("TableBrowser", "Align Left", nullptr));
#if QT_CONFIG(tooltip)
        actionLeftAlign->setToolTip(QCoreApplication::translate("TableBrowser", "Align Left", nullptr));
#endif // QT_CONFIG(tooltip)
        actionCenter->setText(QCoreApplication::translate("TableBrowser", "Center Horizontally", nullptr));
#if QT_CONFIG(tooltip)
        actionCenter->setToolTip(QCoreApplication::translate("TableBrowser", "Center Horizontally", nullptr));
#endif // QT_CONFIG(tooltip)
        actionJustify->setText(QCoreApplication::translate("TableBrowser", "Justify", nullptr));
#if QT_CONFIG(tooltip)
        actionJustify->setToolTip(QCoreApplication::translate("TableBrowser", "Justify", nullptr));
#endif // QT_CONFIG(tooltip)
        actionEditCondFormats->setText(QCoreApplication::translate("TableBrowser", "Edit Conditional Formats...", nullptr));
#if QT_CONFIG(tooltip)
        actionEditCondFormats->setToolTip(QCoreApplication::translate("TableBrowser", "Edit Conditional Formats...", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        actionEditCondFormats->setStatusTip(QCoreApplication::translate("TableBrowser", "Edit conditional formats for the current column", nullptr));
#endif // QT_CONFIG(statustip)
        actionClearFormat->setText(QCoreApplication::translate("TableBrowser", "Clear Format", nullptr));
#if QT_CONFIG(tooltip)
        actionClearFormat->setToolTip(QCoreApplication::translate("TableBrowser", "Clear All Formats", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        actionClearFormat->setStatusTip(QCoreApplication::translate("TableBrowser", "Clear all cell formatting from selected cells and all conditional formats from selected columns", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        actionClearFormat->setWhatsThis(QCoreApplication::translate("TableBrowser", "Clear all cell formatting from selected cells and all conditional formats from selected columns", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionFontColor->setText(QCoreApplication::translate("TableBrowser", "Font Color", nullptr));
#if QT_CONFIG(tooltip)
        actionFontColor->setToolTip(QCoreApplication::translate("TableBrowser", "Font Color", nullptr));
#endif // QT_CONFIG(tooltip)
        actionBackgroundColor->setText(QCoreApplication::translate("TableBrowser", "Background Color", nullptr));
#if QT_CONFIG(tooltip)
        actionBackgroundColor->setToolTip(QCoreApplication::translate("TableBrowser", "Background Color", nullptr));
#endif // QT_CONFIG(tooltip)
        actionToggleFormatToolbar->setText(QCoreApplication::translate("TableBrowser", "Toggle Format Toolbar", nullptr));
#if QT_CONFIG(tooltip)
        actionToggleFormatToolbar->setToolTip(QCoreApplication::translate("TableBrowser", "Show/hide format toolbar", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        actionToggleFormatToolbar->setStatusTip(QCoreApplication::translate("TableBrowser", "This button shows or hides the formatting toolbar of the Data Browser", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        actionToggleFormatToolbar->setWhatsThis(QCoreApplication::translate("TableBrowser", "This button shows or hides the formatting toolbar of the Data Browser", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionSelectColumn->setText(QCoreApplication::translate("TableBrowser", "Select column", nullptr));
#if QT_CONFIG(shortcut)
        actionSelectColumn->setShortcut(QCoreApplication::translate("TableBrowser", "Ctrl+Space", nullptr));
#endif // QT_CONFIG(shortcut)
        actionReplace->setText(QCoreApplication::translate("TableBrowser", "Replace", nullptr));
#if QT_CONFIG(tooltip)
        actionReplace->setToolTip(QCoreApplication::translate("TableBrowser", "Replace text in cells", nullptr));
#endif // QT_CONFIG(tooltip)
        actionFreezeColumns->setText(QCoreApplication::translate("TableBrowser", "Freeze columns", nullptr));
#if QT_CONFIG(tooltip)
        actionFreezeColumns->setToolTip(QCoreApplication::translate("TableBrowser", "Make all columns from the first column up to this column not move when scrolling horizontally", nullptr));
#endif // QT_CONFIG(tooltip)
        label->setText(QCoreApplication::translate("TableBrowser", "&Table:", nullptr));
#if QT_CONFIG(tooltip)
        comboBrowseTable->setToolTip(QCoreApplication::translate("TableBrowser", "Select a table to browse data", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        comboBrowseTable->setWhatsThis(QCoreApplication::translate("TableBrowser", "Use this list to select a table to be displayed in the database view", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        dataTable->setWhatsThis(QCoreApplication::translate("TableBrowser", "This is the database table view. You can do the following actions:\n"
" - Start writing for editing inline the value.\n"
" - Double-click any record to edit its contents in the cell editor window.\n"
" - Alt+Del for deleting the cell content to NULL.\n"
" - Ctrl+\" for duplicating the current record.\n"
" - Ctrl+' for copying the value from the cell above.\n"
" - Standard selection and copy/paste operations.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        editFindExpression->setWhatsThis(QCoreApplication::translate("TableBrowser", "Text pattern to find considering the checks in this frame", nullptr));
#endif // QT_CONFIG(whatsthis)
        editFindExpression->setPlaceholderText(QCoreApplication::translate("TableBrowser", "Find in table", nullptr));
#if QT_CONFIG(tooltip)
        buttonFindPrevious->setToolTip(QCoreApplication::translate("TableBrowser", "Find previous match [Shift+F3]", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        buttonFindPrevious->setWhatsThis(QCoreApplication::translate("TableBrowser", "Find previous match with wrapping", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        buttonFindPrevious->setShortcut(QCoreApplication::translate("TableBrowser", "Shift+F3", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        buttonFindNext->setToolTip(QCoreApplication::translate("TableBrowser", "Find next match [Enter, F3]", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        buttonFindNext->setWhatsThis(QCoreApplication::translate("TableBrowser", "Find next match with wrapping", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        buttonFindNext->setShortcut(QCoreApplication::translate("TableBrowser", "F3", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(whatsthis)
        checkFindCaseSensitive->setWhatsThis(QCoreApplication::translate("TableBrowser", "The found pattern must match in letter case", nullptr));
#endif // QT_CONFIG(whatsthis)
        checkFindCaseSensitive->setText(QCoreApplication::translate("TableBrowser", "Case Sensitive", nullptr));
#if QT_CONFIG(whatsthis)
        checkFindWholeCell->setWhatsThis(QCoreApplication::translate("TableBrowser", "The found pattern must be a whole word", nullptr));
#endif // QT_CONFIG(whatsthis)
        checkFindWholeCell->setText(QCoreApplication::translate("TableBrowser", "Whole Cell", nullptr));
#if QT_CONFIG(tooltip)
        checkFindRegEx->setToolTip(QCoreApplication::translate("TableBrowser", "Interpret search pattern as a regular expression", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        checkFindRegEx->setWhatsThis(QCoreApplication::translate("TableBrowser", "<html><head/><body><p>When checked, the pattern to find is interpreted as a UNIX regular expression. See <a href=\"https://en.wikibooks.org/wiki/Regular_Expressions\">Regular Expression in Wikibooks</a>.</p></body></html>", nullptr));
#endif // QT_CONFIG(whatsthis)
        checkFindRegEx->setText(QCoreApplication::translate("TableBrowser", "Regular Expression", nullptr));
#if QT_CONFIG(tooltip)
        buttonFindClose->setToolTip(QCoreApplication::translate("TableBrowser", "Close Find Bar", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonFindClose->setText(QCoreApplication::translate("TableBrowser", "Close Find Bar", nullptr));
#if QT_CONFIG(whatsthis)
        editReplaceExpression->setWhatsThis(QCoreApplication::translate("TableBrowser", "Text to replace with", nullptr));
#endif // QT_CONFIG(whatsthis)
        editReplaceExpression->setPlaceholderText(QCoreApplication::translate("TableBrowser", "Replace with", nullptr));
#if QT_CONFIG(tooltip)
        buttonReplaceNext->setToolTip(QCoreApplication::translate("TableBrowser", "Replace next match", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonReplaceNext->setText(QCoreApplication::translate("TableBrowser", "Replace", nullptr));
#if QT_CONFIG(tooltip)
        buttonReplaceAll->setToolTip(QCoreApplication::translate("TableBrowser", "Replace all matches", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonReplaceAll->setText(QCoreApplication::translate("TableBrowser", "Replace all", nullptr));
#if QT_CONFIG(tooltip)
        buttonBegin->setToolTip(QCoreApplication::translate("TableBrowser", "<html><head/><body><p>Scroll to the beginning</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        buttonBegin->setWhatsThis(QCoreApplication::translate("TableBrowser", "<html><head/><body><p>Clicking this button navigates to the beginning in the table view above.</p></body></html>", nullptr));
#endif // QT_CONFIG(whatsthis)
        buttonBegin->setText(QCoreApplication::translate("TableBrowser", "|<", nullptr));
#if QT_CONFIG(tooltip)
        buttonPrevious->setToolTip(QCoreApplication::translate("TableBrowser", "Scroll one page upwards", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        buttonPrevious->setWhatsThis(QCoreApplication::translate("TableBrowser", "<html><head/><body><p>Clicking this button navigates one page of records upwards in the table view above.</p></body></html>", nullptr));
#endif // QT_CONFIG(whatsthis)
        buttonPrevious->setText(QCoreApplication::translate("TableBrowser", "<", nullptr));
        labelRecordset->setText(QCoreApplication::translate("TableBrowser", "0 - 0 of 0", nullptr));
#if QT_CONFIG(tooltip)
        buttonNext->setToolTip(QCoreApplication::translate("TableBrowser", "Scroll one page downwards", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        buttonNext->setWhatsThis(QCoreApplication::translate("TableBrowser", "<html><head/><body><p>Clicking this button navigates one page of records downwards in the table view above.</p></body></html>", nullptr));
#endif // QT_CONFIG(whatsthis)
        buttonNext->setText(QCoreApplication::translate("TableBrowser", ">", nullptr));
#if QT_CONFIG(tooltip)
        buttonEnd->setToolTip(QCoreApplication::translate("TableBrowser", "Scroll to the end", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        buttonEnd->setWhatsThis(QCoreApplication::translate("TableBrowser", "<html><head/><body><p>Clicking this button navigates up to the end in the table view above.</p></body></html>", nullptr));
#endif // QT_CONFIG(whatsthis)
        buttonEnd->setText(QCoreApplication::translate("TableBrowser", ">|", nullptr));
#if QT_CONFIG(tooltip)
        buttonGoto->setToolTip(QCoreApplication::translate("TableBrowser", "<html><head/><body><p>Click here to jump to the specified record</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        buttonGoto->setWhatsThis(QCoreApplication::translate("TableBrowser", "<html><head/><body><p>This button is used to navigate to the record number specified in the Go to area.</p></body></html>", nullptr));
#endif // QT_CONFIG(whatsthis)
        buttonGoto->setText(QCoreApplication::translate("TableBrowser", "Go to:", nullptr));
#if QT_CONFIG(tooltip)
        editGoto->setToolTip(QCoreApplication::translate("TableBrowser", "Enter record number to browse", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        editGoto->setWhatsThis(QCoreApplication::translate("TableBrowser", "Type a record number in this area and click the Go to: button to display the record in the database view", nullptr));
#endif // QT_CONFIG(whatsthis)
        editGoto->setText(QCoreApplication::translate("TableBrowser", "1", nullptr));
    } // retranslateUi

};

namespace Ui {
    class TableBrowser: public Ui_TableBrowser {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TABLEBROWSER_H
