/****************************************************************************
** Meta object code from reading C++ file 'TableBrowser.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../src/TableBrowser.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'TableBrowser.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TableBrowser_t {
    QByteArrayData data[94];
    char stringdata0[1301];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TableBrowser_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TableBrowser_t qt_meta_stringdata_TableBrowser = {
    {
QT_MOC_LITERAL(0, 0, 12), // "TableBrowser"
QT_MOC_LITERAL(1, 13, 15), // "projectModified"
QT_MOC_LITERAL(2, 29, 0), // ""
QT_MOC_LITERAL(3, 30, 16), // "selectionChanged"
QT_MOC_LITERAL(4, 47, 11), // "QModelIndex"
QT_MOC_LITERAL(5, 59, 5), // "index"
QT_MOC_LITERAL(6, 65, 29), // "selectionChangedByDoubleClick"
QT_MOC_LITERAL(7, 95, 22), // "statusMessageRequested"
QT_MOC_LITERAL(8, 118, 7), // "message"
QT_MOC_LITERAL(9, 126, 10), // "updatePlot"
QT_MOC_LITERAL(10, 137, 20), // "ExtendedTableWidget*"
QT_MOC_LITERAL(11, 158, 11), // "tableWidget"
QT_MOC_LITERAL(12, 170, 17), // "SqliteTableModel*"
QT_MOC_LITERAL(13, 188, 5), // "model"
QT_MOC_LITERAL(14, 194, 24), // "BrowseDataTableSettings*"
QT_MOC_LITERAL(15, 219, 8), // "settings"
QT_MOC_LITERAL(16, 228, 20), // "keepOrResetSelection"
QT_MOC_LITERAL(17, 249, 10), // "createView"
QT_MOC_LITERAL(18, 260, 11), // "std::string"
QT_MOC_LITERAL(19, 272, 3), // "sql"
QT_MOC_LITERAL(20, 276, 15), // "requestFileOpen"
QT_MOC_LITERAL(21, 292, 4), // "file"
QT_MOC_LITERAL(22, 297, 19), // "currentTableChanged"
QT_MOC_LITERAL(23, 317, 22), // "sqlb::ObjectIdentifier"
QT_MOC_LITERAL(24, 340, 5), // "table"
QT_MOC_LITERAL(25, 346, 17), // "foreignKeyClicked"
QT_MOC_LITERAL(26, 364, 6), // "column"
QT_MOC_LITERAL(27, 371, 5), // "value"
QT_MOC_LITERAL(28, 377, 19), // "dataAboutToBeEdited"
QT_MOC_LITERAL(29, 397, 16), // "prepareForFilter"
QT_MOC_LITERAL(30, 414, 16), // "newDockRequested"
QT_MOC_LITERAL(31, 431, 10), // "setEnabled"
QT_MOC_LITERAL(32, 442, 6), // "enable"
QT_MOC_LITERAL(33, 449, 7), // "refresh"
QT_MOC_LITERAL(34, 457, 12), // "clearFilters"
QT_MOC_LITERAL(35, 470, 14), // "reloadSettings"
QT_MOC_LITERAL(36, 485, 15), // "setCurrentTable"
QT_MOC_LITERAL(37, 501, 4), // "name"
QT_MOC_LITERAL(38, 506, 20), // "updateRecordsetLabel"
QT_MOC_LITERAL(39, 527, 9), // "jumpToRow"
QT_MOC_LITERAL(40, 537, 5), // "clear"
QT_MOC_LITERAL(41, 543, 12), // "updateFilter"
QT_MOC_LITERAL(42, 556, 6), // "size_t"
QT_MOC_LITERAL(43, 563, 23), // "addCondFormatFromFilter"
QT_MOC_LITERAL(44, 587, 13), // "addCondFormat"
QT_MOC_LITERAL(45, 601, 13), // "isRowIdFormat"
QT_MOC_LITERAL(46, 615, 10), // "CondFormat"
QT_MOC_LITERAL(47, 626, 13), // "newCondFormat"
QT_MOC_LITERAL(48, 640, 19), // "clearAllCondFormats"
QT_MOC_LITERAL(49, 660, 17), // "clearRowIdFormats"
QT_MOC_LITERAL(50, 678, 15), // "editCondFormats"
QT_MOC_LITERAL(51, 694, 13), // "enableEditing"
QT_MOC_LITERAL(52, 708, 11), // "enable_edit"
QT_MOC_LITERAL(53, 720, 15), // "showRowidColumn"
QT_MOC_LITERAL(54, 736, 4), // "show"
QT_MOC_LITERAL(55, 741, 13), // "freezeColumns"
QT_MOC_LITERAL(56, 755, 7), // "columns"
QT_MOC_LITERAL(57, 763, 17), // "unlockViewEditing"
QT_MOC_LITERAL(58, 781, 6), // "unlock"
QT_MOC_LITERAL(59, 788, 2), // "pk"
QT_MOC_LITERAL(60, 791, 11), // "hideColumns"
QT_MOC_LITERAL(61, 803, 4), // "hide"
QT_MOC_LITERAL(62, 808, 14), // "showAllColumns"
QT_MOC_LITERAL(63, 823, 30), // "updateInsertDeleteRecordButton"
QT_MOC_LITERAL(64, 854, 15), // "duplicateRecord"
QT_MOC_LITERAL(65, 870, 10), // "currentRow"
QT_MOC_LITERAL(66, 881, 13), // "headerClicked"
QT_MOC_LITERAL(67, 895, 12), // "logicalindex"
QT_MOC_LITERAL(68, 908, 17), // "updateColumnWidth"
QT_MOC_LITERAL(69, 926, 7), // "section"
QT_MOC_LITERAL(70, 934, 8), // "new_size"
QT_MOC_LITERAL(71, 943, 23), // "showDataColumnPopupMenu"
QT_MOC_LITERAL(72, 967, 3), // "pos"
QT_MOC_LITERAL(73, 971, 19), // "showRecordPopupMenu"
QT_MOC_LITERAL(74, 991, 9), // "addRecord"
QT_MOC_LITERAL(75, 1001, 12), // "insertValues"
QT_MOC_LITERAL(76, 1014, 12), // "deleteRecord"
QT_MOC_LITERAL(77, 1027, 16), // "navigatePrevious"
QT_MOC_LITERAL(78, 1044, 12), // "navigateNext"
QT_MOC_LITERAL(79, 1057, 13), // "navigateBegin"
QT_MOC_LITERAL(80, 1071, 11), // "navigateEnd"
QT_MOC_LITERAL(81, 1083, 12), // "navigateGoto"
QT_MOC_LITERAL(82, 1096, 15), // "selectTableLine"
QT_MOC_LITERAL(83, 1112, 12), // "lineToSelect"
QT_MOC_LITERAL(84, 1125, 12), // "clearSorting"
QT_MOC_LITERAL(85, 1138, 17), // "editDisplayFormat"
QT_MOC_LITERAL(86, 1156, 22), // "exportCsvFilteredTable"
QT_MOC_LITERAL(87, 1179, 23), // "exportJsonFilteredTable"
QT_MOC_LITERAL(88, 1203, 16), // "saveFilterAsView"
QT_MOC_LITERAL(89, 1220, 16), // "setTableEncoding"
QT_MOC_LITERAL(90, 1237, 12), // "forAllTables"
QT_MOC_LITERAL(91, 1250, 23), // "setDefaultTableEncoding"
QT_MOC_LITERAL(92, 1274, 14), // "copyColumnName"
QT_MOC_LITERAL(93, 1289, 11) // "fetchedData"

    },
    "TableBrowser\0projectModified\0\0"
    "selectionChanged\0QModelIndex\0index\0"
    "selectionChangedByDoubleClick\0"
    "statusMessageRequested\0message\0"
    "updatePlot\0ExtendedTableWidget*\0"
    "tableWidget\0SqliteTableModel*\0model\0"
    "BrowseDataTableSettings*\0settings\0"
    "keepOrResetSelection\0createView\0"
    "std::string\0sql\0requestFileOpen\0file\0"
    "currentTableChanged\0sqlb::ObjectIdentifier\0"
    "table\0foreignKeyClicked\0column\0value\0"
    "dataAboutToBeEdited\0prepareForFilter\0"
    "newDockRequested\0setEnabled\0enable\0"
    "refresh\0clearFilters\0reloadSettings\0"
    "setCurrentTable\0name\0updateRecordsetLabel\0"
    "jumpToRow\0clear\0updateFilter\0size_t\0"
    "addCondFormatFromFilter\0addCondFormat\0"
    "isRowIdFormat\0CondFormat\0newCondFormat\0"
    "clearAllCondFormats\0clearRowIdFormats\0"
    "editCondFormats\0enableEditing\0enable_edit\0"
    "showRowidColumn\0show\0freezeColumns\0"
    "columns\0unlockViewEditing\0unlock\0pk\0"
    "hideColumns\0hide\0showAllColumns\0"
    "updateInsertDeleteRecordButton\0"
    "duplicateRecord\0currentRow\0headerClicked\0"
    "logicalindex\0updateColumnWidth\0section\0"
    "new_size\0showDataColumnPopupMenu\0pos\0"
    "showRecordPopupMenu\0addRecord\0"
    "insertValues\0deleteRecord\0navigatePrevious\0"
    "navigateNext\0navigateBegin\0navigateEnd\0"
    "navigateGoto\0selectTableLine\0lineToSelect\0"
    "clearSorting\0editDisplayFormat\0"
    "exportCsvFilteredTable\0exportJsonFilteredTable\0"
    "saveFilterAsView\0setTableEncoding\0"
    "forAllTables\0setDefaultTableEncoding\0"
    "copyColumnName\0fetchedData"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TableBrowser[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      60,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      12,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  314,    2, 0x06 /* Public */,
       3,    1,  315,    2, 0x06 /* Public */,
       6,    1,  318,    2, 0x06 /* Public */,
       7,    1,  321,    2, 0x06 /* Public */,
       9,    4,  324,    2, 0x06 /* Public */,
      17,    1,  333,    2, 0x06 /* Public */,
      20,    1,  336,    2, 0x06 /* Public */,
      22,    1,  339,    2, 0x06 /* Public */,
      25,    3,  342,    2, 0x06 /* Public */,
      28,    1,  349,    2, 0x06 /* Public */,
      29,    0,  352,    2, 0x06 /* Public */,
      30,    0,  353,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      31,    1,  354,    2, 0x0a /* Public */,
      33,    0,  357,    2, 0x0a /* Public */,
      34,    0,  358,    2, 0x0a /* Public */,
      35,    0,  359,    2, 0x0a /* Public */,
      36,    1,  360,    2, 0x0a /* Public */,
      38,    0,  363,    2, 0x0a /* Public */,
      39,    3,  364,    2, 0x0a /* Public */,
      40,    0,  371,    2, 0x08 /* Private */,
      41,    2,  372,    2, 0x08 /* Private */,
      43,    2,  377,    2, 0x08 /* Private */,
      44,    3,  382,    2, 0x08 /* Private */,
      48,    1,  389,    2, 0x08 /* Private */,
      49,    1,  392,    2, 0x08 /* Private */,
      50,    1,  395,    2, 0x08 /* Private */,
      51,    1,  398,    2, 0x08 /* Private */,
      53,    1,  401,    2, 0x08 /* Private */,
      55,    1,  404,    2, 0x08 /* Private */,
      57,    2,  407,    2, 0x08 /* Private */,
      57,    1,  412,    2, 0x28 /* Private | MethodCloned */,
      60,    2,  415,    2, 0x08 /* Private */,
      60,    1,  420,    2, 0x28 /* Private | MethodCloned */,
      60,    0,  423,    2, 0x28 /* Private | MethodCloned */,
      62,    0,  424,    2, 0x08 /* Private */,
      63,    0,  425,    2, 0x08 /* Private */,
      64,    1,  426,    2, 0x08 /* Private */,
      66,    1,  429,    2, 0x08 /* Private */,
      68,    3,  432,    2, 0x08 /* Private */,
      71,    1,  439,    2, 0x08 /* Private */,
      73,    1,  442,    2, 0x08 /* Private */,
      74,    0,  445,    2, 0x08 /* Private */,
      75,    0,  446,    2, 0x08 /* Private */,
      76,    0,  447,    2, 0x08 /* Private */,
      77,    0,  448,    2, 0x08 /* Private */,
      78,    0,  449,    2, 0x08 /* Private */,
      79,    0,  450,    2, 0x08 /* Private */,
      80,    0,  451,    2, 0x08 /* Private */,
      81,    0,  452,    2, 0x08 /* Private */,
      82,    1,  453,    2, 0x08 /* Private */,
      84,    0,  456,    2, 0x08 /* Private */,
      85,    0,  457,    2, 0x08 /* Private */,
      86,    0,  458,    2, 0x08 /* Private */,
      87,    0,  459,    2, 0x08 /* Private */,
      88,    0,  460,    2, 0x08 /* Private */,
      89,    1,  461,    2, 0x08 /* Private */,
      89,    0,  464,    2, 0x28 /* Private | MethodCloned */,
      91,    0,  465,    2, 0x08 /* Private */,
      92,    0,  466,    2, 0x08 /* Private */,
      93,    0,  467,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void, 0x80000000 | 10, 0x80000000 | 12, 0x80000000 | 14, QMetaType::Bool,   11,   13,   15,   16,
    QMetaType::Void, 0x80000000 | 18,   19,
    QMetaType::Void, QMetaType::QString,   21,
    QMetaType::Void, 0x80000000 | 23,   24,
    QMetaType::Void, 0x80000000 | 23, 0x80000000 | 18, QMetaType::QByteArray,   24,   26,   27,
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,   32,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 23,   37,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 23, 0x80000000 | 18, QMetaType::QByteArray,   24,   26,   27,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 42, QMetaType::QString,   26,   27,
    QMetaType::Void, 0x80000000 | 42, QMetaType::QString,   26,   27,
    QMetaType::Void, QMetaType::Bool, 0x80000000 | 42, 0x80000000 | 46,   45,   26,   47,
    QMetaType::Void, 0x80000000 | 42,   26,
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void, 0x80000000 | 42,   26,
    QMetaType::Void, QMetaType::Bool,   52,
    QMetaType::Void, QMetaType::Bool,   54,
    QMetaType::Void, 0x80000000 | 42,   56,
    QMetaType::Void, QMetaType::Bool, QMetaType::QString,   58,   59,
    QMetaType::Void, QMetaType::Bool,   58,
    QMetaType::Void, QMetaType::Int, QMetaType::Bool,   26,   61,
    QMetaType::Void, QMetaType::Int,   26,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   65,
    QMetaType::Void, QMetaType::Int,   67,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int,   69,    2,   70,
    QMetaType::Void, QMetaType::QPoint,   72,
    QMetaType::Void, QMetaType::QPoint,   72,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   83,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   90,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void TableBrowser::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TableBrowser *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->projectModified(); break;
        case 1: _t->selectionChanged((*reinterpret_cast< QModelIndex(*)>(_a[1]))); break;
        case 2: _t->selectionChangedByDoubleClick((*reinterpret_cast< QModelIndex(*)>(_a[1]))); break;
        case 3: _t->statusMessageRequested((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 4: _t->updatePlot((*reinterpret_cast< ExtendedTableWidget*(*)>(_a[1])),(*reinterpret_cast< SqliteTableModel*(*)>(_a[2])),(*reinterpret_cast< BrowseDataTableSettings*(*)>(_a[3])),(*reinterpret_cast< bool(*)>(_a[4]))); break;
        case 5: _t->createView((*reinterpret_cast< std::string(*)>(_a[1]))); break;
        case 6: _t->requestFileOpen((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 7: _t->currentTableChanged((*reinterpret_cast< sqlb::ObjectIdentifier(*)>(_a[1]))); break;
        case 8: _t->foreignKeyClicked((*reinterpret_cast< sqlb::ObjectIdentifier(*)>(_a[1])),(*reinterpret_cast< std::string(*)>(_a[2])),(*reinterpret_cast< QByteArray(*)>(_a[3]))); break;
        case 9: _t->dataAboutToBeEdited((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 10: _t->prepareForFilter(); break;
        case 11: _t->newDockRequested(); break;
        case 12: _t->setEnabled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 13: _t->refresh(); break;
        case 14: _t->clearFilters(); break;
        case 15: _t->reloadSettings(); break;
        case 16: _t->setCurrentTable((*reinterpret_cast< const sqlb::ObjectIdentifier(*)>(_a[1]))); break;
        case 17: _t->updateRecordsetLabel(); break;
        case 18: _t->jumpToRow((*reinterpret_cast< const sqlb::ObjectIdentifier(*)>(_a[1])),(*reinterpret_cast< std::string(*)>(_a[2])),(*reinterpret_cast< const QByteArray(*)>(_a[3]))); break;
        case 19: _t->clear(); break;
        case 20: _t->updateFilter((*reinterpret_cast< size_t(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 21: _t->addCondFormatFromFilter((*reinterpret_cast< size_t(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 22: _t->addCondFormat((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< size_t(*)>(_a[2])),(*reinterpret_cast< const CondFormat(*)>(_a[3]))); break;
        case 23: _t->clearAllCondFormats((*reinterpret_cast< size_t(*)>(_a[1]))); break;
        case 24: _t->clearRowIdFormats((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 25: _t->editCondFormats((*reinterpret_cast< size_t(*)>(_a[1]))); break;
        case 26: _t->enableEditing((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 27: _t->showRowidColumn((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 28: _t->freezeColumns((*reinterpret_cast< size_t(*)>(_a[1]))); break;
        case 29: _t->unlockViewEditing((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2]))); break;
        case 30: _t->unlockViewEditing((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 31: _t->hideColumns((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 32: _t->hideColumns((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 33: _t->hideColumns(); break;
        case 34: _t->showAllColumns(); break;
        case 35: _t->updateInsertDeleteRecordButton(); break;
        case 36: _t->duplicateRecord((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 37: _t->headerClicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 38: _t->updateColumnWidth((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 39: _t->showDataColumnPopupMenu((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        case 40: _t->showRecordPopupMenu((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        case 41: _t->addRecord(); break;
        case 42: _t->insertValues(); break;
        case 43: _t->deleteRecord(); break;
        case 44: _t->navigatePrevious(); break;
        case 45: _t->navigateNext(); break;
        case 46: _t->navigateBegin(); break;
        case 47: _t->navigateEnd(); break;
        case 48: _t->navigateGoto(); break;
        case 49: _t->selectTableLine((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 50: _t->clearSorting(); break;
        case 51: _t->editDisplayFormat(); break;
        case 52: _t->exportCsvFilteredTable(); break;
        case 53: _t->exportJsonFilteredTable(); break;
        case 54: _t->saveFilterAsView(); break;
        case 55: _t->setTableEncoding((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 56: _t->setTableEncoding(); break;
        case 57: _t->setDefaultTableEncoding(); break;
        case 58: _t->copyColumnName(); break;
        case 59: _t->fetchedData(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TableBrowser::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TableBrowser::projectModified)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (TableBrowser::*)(QModelIndex );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TableBrowser::selectionChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (TableBrowser::*)(QModelIndex );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TableBrowser::selectionChangedByDoubleClick)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (TableBrowser::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TableBrowser::statusMessageRequested)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (TableBrowser::*)(ExtendedTableWidget * , SqliteTableModel * , BrowseDataTableSettings * , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TableBrowser::updatePlot)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (TableBrowser::*)(std::string );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TableBrowser::createView)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (TableBrowser::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TableBrowser::requestFileOpen)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (TableBrowser::*)(sqlb::ObjectIdentifier );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TableBrowser::currentTableChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (TableBrowser::*)(sqlb::ObjectIdentifier , std::string , QByteArray );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TableBrowser::foreignKeyClicked)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (TableBrowser::*)(const QModelIndex & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TableBrowser::dataAboutToBeEdited)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (TableBrowser::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TableBrowser::prepareForFilter)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (TableBrowser::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TableBrowser::newDockRequested)) {
                *result = 11;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject TableBrowser::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_TableBrowser.data,
    qt_meta_data_TableBrowser,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TableBrowser::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TableBrowser::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TableBrowser.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int TableBrowser::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 60)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 60;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 60)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 60;
    }
    return _id;
}

// SIGNAL 0
void TableBrowser::projectModified()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void TableBrowser::selectionChanged(QModelIndex _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void TableBrowser::selectionChangedByDoubleClick(QModelIndex _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void TableBrowser::statusMessageRequested(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void TableBrowser::updatePlot(ExtendedTableWidget * _t1, SqliteTableModel * _t2, BrowseDataTableSettings * _t3, bool _t4)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void TableBrowser::createView(std::string _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void TableBrowser::requestFileOpen(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void TableBrowser::currentTableChanged(sqlb::ObjectIdentifier _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void TableBrowser::foreignKeyClicked(sqlb::ObjectIdentifier _t1, std::string _t2, QByteArray _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void TableBrowser::dataAboutToBeEdited(const QModelIndex & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void TableBrowser::prepareForFilter()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void TableBrowser::newDockRequested()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
