/****************************************************************************
** Meta object code from reading C++ file 'MainWindow.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../src/MainWindow.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'MainWindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_MainWindow_t {
    QByteArrayData data[141];
    char stringdata0[1950];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_MainWindow_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_MainWindow_t qt_meta_stringdata_MainWindow = {
    {
QT_MOC_LITERAL(0, 0, 10), // "MainWindow"
QT_MOC_LITERAL(1, 11, 8), // "fileOpen"
QT_MOC_LITERAL(2, 20, 0), // ""
QT_MOC_LITERAL(3, 21, 8), // "fileName"
QT_MOC_LITERAL(4, 30, 15), // "openFromProject"
QT_MOC_LITERAL(5, 46, 8), // "readOnly"
QT_MOC_LITERAL(6, 55, 6), // "logSql"
QT_MOC_LITERAL(7, 62, 3), // "sql"
QT_MOC_LITERAL(8, 66, 7), // "msgtype"
QT_MOC_LITERAL(9, 74, 7), // "dbState"
QT_MOC_LITERAL(10, 82, 5), // "dirty"
QT_MOC_LITERAL(11, 88, 7), // "refresh"
QT_MOC_LITERAL(12, 96, 21), // "switchToBrowseDataTab"
QT_MOC_LITERAL(13, 118, 22), // "sqlb::ObjectIdentifier"
QT_MOC_LITERAL(14, 141, 13), // "tableToBrowse"
QT_MOC_LITERAL(15, 155, 17), // "populateStructure"
QT_MOC_LITERAL(16, 173, 35), // "std::vector<sqlb::ObjectIdent..."
QT_MOC_LITERAL(17, 209, 10), // "old_tables"
QT_MOC_LITERAL(18, 220, 16), // "fileDetachDbTree"
QT_MOC_LITERAL(19, 237, 24), // "fileDetachTreeSchemaDock"
QT_MOC_LITERAL(20, 262, 26), // "fileDetachTreeViewSelected"
QT_MOC_LITERAL(21, 289, 10), // "QTreeView*"
QT_MOC_LITERAL(22, 300, 8), // "treeView"
QT_MOC_LITERAL(23, 309, 14), // "reloadSettings"
QT_MOC_LITERAL(24, 324, 10), // "closeFiles"
QT_MOC_LITERAL(25, 335, 14), // "importCSVfiles"
QT_MOC_LITERAL(26, 350, 20), // "std::vector<QString>"
QT_MOC_LITERAL(27, 371, 10), // "inputFiles"
QT_MOC_LITERAL(28, 382, 5), // "table"
QT_MOC_LITERAL(29, 388, 21), // "createTreeContextMenu"
QT_MOC_LITERAL(30, 410, 6), // "qPoint"
QT_MOC_LITERAL(31, 417, 27), // "createSchemaDockContextMenu"
QT_MOC_LITERAL(32, 445, 19), // "changeTreeSelection"
QT_MOC_LITERAL(33, 465, 7), // "fileNew"
QT_MOC_LITERAL(34, 473, 23), // "fileNewInMemoryDatabase"
QT_MOC_LITERAL(35, 497, 18), // "open_create_dialog"
QT_MOC_LITERAL(36, 516, 20), // "refreshTableBrowsers"
QT_MOC_LITERAL(37, 537, 3), // "all"
QT_MOC_LITERAL(38, 541, 9), // "fileClose"
QT_MOC_LITERAL(39, 551, 10), // "fileSaveAs"
QT_MOC_LITERAL(40, 562, 11), // "createTable"
QT_MOC_LITERAL(41, 574, 11), // "createIndex"
QT_MOC_LITERAL(42, 586, 7), // "compact"
QT_MOC_LITERAL(43, 594, 12), // "deleteObject"
QT_MOC_LITERAL(44, 607, 10), // "editObject"
QT_MOC_LITERAL(45, 618, 13), // "helpWhatsThis"
QT_MOC_LITERAL(46, 632, 9), // "helpAbout"
QT_MOC_LITERAL(47, 642, 16), // "updateRecordText"
QT_MOC_LITERAL(48, 659, 21), // "QPersistentModelIndex"
QT_MOC_LITERAL(49, 681, 3), // "idx"
QT_MOC_LITERAL(50, 685, 4), // "text"
QT_MOC_LITERAL(51, 690, 6), // "isBlob"
QT_MOC_LITERAL(52, 697, 12), // "evaluateText"
QT_MOC_LITERAL(53, 710, 11), // "std::string"
QT_MOC_LITERAL(54, 722, 14), // "toggleEditDock"
QT_MOC_LITERAL(55, 737, 7), // "visible"
QT_MOC_LITERAL(56, 745, 25), // "dataTableSelectionChanged"
QT_MOC_LITERAL(57, 771, 11), // "QModelIndex"
QT_MOC_LITERAL(58, 783, 5), // "index"
QT_MOC_LITERAL(59, 789, 16), // "doubleClickTable"
QT_MOC_LITERAL(60, 806, 12), // "executeQuery"
QT_MOC_LITERAL(61, 819, 18), // "importTableFromCSV"
QT_MOC_LITERAL(62, 838, 16), // "exportTableToCSV"
QT_MOC_LITERAL(63, 855, 17), // "exportTableToJson"
QT_MOC_LITERAL(64, 873, 8), // "fileSave"
QT_MOC_LITERAL(65, 882, 10), // "fileRevert"
QT_MOC_LITERAL(66, 893, 4), // "undo"
QT_MOC_LITERAL(67, 898, 19), // "exportDatabaseToSQL"
QT_MOC_LITERAL(68, 918, 21), // "importDatabaseFromSQL"
QT_MOC_LITERAL(69, 940, 14), // "openRecentFile"
QT_MOC_LITERAL(70, 955, 11), // "loadPragmas"
QT_MOC_LITERAL(71, 967, 14), // "updatePragmaUi"
QT_MOC_LITERAL(72, 982, 11), // "savePragmas"
QT_MOC_LITERAL(73, 994, 15), // "mainTabSelected"
QT_MOC_LITERAL(74, 1010, 8), // "tabindex"
QT_MOC_LITERAL(75, 1019, 11), // "openSqlFile"
QT_MOC_LITERAL(76, 1031, 8), // "filename"
QT_MOC_LITERAL(77, 1040, 11), // "saveSqlFile"
QT_MOC_LITERAL(78, 1052, 13), // "saveSqlFileAs"
QT_MOC_LITERAL(79, 1066, 19), // "saveSqlResultsAsCsv"
QT_MOC_LITERAL(80, 1086, 20), // "saveSqlResultsAsJson"
QT_MOC_LITERAL(81, 1107, 20), // "saveSqlResultsAsView"
QT_MOC_LITERAL(82, 1128, 13), // "loadExtension"
QT_MOC_LITERAL(83, 1142, 15), // "checkNewVersion"
QT_MOC_LITERAL(84, 1158, 13), // "versionstring"
QT_MOC_LITERAL(85, 1172, 3), // "url"
QT_MOC_LITERAL(86, 1176, 12), // "openLinkWiki"
QT_MOC_LITERAL(87, 1189, 17), // "openLinkBugReport"
QT_MOC_LITERAL(88, 1207, 22), // "openLinkFeatureRequest"
QT_MOC_LITERAL(89, 1230, 20), // "openLinkSqlCipherFaq"
QT_MOC_LITERAL(90, 1251, 15), // "openLinkWebsite"
QT_MOC_LITERAL(91, 1267, 21), // "openLinkDonatePatreon"
QT_MOC_LITERAL(92, 1289, 11), // "loadProject"
QT_MOC_LITERAL(93, 1301, 16), // "LoadAttempResult"
QT_MOC_LITERAL(94, 1318, 11), // "saveProject"
QT_MOC_LITERAL(95, 1330, 13), // "saveProjectAs"
QT_MOC_LITERAL(96, 1344, 10), // "fileAttach"
QT_MOC_LITERAL(97, 1355, 14), // "editEncryption"
QT_MOC_LITERAL(98, 1370, 26), // "copyCurrentCreateStatement"
QT_MOC_LITERAL(99, 1397, 16), // "fileOpenReadOnly"
QT_MOC_LITERAL(100, 1414, 16), // "requestCollation"
QT_MOC_LITERAL(101, 1431, 4), // "name"
QT_MOC_LITERAL(102, 1436, 8), // "eTextRep"
QT_MOC_LITERAL(103, 1445, 22), // "setFindFrameVisibility"
QT_MOC_LITERAL(104, 1468, 4), // "show"
QT_MOC_LITERAL(105, 1473, 21), // "openFindReplaceDialog"
QT_MOC_LITERAL(106, 1495, 21), // "toggleSqlBlockComment"
QT_MOC_LITERAL(107, 1517, 18), // "openSqlPrintDialog"
QT_MOC_LITERAL(108, 1536, 12), // "runSqlNewTab"
QT_MOC_LITERAL(109, 1549, 5), // "query"
QT_MOC_LITERAL(110, 1555, 5), // "title"
QT_MOC_LITERAL(111, 1561, 7), // "helpUrl"
QT_MOC_LITERAL(112, 1569, 7), // "autoRun"
QT_MOC_LITERAL(113, 1577, 16), // "printDbStructure"
QT_MOC_LITERAL(114, 1594, 24), // "updateDatabaseBusyStatus"
QT_MOC_LITERAL(115, 1619, 4), // "busy"
QT_MOC_LITERAL(116, 1624, 4), // "user"
QT_MOC_LITERAL(117, 1629, 15), // "openPreferences"
QT_MOC_LITERAL(118, 1645, 8), // "closeTab"
QT_MOC_LITERAL(119, 1654, 19), // "showStatusMessage5s"
QT_MOC_LITERAL(120, 1674, 7), // "message"
QT_MOC_LITERAL(121, 1682, 8), // "tabIndex"
QT_MOC_LITERAL(122, 1691, 7), // "saveAll"
QT_MOC_LITERAL(123, 1699, 13), // "openUrlOrFile"
QT_MOC_LITERAL(124, 1713, 9), // "urlString"
QT_MOC_LITERAL(125, 1723, 15), // "newRowCountsTab"
QT_MOC_LITERAL(126, 1739, 10), // "openSqlTab"
QT_MOC_LITERAL(127, 1750, 12), // "resetCounter"
QT_MOC_LITERAL(128, 1763, 11), // "closeSqlTab"
QT_MOC_LITERAL(129, 1775, 5), // "force"
QT_MOC_LITERAL(130, 1781, 9), // "askSaving"
QT_MOC_LITERAL(131, 1791, 12), // "changeSqlTab"
QT_MOC_LITERAL(132, 1804, 12), // "renameSqlTab"
QT_MOC_LITERAL(133, 1817, 24), // "showContextMenuSqlTabBar"
QT_MOC_LITERAL(134, 1842, 3), // "pos"
QT_MOC_LITERAL(135, 1846, 18), // "newTableBrowserTab"
QT_MOC_LITERAL(136, 1865, 17), // "TableBrowserDock*"
QT_MOC_LITERAL(137, 1883, 17), // "forceHideTitleBar"
QT_MOC_LITERAL(138, 1901, 21), // "tableBrowserTabClosed"
QT_MOC_LITERAL(139, 1923, 21), // "changeTableBrowserTab"
QT_MOC_LITERAL(140, 1945, 4) // "dock"

    },
    "MainWindow\0fileOpen\0\0fileName\0"
    "openFromProject\0readOnly\0logSql\0sql\0"
    "msgtype\0dbState\0dirty\0refresh\0"
    "switchToBrowseDataTab\0sqlb::ObjectIdentifier\0"
    "tableToBrowse\0populateStructure\0"
    "std::vector<sqlb::ObjectIdentifier>\0"
    "old_tables\0fileDetachDbTree\0"
    "fileDetachTreeSchemaDock\0"
    "fileDetachTreeViewSelected\0QTreeView*\0"
    "treeView\0reloadSettings\0closeFiles\0"
    "importCSVfiles\0std::vector<QString>\0"
    "inputFiles\0table\0createTreeContextMenu\0"
    "qPoint\0createSchemaDockContextMenu\0"
    "changeTreeSelection\0fileNew\0"
    "fileNewInMemoryDatabase\0open_create_dialog\0"
    "refreshTableBrowsers\0all\0fileClose\0"
    "fileSaveAs\0createTable\0createIndex\0"
    "compact\0deleteObject\0editObject\0"
    "helpWhatsThis\0helpAbout\0updateRecordText\0"
    "QPersistentModelIndex\0idx\0text\0isBlob\0"
    "evaluateText\0std::string\0toggleEditDock\0"
    "visible\0dataTableSelectionChanged\0"
    "QModelIndex\0index\0doubleClickTable\0"
    "executeQuery\0importTableFromCSV\0"
    "exportTableToCSV\0exportTableToJson\0"
    "fileSave\0fileRevert\0undo\0exportDatabaseToSQL\0"
    "importDatabaseFromSQL\0openRecentFile\0"
    "loadPragmas\0updatePragmaUi\0savePragmas\0"
    "mainTabSelected\0tabindex\0openSqlFile\0"
    "filename\0saveSqlFile\0saveSqlFileAs\0"
    "saveSqlResultsAsCsv\0saveSqlResultsAsJson\0"
    "saveSqlResultsAsView\0loadExtension\0"
    "checkNewVersion\0versionstring\0url\0"
    "openLinkWiki\0openLinkBugReport\0"
    "openLinkFeatureRequest\0openLinkSqlCipherFaq\0"
    "openLinkWebsite\0openLinkDonatePatreon\0"
    "loadProject\0LoadAttempResult\0saveProject\0"
    "saveProjectAs\0fileAttach\0editEncryption\0"
    "copyCurrentCreateStatement\0fileOpenReadOnly\0"
    "requestCollation\0name\0eTextRep\0"
    "setFindFrameVisibility\0show\0"
    "openFindReplaceDialog\0toggleSqlBlockComment\0"
    "openSqlPrintDialog\0runSqlNewTab\0query\0"
    "title\0helpUrl\0autoRun\0printDbStructure\0"
    "updateDatabaseBusyStatus\0busy\0user\0"
    "openPreferences\0closeTab\0showStatusMessage5s\0"
    "message\0tabIndex\0saveAll\0openUrlOrFile\0"
    "urlString\0newRowCountsTab\0openSqlTab\0"
    "resetCounter\0closeSqlTab\0force\0askSaving\0"
    "changeSqlTab\0renameSqlTab\0"
    "showContextMenuSqlTabBar\0pos\0"
    "newTableBrowserTab\0TableBrowserDock*\0"
    "forceHideTitleBar\0tableBrowserTabClosed\0"
    "changeTableBrowserTab\0dock"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_MainWindow[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
     109,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    3,  559,    2, 0x0a /* Public */,
       1,    2,  566,    2, 0x2a /* Public | MethodCloned */,
       1,    1,  571,    2, 0x2a /* Public | MethodCloned */,
       1,    0,  574,    2, 0x2a /* Public | MethodCloned */,
       6,    2,  575,    2, 0x0a /* Public */,
       9,    1,  580,    2, 0x0a /* Public */,
      11,    0,  583,    2, 0x0a /* Public */,
      12,    1,  584,    2, 0x0a /* Public */,
      12,    0,  587,    2, 0x2a /* Public | MethodCloned */,
      15,    1,  588,    2, 0x0a /* Public */,
      15,    0,  591,    2, 0x2a /* Public | MethodCloned */,
      18,    0,  592,    2, 0x0a /* Public */,
      19,    0,  593,    2, 0x0a /* Public */,
      20,    1,  594,    2, 0x0a /* Public */,
      23,    0,  597,    2, 0x0a /* Public */,
      24,    0,  598,    2, 0x0a /* Public */,
      25,    2,  599,    2, 0x0a /* Public */,
      25,    1,  604,    2, 0x2a /* Public | MethodCloned */,
      29,    1,  607,    2, 0x08 /* Private */,
      31,    1,  610,    2, 0x08 /* Private */,
      32,    0,  613,    2, 0x08 /* Private */,
      33,    0,  614,    2, 0x08 /* Private */,
      34,    1,  615,    2, 0x08 /* Private */,
      34,    0,  618,    2, 0x28 /* Private | MethodCloned */,
      36,    1,  619,    2, 0x08 /* Private */,
      36,    0,  622,    2, 0x28 /* Private | MethodCloned */,
      38,    0,  623,    2, 0x08 /* Private */,
      39,    0,  624,    2, 0x08 /* Private */,
      40,    0,  625,    2, 0x08 /* Private */,
      41,    0,  626,    2, 0x08 /* Private */,
      42,    0,  627,    2, 0x08 /* Private */,
      43,    0,  628,    2, 0x08 /* Private */,
      44,    0,  629,    2, 0x08 /* Private */,
      45,    0,  630,    2, 0x08 /* Private */,
      46,    0,  631,    2, 0x08 /* Private */,
      47,    3,  632,    2, 0x08 /* Private */,
      52,    2,  639,    2, 0x08 /* Private */,
      54,    1,  644,    2, 0x08 /* Private */,
      56,    1,  647,    2, 0x08 /* Private */,
      59,    1,  650,    2, 0x08 /* Private */,
      60,    0,  653,    2, 0x08 /* Private */,
      61,    0,  654,    2, 0x08 /* Private */,
      62,    0,  655,    2, 0x08 /* Private */,
      63,    0,  656,    2, 0x08 /* Private */,
      64,    0,  657,    2, 0x08 /* Private */,
      65,    0,  658,    2, 0x08 /* Private */,
      66,    0,  659,    2, 0x08 /* Private */,
      67,    0,  660,    2, 0x08 /* Private */,
      68,    0,  661,    2, 0x08 /* Private */,
      69,    0,  662,    2, 0x08 /* Private */,
      70,    0,  663,    2, 0x08 /* Private */,
      71,    0,  664,    2, 0x08 /* Private */,
      72,    0,  665,    2, 0x08 /* Private */,
      73,    1,  666,    2, 0x08 /* Private */,
      75,    2,  669,    2, 0x08 /* Private */,
      75,    0,  674,    2, 0x08 /* Private */,
      77,    0,  675,    2, 0x08 /* Private */,
      78,    0,  676,    2, 0x08 /* Private */,
      79,    0,  677,    2, 0x08 /* Private */,
      80,    0,  678,    2, 0x08 /* Private */,
      81,    0,  679,    2, 0x08 /* Private */,
      82,    0,  680,    2, 0x08 /* Private */,
      83,    2,  681,    2, 0x08 /* Private */,
      86,    0,  686,    2, 0x08 /* Private */,
      87,    0,  687,    2, 0x08 /* Private */,
      88,    0,  688,    2, 0x08 /* Private */,
      89,    0,  689,    2, 0x08 /* Private */,
      90,    0,  690,    2, 0x08 /* Private */,
      91,    0,  691,    2, 0x08 /* Private */,
      92,    2,  692,    2, 0x08 /* Private */,
      92,    1,  697,    2, 0x28 /* Private | MethodCloned */,
      92,    0,  700,    2, 0x28 /* Private | MethodCloned */,
      94,    0,  701,    2, 0x08 /* Private */,
      95,    0,  702,    2, 0x08 /* Private */,
      96,    1,  703,    2, 0x08 /* Private */,
      96,    0,  706,    2, 0x28 /* Private | MethodCloned */,
      97,    0,  707,    2, 0x08 /* Private */,
      98,    0,  708,    2, 0x08 /* Private */,
      99,    0,  709,    2, 0x08 /* Private */,
     100,    2,  710,    2, 0x08 /* Private */,
     103,    1,  715,    2, 0x08 /* Private */,
     105,    0,  718,    2, 0x08 /* Private */,
     106,    0,  719,    2, 0x08 /* Private */,
     107,    0,  720,    2, 0x08 /* Private */,
     108,    4,  721,    2, 0x08 /* Private */,
     108,    3,  730,    2, 0x28 /* Private | MethodCloned */,
     108,    2,  737,    2, 0x28 /* Private | MethodCloned */,
     113,    0,  742,    2, 0x08 /* Private */,
     114,    2,  743,    2, 0x08 /* Private */,
     117,    0,  748,    2, 0x08 /* Private */,
     118,    1,  749,    2, 0x08 /* Private */,
     119,    1,  752,    2, 0x08 /* Private */,
      77,    1,  755,    2, 0x08 /* Private */,
     122,    0,  758,    2, 0x08 /* Private */,
     123,    1,  759,    2, 0x08 /* Private */,
     125,    0,  762,    2, 0x08 /* Private */,
     126,    1,  763,    2, 0x08 /* Private */,
     126,    0,  766,    2, 0x28 /* Private | MethodCloned */,
     128,    3,  767,    2, 0x08 /* Private */,
     128,    2,  774,    2, 0x28 /* Private | MethodCloned */,
     128,    1,  779,    2, 0x28 /* Private | MethodCloned */,
     131,    1,  782,    2, 0x08 /* Private */,
     132,    1,  785,    2, 0x08 /* Private */,
     133,    1,  788,    2, 0x08 /* Private */,
     135,    2,  791,    2, 0x08 /* Private */,
     135,    1,  796,    2, 0x28 /* Private | MethodCloned */,
     135,    0,  799,    2, 0x28 /* Private | MethodCloned */,
     138,    0,  800,    2, 0x08 /* Private */,
     139,    1,  801,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Bool, QMetaType::QString, QMetaType::Bool, QMetaType::Bool,    3,    4,    5,
    QMetaType::Bool, QMetaType::QString, QMetaType::Bool,    3,    4,
    QMetaType::Bool, QMetaType::QString,    3,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::QString, QMetaType::Int,    7,    8,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 13,   14,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 16,   17,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 21,   22,
    QMetaType::Void,
    QMetaType::Bool,
    QMetaType::Void, 0x80000000 | 26, QMetaType::QString,   27,   28,
    QMetaType::Void, 0x80000000 | 26,   27,
    QMetaType::Void, QMetaType::QPoint,   30,
    QMetaType::Void, QMetaType::QPoint,   30,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   35,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   37,
    QMetaType::Void,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 48, QMetaType::QByteArray, QMetaType::Bool,   49,   50,   51,
    QMetaType::Void, 0x80000000 | 48, 0x80000000 | 53,   49,   50,
    QMetaType::Void, QMetaType::Bool,   55,
    QMetaType::Void, 0x80000000 | 57,   58,
    QMetaType::Void, 0x80000000 | 57,   58,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   74,
    QMetaType::Void, QMetaType::Int, QMetaType::QString,   74,   76,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   84,   85,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    0x80000000 | 93, QMetaType::QString, QMetaType::Bool,   76,    5,
    0x80000000 | 93, QMetaType::QString,   76,
    0x80000000 | 93,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::Int,  101,  102,
    QMetaType::Void, QMetaType::Bool,  104,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::QString, QMetaType::Bool,  109,  110,  111,  112,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::QString,  109,  110,  111,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,  109,  110,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool, QMetaType::QString,  115,  116,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   58,
    QMetaType::Void, QMetaType::QString,  120,
    QMetaType::Void, QMetaType::Int,  121,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,  124,
    QMetaType::Void,
    QMetaType::Int, QMetaType::Bool,  127,
    QMetaType::Int,
    QMetaType::Void, QMetaType::Int, QMetaType::Bool, QMetaType::Bool,   58,  129,  130,
    QMetaType::Void, QMetaType::Int, QMetaType::Bool,   58,  129,
    QMetaType::Void, QMetaType::Int,   58,
    QMetaType::Void, QMetaType::Int,   58,
    QMetaType::Void, QMetaType::Int,   58,
    QMetaType::Void, QMetaType::QPoint,  134,
    0x80000000 | 136, 0x80000000 | 13, QMetaType::Bool,   14,  137,
    0x80000000 | 136, 0x80000000 | 13,   14,
    0x80000000 | 136,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 136,  140,

       0        // eod
};

void MainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MainWindow *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { bool _r = _t->fileOpen((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 1: { bool _r = _t->fileOpen((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 2: { bool _r = _t->fileOpen((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 3: { bool _r = _t->fileOpen();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 4: _t->logSql((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 5: _t->dbState((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->refresh(); break;
        case 7: _t->switchToBrowseDataTab((*reinterpret_cast< sqlb::ObjectIdentifier(*)>(_a[1]))); break;
        case 8: _t->switchToBrowseDataTab(); break;
        case 9: _t->populateStructure((*reinterpret_cast< const std::vector<sqlb::ObjectIdentifier>(*)>(_a[1]))); break;
        case 10: _t->populateStructure(); break;
        case 11: _t->fileDetachDbTree(); break;
        case 12: _t->fileDetachTreeSchemaDock(); break;
        case 13: _t->fileDetachTreeViewSelected((*reinterpret_cast< QTreeView*(*)>(_a[1]))); break;
        case 14: _t->reloadSettings(); break;
        case 15: { bool _r = _t->closeFiles();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 16: _t->importCSVfiles((*reinterpret_cast< const std::vector<QString>(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 17: _t->importCSVfiles((*reinterpret_cast< const std::vector<QString>(*)>(_a[1]))); break;
        case 18: _t->createTreeContextMenu((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        case 19: _t->createSchemaDockContextMenu((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        case 20: _t->changeTreeSelection(); break;
        case 21: _t->fileNew(); break;
        case 22: _t->fileNewInMemoryDatabase((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 23: _t->fileNewInMemoryDatabase(); break;
        case 24: _t->refreshTableBrowsers((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 25: _t->refreshTableBrowsers(); break;
        case 26: { bool _r = _t->fileClose();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 27: { bool _r = _t->fileSaveAs();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 28: _t->createTable(); break;
        case 29: _t->createIndex(); break;
        case 30: _t->compact(); break;
        case 31: _t->deleteObject(); break;
        case 32: _t->editObject(); break;
        case 33: _t->helpWhatsThis(); break;
        case 34: _t->helpAbout(); break;
        case 35: _t->updateRecordText((*reinterpret_cast< const QPersistentModelIndex(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 36: _t->evaluateText((*reinterpret_cast< const QPersistentModelIndex(*)>(_a[1])),(*reinterpret_cast< const std::string(*)>(_a[2]))); break;
        case 37: _t->toggleEditDock((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 38: _t->dataTableSelectionChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 39: _t->doubleClickTable((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 40: _t->executeQuery(); break;
        case 41: _t->importTableFromCSV(); break;
        case 42: _t->exportTableToCSV(); break;
        case 43: _t->exportTableToJson(); break;
        case 44: _t->fileSave(); break;
        case 45: _t->fileRevert(); break;
        case 46: _t->undo(); break;
        case 47: _t->exportDatabaseToSQL(); break;
        case 48: _t->importDatabaseFromSQL(); break;
        case 49: _t->openRecentFile(); break;
        case 50: _t->loadPragmas(); break;
        case 51: _t->updatePragmaUi(); break;
        case 52: _t->savePragmas(); break;
        case 53: _t->mainTabSelected((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 54: _t->openSqlFile((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2]))); break;
        case 55: _t->openSqlFile(); break;
        case 56: _t->saveSqlFile(); break;
        case 57: _t->saveSqlFileAs(); break;
        case 58: _t->saveSqlResultsAsCsv(); break;
        case 59: _t->saveSqlResultsAsJson(); break;
        case 60: _t->saveSqlResultsAsView(); break;
        case 61: _t->loadExtension(); break;
        case 62: _t->checkNewVersion((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 63: _t->openLinkWiki(); break;
        case 64: _t->openLinkBugReport(); break;
        case 65: _t->openLinkFeatureRequest(); break;
        case 66: _t->openLinkSqlCipherFaq(); break;
        case 67: _t->openLinkWebsite(); break;
        case 68: _t->openLinkDonatePatreon(); break;
        case 69: { LoadAttempResult _r = _t->loadProject((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< LoadAttempResult*>(_a[0]) = std::move(_r); }  break;
        case 70: { LoadAttempResult _r = _t->loadProject((*reinterpret_cast< QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< LoadAttempResult*>(_a[0]) = std::move(_r); }  break;
        case 71: { LoadAttempResult _r = _t->loadProject();
            if (_a[0]) *reinterpret_cast< LoadAttempResult*>(_a[0]) = std::move(_r); }  break;
        case 72: _t->saveProject(); break;
        case 73: _t->saveProjectAs(); break;
        case 74: _t->fileAttach((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 75: _t->fileAttach(); break;
        case 76: _t->editEncryption(); break;
        case 77: _t->copyCurrentCreateStatement(); break;
        case 78: _t->fileOpenReadOnly(); break;
        case 79: _t->requestCollation((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 80: _t->setFindFrameVisibility((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 81: _t->openFindReplaceDialog(); break;
        case 82: _t->toggleSqlBlockComment(); break;
        case 83: _t->openSqlPrintDialog(); break;
        case 84: _t->runSqlNewTab((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< const bool(*)>(_a[4]))); break;
        case 85: _t->runSqlNewTab((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 86: _t->runSqlNewTab((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 87: _t->printDbStructure(); break;
        case 88: _t->updateDatabaseBusyStatus((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 89: _t->openPreferences(); break;
        case 90: _t->closeTab((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 91: _t->showStatusMessage5s((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 92: _t->saveSqlFile((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 93: _t->saveAll(); break;
        case 94: _t->openUrlOrFile((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 95: _t->newRowCountsTab(); break;
        case 96: { int _r = _t->openSqlTab((*reinterpret_cast< bool(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 97: { int _r = _t->openSqlTab();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 98: _t->closeSqlTab((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 99: _t->closeSqlTab((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 100: _t->closeSqlTab((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 101: _t->changeSqlTab((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 102: _t->renameSqlTab((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 103: _t->showContextMenuSqlTabBar((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        case 104: { TableBrowserDock* _r = _t->newTableBrowserTab((*reinterpret_cast< const sqlb::ObjectIdentifier(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< TableBrowserDock**>(_a[0]) = std::move(_r); }  break;
        case 105: { TableBrowserDock* _r = _t->newTableBrowserTab((*reinterpret_cast< const sqlb::ObjectIdentifier(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< TableBrowserDock**>(_a[0]) = std::move(_r); }  break;
        case 106: { TableBrowserDock* _r = _t->newTableBrowserTab();
            if (_a[0]) *reinterpret_cast< TableBrowserDock**>(_a[0]) = std::move(_r); }  break;
        case 107: _t->tableBrowserTabClosed(); break;
        case 108: _t->changeTableBrowserTab((*reinterpret_cast< TableBrowserDock*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QTreeView* >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject MainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_MainWindow.data,
    qt_meta_data_MainWindow,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *MainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_MainWindow.stringdata0))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int MainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 109)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 109;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 109)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 109;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
