/****************************************************************************
** Meta object code from reading C++ file 'ExtendedScintilla.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../src/ExtendedScintilla.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ExtendedScintilla.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ExtendedScintilla_t {
    QByteArrayData data[17];
    char stringdata0[228];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ExtendedScintilla_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ExtendedScintilla_t qt_meta_stringdata_ExtendedScintilla = {
    {
QT_MOC_LITERAL(0, 0, 17), // "ExtendedScintilla"
QT_MOC_LITERAL(1, 18, 14), // "reloadKeywords"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 14), // "reloadSettings"
QT_MOC_LITERAL(4, 49, 20), // "clearErrorIndicators"
QT_MOC_LITERAL(5, 70, 17), // "setErrorIndicator"
QT_MOC_LITERAL(6, 88, 7), // "fromRow"
QT_MOC_LITERAL(7, 96, 9), // "fromIndex"
QT_MOC_LITERAL(8, 106, 5), // "toRow"
QT_MOC_LITERAL(9, 112, 7), // "toIndex"
QT_MOC_LITERAL(10, 120, 8), // "position"
QT_MOC_LITERAL(11, 129, 21), // "openFindReplaceDialog"
QT_MOC_LITERAL(12, 151, 14), // "openFindDialog"
QT_MOC_LITERAL(13, 166, 15), // "openPrintDialog"
QT_MOC_LITERAL(14, 182, 25), // "updateLineNumberAreaWidth"
QT_MOC_LITERAL(15, 208, 15), // "showContextMenu"
QT_MOC_LITERAL(16, 224, 3) // "pos"

    },
    "ExtendedScintilla\0reloadKeywords\0\0"
    "reloadSettings\0clearErrorIndicators\0"
    "setErrorIndicator\0fromRow\0fromIndex\0"
    "toRow\0toIndex\0position\0openFindReplaceDialog\0"
    "openFindDialog\0openPrintDialog\0"
    "updateLineNumberAreaWidth\0showContextMenu\0"
    "pos"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ExtendedScintilla[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   64,    2, 0x0a /* Public */,
       3,    0,   65,    2, 0x0a /* Public */,
       4,    0,   66,    2, 0x0a /* Public */,
       5,    4,   67,    2, 0x0a /* Public */,
       5,    1,   76,    2, 0x0a /* Public */,
      11,    0,   79,    2, 0x0a /* Public */,
      12,    0,   80,    2, 0x0a /* Public */,
      13,    0,   81,    2, 0x0a /* Public */,
      14,    0,   82,    2, 0x08 /* Private */,
      15,    1,   83,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int, QMetaType::Int,    6,    7,    8,    9,
    QMetaType::Void, QMetaType::Int,   10,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QPoint,   16,

       0        // eod
};

void ExtendedScintilla::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ExtendedScintilla *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->reloadKeywords(); break;
        case 1: _t->reloadSettings(); break;
        case 2: _t->clearErrorIndicators(); break;
        case 3: _t->setErrorIndicator((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4]))); break;
        case 4: _t->setErrorIndicator((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->openFindReplaceDialog(); break;
        case 6: _t->openFindDialog(); break;
        case 7: _t->openPrintDialog(); break;
        case 8: _t->updateLineNumberAreaWidth(); break;
        case 9: _t->showContextMenu((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ExtendedScintilla::staticMetaObject = { {
    QMetaObject::SuperData::link<QsciScintilla::staticMetaObject>(),
    qt_meta_stringdata_ExtendedScintilla.data,
    qt_meta_data_ExtendedScintilla,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ExtendedScintilla::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ExtendedScintilla::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ExtendedScintilla.stringdata0))
        return static_cast<void*>(this);
    return QsciScintilla::qt_metacast(_clname);
}

int ExtendedScintilla::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QsciScintilla::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
