/********************************************************************************
** Form generated from reading UI file 'AboutDialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ABOUTDIALOG_H
#define UI_ABOUTDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_AboutDialog
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QLabel *label_version;
    QLabel *label_imagelogo;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QVBoxLayout *verticalLayout_2;
    QLabel *label;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *AboutDialog)
    {
        if (AboutDialog->objectName().isEmpty())
            AboutDialog->setObjectName(QString::fromUtf8("AboutDialog"));
        AboutDialog->resize(653, 502);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/icons/appicon"), QSize(), QIcon::Normal, QIcon::Off);
        AboutDialog->setWindowIcon(icon);
        AboutDialog->setSizeGripEnabled(false);
        verticalLayout = new QVBoxLayout(AboutDialog);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        label_version = new QLabel(AboutDialog);
        label_version->setObjectName(QString::fromUtf8("label_version"));
        label_version->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        label_version->setMargin(5);
        label_version->setTextInteractionFlags(Qt::TextSelectableByMouse);

        horizontalLayout->addWidget(label_version);

        label_imagelogo = new QLabel(AboutDialog);
        label_imagelogo->setObjectName(QString::fromUtf8("label_imagelogo"));
        QSizePolicy sizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_imagelogo->sizePolicy().hasHeightForWidth());
        label_imagelogo->setSizePolicy(sizePolicy);
        label_imagelogo->setMaximumSize(QSize(96, 96));
        label_imagelogo->setPixmap(QPixmap(QString::fromUtf8(":/icons/appicon")));
        label_imagelogo->setScaledContents(true);

        horizontalLayout->addWidget(label_imagelogo);


        verticalLayout->addLayout(horizontalLayout);

        scrollArea = new QScrollArea(AboutDialog);
        scrollArea->setObjectName(QString::fromUtf8("scrollArea"));
        scrollArea->setFrameShape(QFrame::NoFrame);
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName(QString::fromUtf8("scrollAreaWidgetContents"));
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 635, 350));
        verticalLayout_2 = new QVBoxLayout(scrollAreaWidgetContents);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        label = new QLabel(scrollAreaWidgetContents);
        label->setObjectName(QString::fromUtf8("label"));
        label->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        label->setWordWrap(true);
        label->setOpenExternalLinks(true);

        verticalLayout_2->addWidget(label);

        scrollArea->setWidget(scrollAreaWidgetContents);

        verticalLayout->addWidget(scrollArea);

        buttonBox = new QDialogButtonBox(AboutDialog);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Close);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(AboutDialog);
        QObject::connect(buttonBox, SIGNAL(accepted()), AboutDialog, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), AboutDialog, SLOT(reject()));

        QMetaObject::connectSlotsByName(AboutDialog);
    } // setupUi

    void retranslateUi(QDialog *AboutDialog)
    {
        AboutDialog->setWindowTitle(QCoreApplication::translate("AboutDialog", "About DB Browser for SQLite", nullptr));
        label_version->setText(QCoreApplication::translate("AboutDialog", "Version", nullptr));
        label_imagelogo->setText(QString());
        label->setText(QCoreApplication::translate("AboutDialog", "<html><head/><body><p>DB Browser for SQLite is an open source, freeware visual tool used to create, design and edit SQLite database files.</p><p>It is bi-licensed under the Mozilla Public License Version 2, as well as the GNU General Public License Version 3 or later. You can modify or redistribute it under the conditions of these licenses.</p><p>See <a href=\"https://www.gnu.org/licenses/gpl.html\"><span style=\" text-decoration: underline; color:#0000ff;\">https://www.gnu.org/licenses/gpl.html</span></a> and <a href=\"https://www.mozilla.org/MPL/2.0/index.txt\"><span style=\" text-decoration: underline; color:#0000ff;\">https://www.mozilla.org/MPL/2.0/index.txt</span></a> for details.</p><p>For more information on this program please visit our website at: <a href=\"https://sqlitebrowser.org\"><span style=\" text-decoration: underline; color:#0000ff;\">https://sqlitebrowser.org</span></a></p><p><span style=\" font-size:small;\">This software uses the GPL/LGPL Qt Toolkit from </span><a href=\"https://qt-projec"
                        "t.org/\"><span style=\" font-size:small; text-decoration: underline; color:#0000ff;\">https://qt-project.org/</span></a><span style=\" font-size:small;\"><br/>See </span><a href=\"https://doc.qt.io/qt-5/licensing.html\"><span style=\" font-size:small; text-decoration: underline; color:#0000ff;\">https://doc.qt.io/qt-5/licensing.html</span></a><span style=\" font-size:small;\"> for licensing terms and information.</span></p><p><span style=\" font-size:8pt;\">We use the nalgeon/sqlean library for SQLite extensions support.<br/>This library is licensed under the MIT license, see the following for more information:<br/></span><a href=\"https://github.com/nalgeon/sqlean\"><span style=\" font-size:8pt; text-decoration: underline; color:#0000ff;\">https://github.com/nalgeon/sqlean</span></a></p><p><span style=\" font-size:small;\">It also uses the Pastel SVG icon set by Michael Buckley under a Creative Commons Attribution Share Alike 4.0 license.<br/>See </span><a href=\"https://codefisher.org/pastel-svg/\"><span sty"
                        "le=\" font-size:small; text-decoration: underline; color:#0000ff;\">https://codefisher.org/pastel-svg/</span></a><span style=\" font-size:small;\"> for details.</span></p></body></html>", nullptr));
    } // retranslateUi

};

namespace Ui {
    class AboutDialog: public Ui_AboutDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ABOUTDIALOG_H
