/****************************************************************************
** Meta object code from reading C++ file 'SqlExecutionArea.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../src/SqlExecutionArea.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'SqlExecutionArea.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_SqlExecutionArea_t {
    QByteArrayData data[20];
    char stringdata0[242];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SqlExecutionArea_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SqlExecutionArea_t qt_meta_stringdata_SqlExecutionArea = {
    {
QT_MOC_LITERAL(0, 0, 16), // "SqlExecutionArea"
QT_MOC_LITERAL(1, 17, 26), // "findFrameVisibilityChanged"
QT_MOC_LITERAL(2, 44, 0), // ""
QT_MOC_LITERAL(3, 45, 7), // "visible"
QT_MOC_LITERAL(4, 53, 15), // "finishExecution"
QT_MOC_LITERAL(5, 69, 6), // "result"
QT_MOC_LITERAL(6, 76, 2), // "ok"
QT_MOC_LITERAL(7, 79, 9), // "saveAsCsv"
QT_MOC_LITERAL(8, 89, 10), // "saveAsJson"
QT_MOC_LITERAL(9, 100, 14), // "reloadSettings"
QT_MOC_LITERAL(10, 115, 11), // "fetchedData"
QT_MOC_LITERAL(11, 127, 22), // "setFindFrameVisibility"
QT_MOC_LITERAL(12, 150, 4), // "show"
QT_MOC_LITERAL(13, 155, 12), // "findPrevious"
QT_MOC_LITERAL(14, 168, 8), // "findNext"
QT_MOC_LITERAL(15, 177, 24), // "findLineEdit_textChanged"
QT_MOC_LITERAL(16, 202, 4), // "text"
QT_MOC_LITERAL(17, 207, 13), // "hideFindFrame"
QT_MOC_LITERAL(18, 221, 11), // "fileChanged"
QT_MOC_LITERAL(19, 233, 8) // "filename"

    },
    "SqlExecutionArea\0findFrameVisibilityChanged\0"
    "\0visible\0finishExecution\0result\0ok\0"
    "saveAsCsv\0saveAsJson\0reloadSettings\0"
    "fetchedData\0setFindFrameVisibility\0"
    "show\0findPrevious\0findNext\0"
    "findLineEdit_textChanged\0text\0"
    "hideFindFrame\0fileChanged\0filename"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SqlExecutionArea[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   74,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    2,   77,    2, 0x0a /* Public */,
       7,    0,   82,    2, 0x0a /* Public */,
       8,    0,   83,    2, 0x0a /* Public */,
       9,    0,   84,    2, 0x0a /* Public */,
      10,    0,   85,    2, 0x0a /* Public */,
      11,    1,   86,    2, 0x0a /* Public */,
      13,    0,   89,    2, 0x08 /* Private */,
      14,    0,   90,    2, 0x08 /* Private */,
      15,    1,   91,    2, 0x08 /* Private */,
      17,    0,   94,    2, 0x08 /* Private */,
      18,    1,   95,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    3,

 // slots: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::Bool,    5,    6,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   12,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   16,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   19,

       0        // eod
};

void SqlExecutionArea::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SqlExecutionArea *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->findFrameVisibilityChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->finishExecution((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const bool(*)>(_a[2]))); break;
        case 2: _t->saveAsCsv(); break;
        case 3: _t->saveAsJson(); break;
        case 4: _t->reloadSettings(); break;
        case 5: _t->fetchedData(); break;
        case 6: _t->setFindFrameVisibility((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->findPrevious(); break;
        case 8: _t->findNext(); break;
        case 9: _t->findLineEdit_textChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 10: _t->hideFindFrame(); break;
        case 11: _t->fileChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SqlExecutionArea::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SqlExecutionArea::findFrameVisibilityChanged)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject SqlExecutionArea::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_SqlExecutionArea.data,
    qt_meta_data_SqlExecutionArea,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SqlExecutionArea::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SqlExecutionArea::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SqlExecutionArea.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int SqlExecutionArea::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void SqlExecutionArea::findFrameVisibilityChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
